/*
 * @GXsection.java Version 1.1 02/28/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

import java.net.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;
import javax.swing.event.*;

import cmn.cmnStruct;

import iqstrat.iqstratStruct;
import iqstrat.iqstratHeadersListStruct;

//import gemini.gui.geminiLoginPanel;
import gemini.gui.geminiLoadLAS3Panel;

import horizon.strat.stratListStruct;
import horizon.strat.io.ReadStratUnitsXMLFile;

import horizon.bio.bioStratListStruct;
import horizon.bio.io.ReadBioStratUnitsXMLFile;

import lith.lithology.lithologySymbolsStruct;
import lith.lithology.io.ReadLithologySymbolsXMLFile;

import mask.maskSymbolsListStruct;
import mask.io.ReadMaskSymbolsXMLFile;

import parse.io.ReadParseSedStructXMLFile;
import parse.io.ReadParseFossilXMLFile;
import parse.io.ReadParseColorXMLFile;
import parse.io.ReadParsePHIXMLFile;
import parse.io.ReadRockColumnSymbolsXMLFile;

import rock.color.rockColorChartListStruct;
import rock.color.io.ReadColorChartXML;

/** Class GXsection
 *  <p> This Class is the starting point for the Type Log Web App.
 *
 *  @version 1.1 02/28/2013
 *  @author  John R. Victorine
 */

public class GXsection extends Applet
{
  // Runtime object

  private Runtime r = Runtime.getRuntime();

  // Web Application is runing Stand Alone Indicator

  private boolean isStandalone = false;

  private String  sLIST            = new String("0"); // LAS KID
  private String  sLIST_Standalone = new String("0");

  // LAS Viewer Global Variables

  private iqstratStruct stStruct = new iqstratStruct();
  private cmnStruct     stCMN    = new cmnStruct();
  private String        sPath    = new String(".");

  /** getParameter()
   *  <p> This method wil get a parameter value
   *  @param key = The Name of the Parameter
   *  @param def = The Object Type of the Parameter
   *  @return Parameter
   */

  public String getParameter(String key, String def)
  {
    return (isStandalone ? System.getProperty(key, def) :
            (getParameter(key) != null ? getParameter(key) : def));
  }

  /** Constructor GXsection()
   *  <p> This is the Constructor for this class.
   */

  public GXsection() {  }

  /** Method init()
   *  <p> This Method will retrieve the data values from the HTML or the main()
   *      arguments and run the Type Log Viewer Module.
   */

  public void init()
  {
    try
    {
      sLIST = this.getParameter("sLIST", sLIST_Standalone);
      if (sLIST.equals("undefined"))
        sLIST = new String("0");
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }

    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will launch the Field LAS Catalog.
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
//	geminiLoginPanel panel  = null;
	geminiLoadLAS3Panel panel = null;

    // initialize Global LAS Data Structure

    stCMN.bStandalone   = isStandalone;
//    stStruct.depthStart = 0.0;
//    stStruct.depthEnd   = 2000.0;

    // Retrieve KGS Image Icon for printing

    if (!isStandalone)
    {
      stCMN.urlDirCodeBase = getCodeBase();
      stCMN.appletContext  = getAppletContext();
      stCMN.pIcon          = getImage(getCodeBase(), cmn.cmnStruct.ICON);
      stCMN.pIcon2         = getImage(getCodeBase(), cmn.cmnStruct.ICON2);
    }
    else
    {
      URL url = null;

      stCMN.urlDirCodeBase =  getClass().getClassLoader().getResource(".");

      ClassLoader cl = this.getClass().getClassLoader();
      sPath = cl.getResource("images").toString();
      stCMN.sImagePath = new String(sPath);

      try
      {
        url = new URL(sPath+"\\kgsSm.png");
        stCMN.pIcon = Toolkit.getDefaultToolkit().getImage(url);
        url = new URL(sPath+"\\kgsSm2.png");
        stCMN.pIcon2 = Toolkit.getDefaultToolkit().getImage(url);
      }
      catch (java.net.MalformedURLException e)
      {
        e.printStackTrace();
        String sMessage = new String("GXsection(): \n" +
                              e.getMessage() + "\n" +
                              "Couldn't create image: " + sPath+"\\kgsSm.gif");
        JOptionPane.showMessageDialog((Component) null,
                                      sMessage,
                                      "ERROR",
                                      JOptionPane.ERROR_MESSAGE);
      }
    }

    // Get the Path of the XML Files stored in JAR File.

    ClassLoader cl = this.getClass().getClassLoader();
//    sPath = cl.getResource("data").toString();
//    stCMN.sPath = new String(sPath);

    sPath = cl.getResource("images").toString();
    stCMN.sImagePath = new String(sPath);

    stCMN.cl = cl;

    stStruct.stCMN = stCMN;

    // Read in Lookup Data

    readICSUnits();                // Read in the ISC Stratigraphic Units
    readKGSUnits();                // Read Kansas Stratigraphic Units

    readLithologySymbols();        // Read the Lithology Symbols Data

    readPorositySymbols();         // Read the Porosity Types Symbols Data
    readFossilSymbols();           // Read the Fossil Symbols Data
    readSedStructureSymbols();     // Read the Sedimentary Structure Symbols Data

    readParsePorosityThesaurus();  // Read the Porosity Types Thesaurus XML File
    readParseFossilsThesaurus();   // Read the Fossil Thesaurus XML FIle
    readParseSedStructSymbols();   // Read the Sedimentary Structure Thesaurus
    readParseColors();             // Read the Colors XML Thesaurus File

    readRockColumn();              // Read Rock Column Lithology XML File

    readBioStratigraphicUnits();   // Read Fossil by their Scientific Names
    readKSBioStratigraphicUnits(); // Read Kansas Fossils by their Scientific Names

    readRockColorChart();          // Read Rock Color Chart
    readSoilColorChart();          // Read Soil Color Chart

System.out.println(sLIST);

    // Display the LAS Viewer Start

    if (!isStandalone)
    {
//      panel = new geminiLoginPanel( null, stStruct );
      panel = new geminiLoadLAS3Panel( null, stStruct, sLIST );
      this.setLayout(new BorderLayout());
      this.add(panel, BorderLayout.CENTER);
    }
    else
    {
      JFrame frame = new JFrame();

      frame.setTitle("SELECT");
      panel = new geminiLoadLAS3Panel( frame, stStruct, sLIST );
//      panel = new geminiLoginPanel( frame, stStruct );
      frame.getContentPane().add(panel, null);

      // Set the Size and Display the Frame

      frame.setSize(800, 650);
      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
      frame.setLocation((d.width - frame.getSize().width)/2,
                        (d.height - frame.getSize().height)/2);

      frame.setResizable(false);
      frame.setVisible(true);
    }

//    panel.loadData();
  }

  /* ======================================================================= *
   * ------------------------ READ DATA XML METHODS ------------------------ *
   * ======================================================================= */

  /** Method readICSUnits()
   * <p> This method will read in the International Committee on Stratigraphy
   *     Stratigraphic Units.
   */

  private void readICSUnits()
  {
    ReadStratUnitsXMLFile pData =
        new ReadStratUnitsXMLFile(horizon.strat.io.ReadStratUnitsXMLFile.URL);
    stStruct.stICS = pData.Process(cmn.cmnStruct.DOE_CO2_DATA + "/ISC.xml");
//    stStruct.stICS = pData.Process(stStruct.stCMN.sPath + "/ISC.xml");

//horizon.strat.stratUtility.print(stStruct.stICS);
  }

  /** Method readKGSUnits()
   * <p> This method will read in the KGS Stratigraphic Units.
   */

  private void readKGSUnits()
  {
    ReadStratUnitsXMLFile pData =
        new ReadStratUnitsXMLFile(horizon.strat.io.ReadStratUnitsXMLFile.URL);
//    stStruct.stKGS = pData.Process(cmn.cmnStruct.TYPE_LOG_TOPS);
//    stStruct.stKGS = pData.Process(stStruct.stCMN.sPath + "/kansas.xml");
    stStruct.stKGS = pData.Process(cmn.cmnStruct.DOE_CO2_DATA + "/kansas.xml");
    stStruct.stKGS = horizon.strat.stratUtility.addICSArray( stStruct.stKGS,
                                                             stStruct.stICS );
//horizon.strat.stratUtility.print(stStruct.stKGS);
  }

  /** Method readLithologySymbols()
   * <p> This method will read in the Lithology Plot Symbols
   */

  private void readLithologySymbols()
  {
    ReadLithologySymbolsXMLFile pData =
        new ReadLithologySymbolsXMLFile(
          lith.lithology.io.ReadLithologySymbolsXMLFile.URL);

    stStruct.stSymbols = pData.Process(cmn.cmnStruct.DOE_CO2_DATA + "/lithology.xml");
//    stStruct.stSymbols = pData.Process(stStruct.stCMN.sPath + "/lithology.xml");
//lith.lithology.lithologySymbolsUtility.print(stStruct.stSymbols);
  }

  /** Method readPorositySymbols()
   * <p> This method will read in the Porosity Type Plot Symbols
   */

  private void readPorositySymbols()
  {
    ReadMaskSymbolsXMLFile pData =
        new ReadMaskSymbolsXMLFile( mask.io.ReadMaskSymbolsXMLFile.URL );

//    stStruct.stPorosity = pData.Process( stStruct.stCMN.sPath +
    stStruct.stPorosity = pData.Process( cmn.cmnStruct.DOE_CO2_DATA +
                                          "/porosity_types.xml");

//mask.maskSymbolsUtility.print(stStruct.stPorosity);
  }

  /** Method readParsePorosityThesaurus()
   * <p> This method will read in the Porosity Types Thesaurus
   */

  private void readParsePorosityThesaurus()
  {
    ReadParsePHIXMLFile pData =
        new ReadParsePHIXMLFile( parse.io.ReadParsePHIXMLFile.URL );

    stStruct.stParsePorosity =
        pData.Process( cmn.cmnStruct.DOE_CO2_DATA + "/porosity.xml");
//        pData.Process( stStruct.stCMN.sPath + "/porosity.xml");

//parse.parsePHIUtility.print( stStruct.stParsePorosity );
  }


  /** Method readFossilSymbols()
   * <p> This method will read in the Fossil Plot Symbols
   */

  private void readFossilSymbols()
  {
    ReadMaskSymbolsXMLFile pData =
        new ReadMaskSymbolsXMLFile( mask.io.ReadMaskSymbolsXMLFile.URL );

    stStruct.stFossils = pData.Process( cmn.cmnStruct.DOE_CO2_DATA + "/fossils.xml" );
//    stStruct.stFossils = pData.Process( stStruct.stCMN.sPath + "/fossils.xml" );
//mask.maskSymbolsUtility.print(stStruct.stFossils);
  }

  /** Method readParseFossilsThesaurus()
   * <p> This method will read in the Fossil Thesaurus
   */

  private void readParseFossilsThesaurus()
  {
    ReadParseFossilXMLFile pData =
        new ReadParseFossilXMLFile( parse.io.ReadParseFossilXMLFile.URL );

    stStruct.stParseFossils =
        pData.Process( cmn.cmnStruct.DOE_CO2_DATA + "/fossils_th.xml");
//        pData.Process( stStruct.stCMN.sPath + "/fossils_th.xml");

//parse.parseFossilUtility.print( stStruct.stParseFossils );
  }

  /** Method readSedStructureSymbols()
   * <p> This method will read in the Sedimentary Structure Plot Symbols
   */

  private void readSedStructureSymbols()
  {
    ReadMaskSymbolsXMLFile pData =
        new ReadMaskSymbolsXMLFile( mask.io.ReadMaskSymbolsXMLFile.URL );

//    stStruct.stSedStruct = pData.Process( stStruct.stCMN.sPath +
    stStruct.stSedStruct = pData.Process( cmn.cmnStruct.DOE_CO2_DATA +
                                          "/sed_structure.xml");
    stStruct.stSedStruct = mask.maskSymbolsUtility.setHierarchy(
        mask.maskSymbolsListStruct._SYMBOLS, stStruct.stSedStruct );

//mask.maskSymbolsUtility.print(stStruct.stSedStruct);
  }

  /** Method readParseSedStructSymbols()
   * <p> This method will read in the Texture Type Plot Symbols
   */

  private void readParseSedStructSymbols()
  {
    ReadParseSedStructXMLFile pData =
        new ReadParseSedStructXMLFile( parse.io.ReadParseSedStructXMLFile.URL );

    stStruct.stParseSedStruct =
        pData.Process( cmn.cmnStruct.DOE_CO2_DATA + "/sed_structures.xml");
//        pData.Process( stStruct.stCMN.sPath + "/sed_structures.xml");

//parse.parseSedStructUtility.print(stStruct.stParseSedStruct);
  }

  /** Method readParseColors()
   * <p> This method will read in the Colors
   */

  private void readParseColors()
  {
    ReadParseColorXMLFile pData =
        new ReadParseColorXMLFile( parse.io.ReadParseColorXMLFile.URL );

    stStruct.stParseColors =
        pData.Process( cmn.cmnStruct.DOE_CO2_DATA + "/colors.xml");
//        pData.Process( stStruct.stCMN.sPath + "/colors.xml");

//parse.parseColorUtility.print( stStruct.stParseColors );
  }

  /** Method readBioStratigraphicUnits()
   * <p> This method will read in the bio-stratigraphy fossil names
   */

  private void readBioStratigraphicUnits()
  {
    ReadBioStratUnitsXMLFile pData =
        new ReadBioStratUnitsXMLFile(
            horizon.bio.io.ReadBioStratUnitsXMLFile.URL );

//    stStruct.stBio = pData.Process( stStruct.stCMN.sPath + "/fossils_bio.xml",
    stStruct.stBio = pData.Process( cmn.cmnStruct.DOE_CO2_DATA + "/fossils_bio.xml",
                                    stStruct.stParseFossils,
                                    stStruct.stICS );

//horizon.bio.bioStratUtility.print( stStruct.stBio );
  }

  /** Method readKSBioStratigraphicUnits()
   * <p> This method will read in the Kansas bio-stratigraphy fossil names
   */

  private void readKSBioStratigraphicUnits()
  {
    ReadBioStratUnitsXMLFile pData =
        new ReadBioStratUnitsXMLFile(
            horizon.bio.io.ReadBioStratUnitsXMLFile.URL );

//    stStruct.stKSBio = pData.Process( stStruct.stCMN.sPath + "/kansas_bio.xml",
    stStruct.stKSBio = pData.Process( cmn.cmnStruct.DOE_CO2_DATA + "/kansas_bio.xml",
                                      stStruct.stParseFossils,
                                      stStruct.stICS );

//horizon.bio.bioStratUtility.print( stStruct.stKSBio );
  }

  /** Method readRockColorChart()
   * <p> This method will read in the rock color chart
   */

  private void readRockColorChart()
  {
    ReadColorChartXML pData =
        new ReadColorChartXML(
            rock.color.io.ReadColorChartXML.URL );

    stStruct.stRockColor =
        pData.Process( cmn.cmnStruct.DOE_CO2_DATA + "/rock_color_catalog.xml" );
//        pData.Process( stStruct.stCMN.sPath + "/rock_color_catalog.xml" );

//rock.color.rockColorChartUtility.print( stStruct.stRockColor );
  }

  /** Method readSoilColorChart()
   * <p> This method will read in the soil color chart
   */

  private void readSoilColorChart()
  {
    ReadColorChartXML pData =
        new ReadColorChartXML(
            rock.color.io.ReadColorChartXML.URL );

    stStruct.stSoilColor =
        pData.Process( cmn.cmnStruct.DOE_CO2_DATA + "/soil_color_catalog.xml" );
//        pData.Process( stStruct.stCMN.sPath + "/soil_color_catalog.xml" );

//rock.color.rockColorChartUtility.print( stStruct.stSoilColor );
  }

  /** Method readRockColumn()
   * <p> This method will read in the Rock Column Lithology Data
   */

  private void readRockColumn()
  {
    ReadRockColumnSymbolsXMLFile pData =
        new ReadRockColumnSymbolsXMLFile(
            parse.io.ReadRockColumnSymbolsXMLFile.URL );

    stStruct.stParseRock =
        pData.Process( cmn.cmnStruct.DOE_CO2_DATA + "/rock_lithology.xml" );
//        pData.Process( stStruct.stCMN.sPath + "/rock_lithology.xml" );
//
//parse.parseRockColumnSymbolsUtility.print( stStruct.stParseRock );
  }

  /* ======================================================================= *
   * --------------------------- APPLET METHODS ---------------------------- *
   * ======================================================================= */

  /** Method start()
   *  <p> This method will start the applet
   */

  public void start() { }

  /** Method stop()
   *  <p> This method will stop the applet
   */

  public void stop() { }

  /** Method destroy()
   *  <p> This method will destroy the applet
   */

  public void destroy() { }

  /** Method getAppletInfo()
   *  <p> This method will return Applet information
   *  @return - Type Log Plot
   */

  public String getAppletInfo() { return ("LAS IO Applet"); }

  /** Method getParameterInfo()
   *  <p> This method will get the parameter info
   *  @return pinfo - Array of Parameters Expected.
   */

  public String[][] getParameterInfo()
  {
    String[][] pinfo = { {"sLIST",  "String", "List of Well Header KID's"}, };

    return pinfo;
  }

  /** Method main()
   *  <p> This method will allow this Module to run either as an Application or
   *      an Applet.
   *  @param args = The Arguments passed in to query for the production data
   */

  public static void main(String[] args)
  {
    GXsection applet = new GXsection();

    applet.isStandalone = true;
    applet.sLIST_Standalone = new String("0");
    if (args.length > 0)
      applet.sLIST_Standalone = new String(args[0]);

    applet.init();
    applet.start();
  }
}


/*
 *  @version 1.1 02/28/2013
 *  @author  John Victorine
 */
