/*
 * @geminiLoadDataPanel.java Version 1.1 10/02/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package gemini.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

//import gemini.gui.geminiSourceTable;           // Table holding tops source list

import cmn.cmnStruct;                          // GLOBAL Common Data Structure
import cmn.cmnLASFileListStruct;               // LAS File List Data Structure
import cmn.cmnLASFileStruct;                   // LAS File Data Structure

import iqstrat.iqstratStruct;                  // GLOBAL IQSTRAT Data Structure
import iqstrat.iqstratHeadersListStruct;       // IQSTRAT Headers List Structure
import iqstrat.iqstratHeadersStruct;           // IQSTRAT Headers Data Structure
import iqstrat.iqstratRemarkListStruct;        // Geologist Descriptions List Data Strucutre

import las.lasFileDataStruct;                  // LAS File Data Structure

import las3.io.las3Read;                        // LAS 3 File Read Class
import las3.las3ListStruct;                    // LAS 3.0 List Structure
import las3.las3Struct;                        // LAS 3.0 Data Structure

import horizon.strat.stratListStruct;          // Stratigraphic Units List Structure

import lith.rock.rockColumnListStruct;         // Rock Column Data List Structure

import rock.rockDataListStruct;                // Rock Data List Data Structure

import rock.fossil.fossilListStruct;           // Fossil List Data Structure
import rock.sedimentary.sedimentaryListStruct; // Sed Structure List Data Structure
import rock.phi.phiListStruct;                 // Porosity List Data Strurture
import rock.color.rockColorListStruct;         // Rock Color List Data Structure

/** Class geminiLoadDataPanel
 *  <p> This file will display the data & summary of the data loaded.
 *
 *  @version 1.1 10/02/2012
 *  @author  John R. Victorine
 */

public class geminiLoadDataPanel extends JPanel implements ActionListener, Observer
{
  // Input Variables

  private int                  iWell      = -1;   // Well Order Number
  private String               sKID       = "0";  // Well Header KID
  private Observable           pNotifier  = null; // Observable Object this Frame
  private iqstratStruct        stStruct   = null; // Global IQSTRAT Data Structure
  private cmnStruct            stCMN      = null; // Global Common Data Structure

  // Global Variables

  private Observable           notifier   = null; // Observable Object this Frame
  private las3Read             p3         = null; // LAS 3.0 File Read

  public static final int _HEADERS     = 0;  // Headers Data
  public static final int _LAS_DATA    = 2;  // LAS File Data
  public static final int _TOPS_DATA   = 5;  // LAS File Data
  public static final int _CORE_DATA   = 3;  // Core Data

  // -- Rock Observational & Computed Data

  public static final int _ROCK_COLUMN = 20; // Rock Column Data

  public static final int _ROCK_PHI    = 10; // Rock Porosity
  public static final int _SED_STRUCT  = 11; // Sedimentary Structure
  public static final int _FOSSILS     = 12; // Fossils
  public static final int _ROCK_COLOR  = 13; // Rock Color
  public static final int _REMARKS     = 14; // Remarks/Comments

  // .. Header Information Data Structure

  private iqstratHeadersStruct stHeader = null; // Header Data Structure

  // .. LAS Data

  private int                     MAX_LOG   = 3;    // Maximum LAS Files Read in
  private int                     iLog      = 0;    // Log data count
  private lasFileDataStruct       stLog[]   = null; // LAS File Data Struct
  private cmnLASFileListStruct    stFiles   = null; // LAS File Info List Structure

  private lasFileDataStruct       stLAS     = null; // LAS File Data Struct

  // .. Horizons Data

//  private kgsUserListStruct       stUsers   = null; // List of Users
//  private kgsUserStruct           stSource  = null; // Source User

  private stratListStruct         stStrat   = null; // Tops List Data

  // .. Rock Data

  private rockDataListStruct      stCore    = null; // Measured Rock Data

  private rockColumnListStruct    stColumn  = null; // Rock Column Data List
  private phiListStruct           stPHI     = null; // Porosity type struct
  private fossilListStruct        stFossil  = null; // Fossil List struct
  private sedimentaryListStruct   stSedStruct = null; // Sed Struct Data
  private rockColorListStruct     stColor   = null; // Rock Color Data
  private iqstratRemarkListStruct stRemarks = null; // Remarks Data

  // Global Panel Widgets
  // -- Well Header Information Panel Data

  private static final int _NAME    = 0;
  private static final int _API     = 1;
  private static final int _STATUS  = 2;
  private static final int _LAT     = 3;
  private static final int _LONG    = 4;
  private static final int _TRS     = 5;
  private static final int _GL      = 6;
  private static final int _KB      = 7;
  private static final int _DF      = 8;
  private static final int _TD      = 9;
  private static final int _TOTAL_H = 10;

  private JLabel  lbl[] = null;

  // -- Panel Widgets

//  private JButton   btnKGS    = null; // KGS Icon - No Action
//  private JButton   btnGEO    = null; // Georeport Data
//  private JButton   btnClear  = null; // Clear Data

  // .. Tops List Dialog Widgets

//  private JFrame            frame  = null; // Frame holding source tops list
//  private geminiSourceTable pTable = null; // Tops Source List Table

//  private JButton   btnSelect = null; // Select Source
//  private JButton   btnCancel = null; // Cancel Action

  // -- LAS Filename Widgets

  private JPanel     pnlLASFile[] = null;
  private JLabel     lblLASFile[] = null;
  private JTextField txtLAS[]     = null;

  private static final int _ROW1  = 0;
  private static final int _ROW2  = 1;

  private static final int _GR    = 0;
  private static final int _OHM   = 1;
  private static final int _NPHI  = 2;
  private static final int _DPHI  = 3;
  private static final int _PE    = 4;
  private static final int _SPHI  = 5;
  private static final int _TOPS  = 6;
  private static final int _GEO   = 7;
  private static final int _CORE  = 8;
  private static final int _TOTAL = 9;

  private JLabel     lblData[][]  = null;

  private String DATA[][] = {
    { "GR", "OHM", "NPHI", "DPHI", "PE", "Sonic", "Tops", "GEO", "Core"  },
    { "NO", "NO",  "NO",   "NO",   "NO", "NO",    "NO",   "NO", "NO"  }
  };

  /** Constructor geminiLoadDataPanel()
   *  <p> This is the Constructor for this class.
   *  @param iWell     = Well Order Number
   *  @param sKID      = Well Header KID
   *  @param pNotifier = Observables passed
   *  @param stStruct  = Global IQSTRAT Data Structure
   */

  public geminiLoadDataPanel( int iWell, String sKID,
                              Observable pNotifier, iqstratStruct stStruct)
  {
    try
    {
	  this.iWell     = iWell;
	  this.sKID      = sKID;
	  this.pNotifier = pNotifier;
      this.stStruct  = stStruct;

      stCMN = stStruct.stCMN;

      jbInit();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create the Login Panel.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	int i,j;
    JPanel pnlTop         = new JPanel();
    JPanel panel          = new JPanel();

    JPanel pnlCenter      = new JPanel();
    JPanel pnlLASCenter   = new JPanel();

    JPanel pnlBase        = new JPanel();
    JPanel pnlHeader      = new JPanel();
    JPanel pnlData        = new JPanel();

    TitledBorder titledBorderTop = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Well (" + iWell + ")");
    titledBorderTop.setTitleFont(new java.awt.Font("Dialog", 1, 14));

    TitledBorder titledBorderLAS = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Log ASCII Standard (LAS) Files:");
    titledBorderLAS.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // Create an Observable

    notifier = new geminiLoadDataPanelObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver( this );

    // Build Frame

    this.setLayout(new BorderLayout());
    this.setBorder(titledBorderTop);

    // Top Panel

    pnlTop.setLayout( new BorderLayout() );
    pnlTop.setPreferredSize( new Dimension(10, 90) );

    // Build Header Panel

    panel = buildHeaderPanel( "Well (" + iWell + ")" );

    // Center Panel

    pnlCenter.setLayout( new BorderLayout() );

    // -- LAS Center Panel

    pnlLASCenter.setBorder(titledBorderLAS);
    pnlLASCenter.setLayout( new GridLayout(3,1) );

    // ... LAS File Panels

    pnlLASFile = new JPanel[3];
    lblLASFile = new JLabel[3];
    txtLAS     = new JTextField[3];

    for (i=0; i<3; i++)
    {
	  pnlLASFile[i] = new JPanel();
      pnlLASFile[i].setLayout( new BorderLayout() );

      lblLASFile[i] = new JLabel();
      lblLASFile[i].setHorizontalAlignment(SwingConstants.LEFT);
      lblLASFile[i].setFont(new java.awt.Font("Monospaced", 1, 11));
      lblLASFile[i].setText("" + i + ": ");

      txtLAS[i] = new JTextField();
      txtLAS[i].setFont(new java.awt.Font("Dialog", 1, 11));
      txtLAS[i].setEditable(false);
      txtLAS[i].setForeground(Color.blue);
      txtLAS[i].setText("");

      pnlLASCenter.add( pnlLASFile[i],  null );
      pnlLASFile[i].add( lblLASFile[i], BorderLayout.WEST );
      pnlLASFile[i].add( txtLAS[i],     BorderLayout.CENTER );
	}

    // Base Panel

    pnlBase.setLayout( new GridLayout(2,1) );
    pnlHeader.setLayout( new GridLayout(1,_TOTAL) );
    pnlData.setLayout( new GridLayout(1,_TOTAL) );

    lblData = new JLabel[2][_TOTAL];
    for (i=0; i<2; i++)
    {
	  for(j=0; j<_TOTAL; j++)
      {
		lblData[i][j] = new JLabel();
		lblData[i][j].setText( DATA[i][j] );
        lblData[i][j].setHorizontalAlignment(SwingConstants.CENTER);
        lblData[i][j].setFont(new java.awt.Font("Monospaced", 1, 11));

		if (i==0)
		  pnlHeader.add(lblData[i][j], null);
		else
		  pnlData.add(lblData[i][j], null);
      }
	}

    // Add Widgets to Panel

    this.add( pnlTop,              BorderLayout.NORTH );
    pnlTop.add( panel,             BorderLayout.CENTER );

    this.add( pnlCenter,           BorderLayout.CENTER );
    pnlCenter.add( pnlLASCenter,   BorderLayout.NORTH );

    this.add( pnlBase,             BorderLayout.SOUTH );
    pnlBase.add( pnlHeader,        null );
    pnlBase.add( pnlData,          null );
  }

  /** Metod buildHeaderPanel()
   *  <p> This method will build the header panel
   *  @param  sTitle    = Title of Panel
   *  @return pnlHeader = Well Header Panel
   */

  private JPanel buildHeaderPanel( String sTitle )
  {
	JPanel panel           = new JPanel();
	JPanel pnlTop          = new JPanel();
	JPanel pnlHeader       = new JPanel();
	JPanel pnlInfo         = new JPanel();
	JPanel pnlID           = new JPanel();
	JPanel pnlName         = new JPanel();
	JPanel pnlStatus       = new JPanel();
	JPanel pnlInfoPosition = new JPanel();
	JPanel pnlInfoXY       = new JPanel();
	JPanel pnlInfoZ        = new JPanel();

    TitledBorder titledBorderPanel = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        sTitle);
    titledBorderPanel.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    lbl = new JLabel[_TOTAL_H];
    for (int i=0; i<_TOTAL_H; i++)
    {
      lbl[i] = new JLabel();
    }

    panel.setLayout(new BorderLayout());

    // -- Top Panel

    pnlTop.setLayout(new BorderLayout());

    // -- Header Information Plot Labels, & Cross Section Depth Range Panels

    pnlHeader.setLayout(new BorderLayout());

    // ---- Header Information Plot Labels Panel

    pnlInfo.setLayout(new BorderLayout());

    // .. Well Information Panel

    pnlID.setLayout(new GridLayout(2,1));

    // .... Well Name Panel

    pnlName.setLayout(new BorderLayout());

    lbl[_NAME].setText("Name: ");

    // .... Well API-Number & Well Status Panel

    pnlStatus.setLayout(new GridLayout());

    lbl[_API].setText("API: ");
    lbl[_STATUS].setText("Status:");

    // .. Well Position Panel

    pnlInfoPosition.setLayout(new BorderLayout());

    // .... XY Well Position Panel

    pnlInfoXY.setLayout(new GridLayout());

    lbl[_LAT].setText("Lat: ");
    lbl[_LONG].setText("Long: ");

    // .... Z Well Position Panel

    pnlInfoZ.setLayout(new GridLayout());

    lbl[_TD].setText("Depth:");
    lbl[_GL].setText("GL:");
    lbl[_KB].setText("KB:");
    lbl[_DF].setText("DF:");

    panel.add(pnlHeader,           BorderLayout.CENTER);
    pnlHeader.add(pnlInfo,         BorderLayout.NORTH);

    pnlInfo.add(pnlID,             BorderLayout.NORTH);
    pnlID.add(pnlName,             null);
    pnlName.add(lbl[_NAME],        BorderLayout.WEST);

    pnlID.add(pnlStatus,           null);
    pnlStatus.add(lbl[_API],       BorderLayout.WEST);
    pnlStatus.add(lbl[_STATUS],    BorderLayout.CENTER);

    pnlInfo.add(pnlInfoPosition,   BorderLayout.CENTER);
    pnlInfoPosition.add(pnlInfoXY, BorderLayout.NORTH);
    pnlInfoXY.add(lbl[_LAT],       null);
    pnlInfoXY.add(lbl[_LONG],      null);

    pnlInfoPosition.add(pnlInfoZ,  BorderLayout.CENTER);
    pnlInfoZ.add(lbl[_TD],         null);
    pnlInfoZ.add(lbl[_GL],         null);
    pnlInfoZ.add(lbl[_KB],         null);
    pnlInfoZ.add(lbl[_DF],         null);

    return (panel);
  }

  /** Method close()
   *  <p> This method will release the memory allocated for global variables
   */

  public void close()
  {
    // Input Variables

    pNotifier = null; // Observable Object this Frame
    sKID      = null; // Well Header KID
    stStruct  = null; // Global IQSTRAT Data Structure
    stCMN     = null; // Global Common Data Structure

    // Global Variables

    notifier  = null; // Observable Object this Frame

    closeLASFile();

    if (stHeader != null)
      stHeader.delete();
    stHeader = null;      // Header Information Data

    closeData();

    // Global Panel Widgets

    // -- LAS Filename Widgets

    pnlLASFile = null;
    lblLASFile = null;
    txtLAS     = null;

    lblData    = null;
    DATA       = null;
  }

  /** Method closeData()
   * <p> This method will clear the data structures
   */

  public void closeData()
  {
	if (iLog > 0)
	{
	  for (int i=0; i<iLog; i++)
	  {
		if (stLog[i] != null)
		  stLog[i].delete();
	    stLog[i] = null;
	  }

	  iLog = 0;
	}

    if (stFiles != null)
      stFiles.delete();
    stFiles = null;       // LAS File Information

    // LAS Data

    if (stLAS != null)
      stLAS.delete();
    stLAS  = null;        // LAS File Data Structure

    // Horizons Data

    if (stStrat != null)
      stStrat.delete();
    stStrat = null;       // Stratigraphic Units

    if (stCore != null)
      stCore.delete();
    stCore = null;        // Measured Rock Data List Structure

    if (stPHI != null)
      stPHI.delete();
    stPHI = null;         // Porosity Track data

    if (stFossil != null)
      stFossil.delete();
    stFossil = null;      // Fossils

    if (stSedStruct != null)
      stSedStruct.delete();
    stSedStruct = null; // Sedimentary Structure

    if (stColor != null)
      stColor.delete();
    stColor = null;   // Rock Color Data

    if (stRemarks != null)
      stRemarks.delete();
    stRemarks = null;     // Remarks

    if (stColumn != null)
      stColumn.delete();
    stColumn      = null; // Rock Column Data List
  }

  /** Method closeLASFile()
   * <p> This method will close the LAS File Chooser Dialog
   */

  public void closeLASFile()
  {
    if (p3 != null)
      p3.delete();
    p3 = null;
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getWellOrder()
   *  <p> This method will identify the Well Data that is to be cleared
   */

  public int getWellOrder() { return (iWell); }

  /** Method getWellInfo()
   *  <p> This method will return well header information
   *  @return stHeader = well header information
   */

  public iqstratHeadersStruct getWellInfo() { return (stHeader); }

  /** Method getFiles
   *  <p> This method will get the LAS File Information Data
   *  return stFiles = LAS File Information Data
   */

  public cmnLASFileListStruct getFiles() { return (stFiles); }

  /** Method getLAS()
   * <p> These methods will return the LAS File Data Structure
   * @return stLAS = LAS File Data Structure
   */

  public lasFileDataStruct getLAS()
  {
	int    i,j;
	int    iRows  = 0;
	double dStart = 0.0;
	double dEnd   = 0.0;
	double dStep  = 0.0;
	double dNull  = -999.25;
	int    iTemp  = 0;
	double dStp   = 0.0;
    las3ListStruct st3 = null;

	if (iLog > 0)
	{
      if (iLog == 1)
      {
 	    stLAS = las.lasFileDataUtility.copy( stLog[0] );
  	  }
  	  else
  	  {
        stLAS = new lasFileDataStruct();

	    for (i=0; i<iLog; i++)
	    {
          dStep  = stLog[i].depthStep;
          if (dStep > stLog[i].depthStep)
          {
            dStep  = stLog[i].depthStep;
		  }

		  if (stLog[i].iLogs > 0)
		  {
		    for (j=0; j<stLog[i].iLogs; j++)
		      st3 = las3.las3Utility.add( stLog[i].stLAS3[j], st3 );
	      }
        }

	    for (i=0; i<iLog; i++)
	    {
		  if (i==0)
		  {
		    dStart = stLog[i].depthStart;
		    dEnd   = stLog[i].depthEnd;
		    dStep  = stLog[i].depthStep;
		    dNull  = stLog[i].dNull;

            stLAS = las.lasFileDataUtility.copyHeader( stLog[i], stLAS );
		  }

		  if (dStart > stLog[i].depthStart)
		    dStart = stLog[i].depthStart;

          if (dEnd < stLog[i].depthEnd)
		    dEnd   = stLog[i].depthEnd;

          if (dStep > stLog[i].depthStep)
          {
            dStep  = stLog[i].depthStep;
		  }
	    }

        iTemp  = (int) dStart;
        dStart = (double) iTemp;

        iTemp  = 1 + (int) dEnd;
        dEnd   = (double) iTemp;

        dStp = 0.5;
        if (dStep < 0.5)
          dStp = 0.25;
        if (dStep < 0.25)
          dStp = 0.1;

        dStep = dStp;
	    iRows = (int) ( Math.abs(dEnd - dStart) / dStep ) + 1;

        stLAS.depthStart = dStart;
        stLAS.depthEnd   = dEnd;
        stLAS.depthStep  = dStep;

        stLAS = las3.las3LoadLogData.initData( iRows, stLAS );

	    for (i=0; i<iLog; i++)
	    {
          stLAS = las3.las3LoadLogData.mergeLogData( stLog[i], stLAS );
	    }
	  }
    }

    if (st3 != null)
    {
	  if (st3.iCount > 0)
	  {
	    if (stLAS != null)
	    {
		  stLAS.stLAS3 = new las3Struct[st3.iCount];
		  stLAS.iLogs  = st3.iCount;
		  for (i=0; i<st3.iCount; i++)
		  {
		    stLAS.stLAS3[i] = new las3Struct();
		    stLAS.stLAS3[i] = las3.las3Utility.copy( st3.stItem[i] );
		  }
		}
	  }
	}

    return (stLAS);
  }

  /** Method getTops()
   * <p> This method will return the Formation Tops
   * @return stStrat = Formation Tops Data List Struct
   */

  public stratListStruct getTops() { return ( stStrat ); }

  /** Method get Measured Section Data()
   * <p> These methods will return the Measured Section Data
   * @return stCore      = Rock Core Data
   * @return stColumn    = Rock Lithology Column Data
   * @return stPHI       = Porosity Type Data
   * @return stFossil    = General Fossil Data
   * @return stSedStruct = Sedimentary Structure Data
   * @return stColor     = Rock Color Data
   * @return stRemarks   = Measured Section Raw Data
   */

  public rockDataListStruct      getCoreData()   { return ( stCore ); }      // Rock Data
  public rockColumnListStruct    getRockColumn() { return ( stColumn ); }    // Rock Column Data
  public phiListStruct           getPorosity()   { return ( stPHI ); }       // Porosity type Data
  public fossilListStruct        getFossils()    { return ( stFossil ); }    // Fossil Data
  public sedimentaryListStruct   getSedStruct()  { return ( stSedStruct ); } // Sed Structure Data
  public rockColorListStruct     getColor()      { return ( stColor ); }     // Rock Color Data
  public iqstratRemarkListStruct getGeo()        { return ( stRemarks ); }   // Remarks Data

  /** Method getData()
   * <p> This method will retrieve data
   * @param iLocation = Location of Data being retrieved
   * @param iData     = Data Type being retrieved
   */

  private void getData( int iData)
  {
	int i=0;

    switch (iData)
    {
      case _HEADERS:    // Headers Data
          stHeader = iqstrat.iqstratHeadersUtility.transfer( p3.getWellData() );
          setWellHeaderLabels( stHeader );
        break;

      case _LAS_DATA:   // LAS File Data
        if (stFiles == null)
        {
          stFiles        = new cmnLASFileListStruct();
          stFiles.stItem = new cmnLASFileStruct[MAX_LOG];
          stLog          = new lasFileDataStruct[MAX_LOG];

          for (i=0; i<MAX_LOG; i++)
          {
            stFiles.stItem[i] = new cmnLASFileStruct();
            stLog[i]          = new lasFileDataStruct();
          }
		}

        if (iLog < MAX_LOG)
        {
		  if (p3 != null)
		  {
			stLog[iLog] = las.lasFileDataUtility.transfer( p3.getLASData() );

            if (stLog[iLog] != null)
            {
			  stLog[iLog].iSource = las3.las3Constants._SOURCE_KGS;

			  if (stLog[iLog].iGR > 0)
			  {
				lblData[_ROW2][_GR].setText( "YES" );
				lblData[_ROW2][_GR].setForeground(Color.blue);
		      }

			  if (stLog[iLog].iPE > 0)
			  {
				lblData[_ROW2][_PE].setText( "YES" );
				lblData[_ROW2][_PE].setForeground(Color.blue);
		      }

              for (i=las.lasStandardTools._RES; i<las.lasStandardTools._TH; i++)
              {
  		 	    if (stLog[iLog].checkData(i))
		  	    {
				  lblData[_ROW2][_OHM].setText( "YES" );
			  	  lblData[_ROW2][_OHM].setForeground(Color.blue);
		        }
              }

			  if (stLog[iLog].iNPHI > 0)
			  {
				lblData[_ROW2][_NPHI].setText( "YES" );
				lblData[_ROW2][_NPHI].setForeground(Color.blue);
		      }

			  if ((stLog[iLog].iDPHI > 0) || (stLog[iLog].iRHOB > 0))
			  {
				lblData[_ROW2][_DPHI].setText( "YES" );
				lblData[_ROW2][_DPHI].setForeground(Color.blue);
		      }

			  if ((stLog[iLog].iSPHI > 0) || (stLog[iLog].iDT > 0))
			  {
				lblData[_ROW2][_SPHI].setText( "YES" );
				lblData[_ROW2][_SPHI].setForeground(Color.blue);
		      }
			}
		  }

          stFiles.stItem[iLog].iType = util.utilFileIO.DIRECTORY;
          if (stLog[iLog].sReadMethod.equals("URL"))
            stFiles.stItem[iLog].iType = util.utilFileIO.URL_FILE;

          stFiles.stItem[iLog].sDirectory = new String( stLog[iLog].sDirectory );
          stFiles.stItem[iLog].sFilename  = new String( stLog[iLog].sFilename );
          stFiles.stItem[iLog].sVersion   = new String( stLog[iLog].sVersion );

          stFiles.stItem[iLog].dNull      = stLog[iLog].dNull;
          stFiles.stItem[iLog].depthStart = stLog[iLog].depthStart;
          stFiles.stItem[iLog].depthEnd   = stLog[iLog].depthEnd;
          stFiles.stItem[iLog].depthStep  = stLog[iLog].depthStep;

          iLog++;
          stFiles.iCount = iLog;
	    }
        break;

      case _TOPS_DATA:       // Tops Data
        if (stStrat == null)
        {
          stStrat = horizon.strat.stratUtility.transfer( p3.getTops() );

          if (stStrat != null)
          {
            stStrat = horizon.strat.stratUtility.addICSArray( stStrat, stStruct.stICS );
            stStrat = horizon.strat.stratUtility.fillBaseDepth( stStrat, stStruct.stKGS );

            stStrat.sTOPS  = new String("Kansas Geological Survey Type Log"); // Formation Source
            stStrat.sTOPDR = new String("Log Depth");    // Tops Depth Reference
            stStrat.iSource = las3.las3Constants._SOURCE_KGS;

 	        if (stStrat.iCount > 0)
		    {
		      lblData[_ROW2][_TOPS].setText( "YES" );
		      lblData[_ROW2][_TOPS].setForeground(Color.blue);
	  	    }
  	      }
	    }
        break;

      case _CORE_DATA:       // Core Data
        stCore = rock.rockDataUtility.transfer( p3.getRockData() );

        if (stCore != null)
        {
		  lblData[_ROW2][_CORE].setText( "YES" );
		  lblData[_ROW2][_CORE].setForeground(Color.blue);
		}
        break;

      case _REMARKS: // Remarks/Comments
		if (p3 != null)
		{
          stRemarks = iqstrat.iqstratRemarkUtility.transfer( p3.getRemarks() );

          if (stRemarks != null)
          {
		    lblData[_ROW2][_GEO].setText( "YES" );
		    lblData[_ROW2][_GEO].setForeground(Color.blue);
		  }
	    }
        break;

      case _ROCK_COLUMN:
        stColumn = lith.rock.rockColumnUtility.copyList(
		             parse.parseRockColumnSymbolsUtility.parse(
		               iqstrat.iqstratRemarkUtility.copyList( stRemarks ),
		               stStruct.stParseRock ));
        break;

      case _ROCK_PHI:  // Rock Porosity
        stPHI = rock.phi.phiUtility.copyList(
		          parse.parsePHIUtility.parse(
                    iqstrat.iqstratRemarkUtility.copyList( stRemarks ),
                    stStruct.stParsePorosity ));
        break;

      case _SED_STRUCT: // Sedimentary Structure
        stSedStruct = rock.sedimentary.sedimentaryUtility.copyList(
                        parse.parseSedStructUtility.parse(
                          iqstrat.iqstratRemarkUtility.copyList( stRemarks ),
                          stStruct.stParseSedStruct ));
        break;

      case _FOSSILS: // Fossils
        stFossil = rock.fossil.fossilUtility.copyList(
                     parse.parseFossilUtility.parse(
                       iqstrat.iqstratRemarkUtility.copyList( stRemarks ),
                       stStruct.stParseFossils,
                       stStruct.stBio,
                       stStruct.stKSBio ));
        break;

      case _ROCK_COLOR: // Rock Color
        stColor = rock.color.rockColorUtility.copyList(
			        parse.parseColorUtility.parse(
                      iqstrat.iqstratRemarkUtility.copyList( stRemarks ),
                      stStruct ));
        break;
    }
  }

  /** Method getGeoReportData()
   * <p> This method will compute the Geologist Report Data tracks if the
   *     tracks are empty
   */

  private void getGeoReportData()
  {
    if (stRemarks != null)
    {
      // Rock Column Lithology
      if (stColumn == null)    { getData( _ROCK_COLUMN ); }
      // Fossils
      if (stFossil == null)    { getData( _FOSSILS ); }
      // Sedimentary Structures
      if (stSedStruct == null) { getData( _SED_STRUCT ); }
      // Rock Color
      if (stColor == null)     { getData( _ROCK_COLOR ); }
      // Porosity Type
      if (stPHI == null)       { getData( _ROCK_PHI ); }
    }
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setWellHeaderLabels()
   *  <p> This method will set the Well Header Label Fields
   *  @param st = Well Header Information Data Structure
   */

  private void setWellHeaderLabels( iqstratHeadersStruct st )
  {
    if (st != null)
    {
      lbl[_NAME].setText( "Name: "     + st.sName );
      lbl[_API].setText(st.sAPI  + " ");
      lbl[_STATUS].setText( "Status: " + st.status );
      lbl[_LAT].setText( "Lat: "       + st.dLatitude );
      lbl[_LONG].setText( "Long: "     + st.dLongitude );
      lbl[_TD].setText( "Depth: "      + st.depth );

      lbl[_GL].setText("GL:");
      lbl[_KB].setText("KB:");
      lbl[_DF].setText("DF:");

      if (st.dGL > 0.0)
        lbl[_GL].setText("GL: " + st.dGL);
      if (st.dKB > 0.0)
        lbl[_KB].setText("KB: " + st.dKB);
      if (st.dDF > 0.0)
        lbl[_DF].setText("DF: " + st.dDF);
    }
    else
    {
      lbl[_NAME].setText( "Name: " );
      lbl[_API].setText( " " );
      lbl[_STATUS].setText( "Status: " );
      lbl[_LAT].setText( "Lat: " );
      lbl[_LONG].setText( "Long: " );
      lbl[_TD].setText("Depth:");
      lbl[_GL].setText("GL:");
      lbl[_KB].setText("KB:");
      lbl[_DF].setText("DF:");
    }
  }

  /* ===================================================================== *
   * ---------------------- LOAD KGS DATA METHODS ------------------------ *
   * ===================================================================== */


  /** Method loadLASData()
   * <p> This method will display a dialog with a list of available LAS Files
   */

  public void loadLASData()
  {
	String sDirectory = cmn.cmnStruct.DOE_CO2_LAS3;
	String sName      = sKID + ".las";

    p3 = new las3Read( stStruct );
    p3.setPath( "URL", sDirectory, sName );
    p3.getData();

    getData( _HEADERS );
    getData( _LAS_DATA );
    getData( _TOPS_DATA );
    getData( _CORE_DATA );
    getData( _REMARKS );

    // Compute Rock Image Tracks from Remarks Data

    getGeoReportData();

    if (iLog > 0)
    {
      if (iLog == 1) { txtLAS[0].setText(stFiles.stItem[0].sFilename); }
      if (iLog == 2) { txtLAS[1].setText(stFiles.stItem[1].sFilename); }
      if (iLog == 3) { txtLAS[2].setText(stFiles.stItem[2].sFilename); }
    }

    p3.delete();
    p3 = null;

    if (pNotifier != null)
      pNotifier.notifyObservers( new String( "FREE MEMORY" ) );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Gemini Start Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (pNotifier != null)
      pNotifier.notifyObservers( new String( "FREE MEMORY" ) );
  }

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

    if (pNotifier != null)
      pNotifier.notifyObservers( new String( "FREE MEMORY" ) );
  }
}

/** CLASS geminiLoadDataPanelObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class geminiLoadDataPanelObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 10/02/2012
 *  @author  John Victorine
 */
