/*
 * @geminiLoadLAS3Panel.java Version 1.1 03/04/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package gemini.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import cmn.cmnStruct;                          // GLOBAL Common Data Structure
import cmn.cmnLASFileListStruct;               // LAS File List Data Structure

import gui.guiMemoryPanel2;                    // Memory Panel

import iqstrat.iqstratStruct;                  // GLOBAL IQSTRAT Data Structure
import iqstrat.iqstratHeadersListStruct;       // IQSTRAT Headers List Structure
import iqstrat.iqstratHeadersStruct;           // IQSTRAT Headers Data Structure
import iqstrat.iqstratRemarkListStruct;        // Geologist Descriptions List Data Strucutre

import gemini.gui.geminiLoadDataPanel;         // Load Data Panel

import las.lasFileDataStruct;                  // LAS File Data Structure

import horizon.strat.stratListStruct;          // Stratigraphic Units List Structure

import lith.rock.rockColumnListStruct;         // Rock Column Data List Structure

import rock.rockDataListStruct;                // Rock Data List Data Structure

import rock.fossil.fossilListStruct;           // Fossil List Data Structure
import rock.sedimentary.sedimentaryListStruct; // Sed Structure List Data Structure
import rock.phi.phiListStruct;                 // Porosity List Data Strurture
import rock.color.rockColorListStruct;         // Rock Color List Data Structure

import xsection.gui.xsectionControlFrame;      // Cross Section Plot Control Dialog

import util.utilWaitAnimation;          // Utility Wait Animation Class
import util.SwingWorker;                // Utility Swing Worker Class

/** Class geminiLoadLAS3Panel
 *  <p> This file will display the Kansas Oil & Gas Wells.
 *
 *  @version 1.1 03/04/2013
 *  @author  John R. Victorine
 */

public class geminiLoadLAS3Panel extends JPanel implements ActionListener, Observer
{
  // Input Variables

  private JFrame                   parent   = null; // Standalone Frame
  private iqstratStruct            stStruct = null; // Global IQSTRAT Data Structure
  private String                   sKIDs[]  = null; // List of Well Header KID's

  // Global Variables

  private Observable               notifier = null; // Observable Object this Frame
//  private iqstratHeadersListStruct stWells  = null; // Well List Data Structure

  // -- Well Data Panel & Cross Section Control Frame

  private geminiLoadDataPanel      pnlLoad[] = null; // Load Well Data Panel
  private guiMemoryPanel2          pnlMemory = new guiMemoryPanel2(); // Memory Panel
  private xsectionControlFrame     profile   = null; // Profile Plot Control

  // -- Well Information Data

  private int iWells    = 4;   // Total Wells Selected
  private int iCount    = 0;   // Total number of wells loaded
  private int iTotal    = 0;   // Total number of wells requested

  // -- Screen Width

  private int iControlWidth = 400;  // Control Panel Width

  // Global Frame Widgets

  private JProgressBar pbWait = null;
  private JLabel       lbl    = new JLabel();     // Status

  private JButton btnContinue = new JButton();

  /** Constructor geminiLoadLAS3Panel()
   *  <p> This is the Constructor for this class
   *  @param parent    = standalone frame
   *  @param stStruct  = Global IQSTRAT Data Structure
   *  @param sKIDList  = Well Header KIDS String
   */

  public geminiLoadLAS3Panel( JFrame parent, iqstratStruct stStruct, String sKIDList )
  {
    try
    {
	  this.parent   = parent;
      this.stStruct = stStruct;
      this.sKIDs    = getKIDs( sKIDList );

      if (this.sKIDs != null)
      {
        this.stStruct.iWells = this.sKIDs.length;
        iTotal = this.sKIDs.length;
	  }

      jbInit();

      pnlMemory.setMemory();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
      String sMessage = new String("geminiLoadLAS3Panel(): " + ex.getMessage());
      JOptionPane.showMessageDialog((Component) null,
                                    sMessage,
                                    "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Panel.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    int i,j;
    String sTemp     = "0";
    JPanel pnlTop    = new JPanel();
    JPanel pnlCenter = new JPanel();
    JPanel panels[]  = null;
    JPanel pnlBase   = new JPanel();
    JPanel pnlLbl    = new JPanel();

    // Create an Observable

    notifier = new geminiLoadLAS3PanelObservable();

    // -- Add this Dialog to the notifier as observer

    notifier.addObserver( this );

    // Build Frame

    this.setLayout(new BorderLayout());

    // -- Center Panel

    if (iWells < 4)
      pnlCenter.setLayout(new GridLayout());
    else
      pnlCenter.setLayout(new GridLayout(2, 2));

    pnlLoad = new geminiLoadDataPanel[iWells];

    for (i=0; i<iWells; i++)
    {
	  sTemp = "0";
	  if (sKIDs != null)
	  {
		if (i < sKIDs.length)
		{
		  sTemp = new String(sKIDs[i]);
		}
	  }

      pnlLoad[i] = new geminiLoadDataPanel(i, sTemp, notifier, stStruct);
    }

    // -- Base Panel

    pnlTop.setLayout(new GridLayout(2,1));
    pnlBase.setLayout(new GridLayout(2,1));

    pnlLbl.setLayout(new BorderLayout());

    lbl.setText("Select Button above to Start Loading Data");
	lbl.setHorizontalTextPosition(JLabel.CENTER);
	lbl.setFont(new java.awt.Font("Dialog", 1, 14));
    lbl.setBackground(Color.white);
    lbl.setForeground(Color.red);

    pbWait = new JProgressBar(0, 100);
    pbWait.setValue( getProgress() );
    pbWait.setStringPainted( true );

    btnContinue.setFont(new java.awt.Font("Dialog", 1, 15));
    btnContinue.setText("Press this Button to Load Data");
    btnContinue.setForeground( Color.red );
    btnContinue.addActionListener(this);

    // Add Widgets to Frame

    this.add(pnlTop,        BorderLayout.NORTH);
    pnlTop.add(btnContinue, null);
    pnlTop.add(pnlMemory,   null);

    this.add(pnlCenter,   BorderLayout.CENTER);
    for (i=0; i<iWells; i++)
    {
      pnlCenter.add(pnlLoad[i],    BorderLayout.CENTER);
	}

    this.add(pnlBase,              BorderLayout.SOUTH);
    pnlBase.add(pnlLbl,            null);
    pnlLbl.add(lbl,                BorderLayout.CENTER);
    pnlBase.add(pbWait,            null);
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    stStruct  = null; // Global IQSTRAT Data structure
    sKIDs     = null; // List of Well Header KID's

    // Global Variables

    notifier  = null; // Observable Object this Frame

    // -- Well Data

    for (int i=0; i<iWells; i++)
    {
      if (pnlLoad != null)
      {
        if (pnlLoad[i] != null)
          pnlLoad[i].close();
        pnlLoad[i] = null;
	  }
	}
	pnlLoad = null;

    if (pnlMemory != null)
      pnlMemory.close();
    pnlMemory = null;

	if (profile != null)
	  profile.close();
	profile = null;

    // Global Frame Widgets

    pbWait = null;
    lbl    = null;
    btnContinue = null;

    if (parent != null)
    {
      parent.dispose();
      parent = null;

      System.exit(0);
    }
  }

  /** Method closeData()
   *  @param iPanel = the Data Load Panel selected
   */

  public void closeData( int iPanel ) { pnlLoad[iPanel].closeData(); }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getKIDs()
   *  <p> This method will retrieve the KID List
   */

  private static String[] getKIDs(String sList)
  {
    String tokens[] = null;
    String sDelimiter = new String("[,]+");

    tokens = sList.split(sDelimiter);

    return (tokens);
  }

  /** Method getWellCount()
   *  <p> This method will retrieve the total well count
   *  @return iWells = Total wells selected
   */

  public int getWellCount() { return (iWells); }

  /** Method getWellInfo()
   *  <p> This method will return well header information
   *  @param  iPanel = the Data Load Panel selected
   *  @return well header information
   */

  public iqstratHeadersStruct getWellInfo( int iPanel )
  {
	return ( pnlLoad[iPanel].getWellInfo() );
  }

  /** Method getFiles
   *  <p> This method will get the LAS File Information Data
   *  @param  iPanel = the Data Load Panel selected
   *  @return LAS File Information Data
   */

  public cmnLASFileListStruct getFiles( int iPanel )
  {
	return (pnlLoad[iPanel].getFiles());
  }

  /** Method getLAS()
   * <p> These methods will return the LAS File Data Structure
   *  @param  iPanel = the Data Load Panel selected
   *  @return LAS File Data Structure
   */

  public lasFileDataStruct getLAS( int iPanel )
  {
	return ( pnlLoad[iPanel].getLAS() );
  }

  /** Method getTops()
   * <p> This method will return the Formation Tops
   *  @param  iPanel = the Data Load Panel selected
   *  @return Formation Tops Data List Struct
   */

  public stratListStruct getTops( int iPanel )
  {
	return ( pnlLoad[iPanel].getTops() );
  }

  /** Method get Measured Section Data()
   * <p> These methods will return the Measured Section Data
   * @return Rock Core Data
   * @return Rock Lithology Column Data
   * @return Porosity Type Data
   * @return General Fossil Data
   * @return Sedimentary Structure Data
   * @return Rock Color Data
   * @return Measured Section Raw Data
   */

  public rockDataListStruct getCoreData( int iPanel )
  {
	return ( pnlLoad[iPanel].getCoreData() );     // Rock Column Data
  }

  public rockColumnListStruct getRockColumn( int iPanel )
  {
	return ( pnlLoad[iPanel].getRockColumn() );     // Rock Column Data
  }

  public phiListStruct getPorosity( int iPanel )
  {
	return ( pnlLoad[iPanel].getPorosity() );        // Porosity type Data
  }

  public fossilListStruct getFossils( int iPanel )
  {
	return ( pnlLoad[iPanel].getFossils() );     // Fossil Data
  }

  public sedimentaryListStruct getSedStruct( int iPanel )
  {
	return ( pnlLoad[iPanel].getSedStruct() );  // Sed Structure Data
  }

  public rockColorListStruct getColor( int iPanel )
  {
	return ( pnlLoad[iPanel].getColor() );      // Rock Color Data
  }

  public iqstratRemarkListStruct getGeo( int iPanel )
  {
	return ( pnlLoad[iPanel].getGeo() );    // Remarks Data
  }

  /** Method getProgress()
   * <p> This method will compute the percent of the Total wells loaded
   * @return iPercent = Percent of memory used
   */

  public int getProgress()
  {
    int iPercent = 0;

    if (iTotal > 0)
      iPercent = (int) ( (100 * iCount) / iTotal );

    return ( iPercent );
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method WaitLoadData()
   * <p> This method will display the Wait Animation for opening and parsing a
   *     digtial LAS File.
   */

  private void WaitLoadData()
  {
    java.net.URL url = null;

    // Get the URL Path to display the Wait Animation

    try
    {
      url = new java.net.URL(cmn.cmnStruct.IQSTRAT_BASE + "/");
    }
    catch(Exception ex)
    {
      System.out.println("Wrong Image Directory" + ex);
    }

    // Start the Wait Animation Dialog

    final utilWaitAnimation pDialog = new utilWaitAnimation(url);
    pDialog.startAnimation();

    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
        loadData();

        return "OK";
      }

      //Runs on the event-dispatching thread.

      public void finished() { pDialog.stopAnimation(); }
    };

    // Start the Wait Animation Thread

    worker.start();  //required for SwingWorker
  }

  /** Method loadData()
   *  <p> This method will load the data panels
   */

  public void loadData()
  {
    for (int i=0; i<this.sKIDs.length; i++)
    {
      lbl.setText("Loading Please Wait - Well " + i);

      pnlLoad[i].loadLASData();

      iCount++;
      pbWait.setValue( getProgress() );
    }

    lbl.setText("Data Loaded");
    plotProfile();
  }

  /** Method setData()
   *  <p> This method will retrieve data from the Area Control Frame
   */

  private void setData()
  {
	if (profile != null)
	{
  	  if (pnlLoad != null)
  	  {
		for (int i=0; i<sKIDs.length; i++)
		{
	      profile.setData( i,
	        iqstrat.iqstratHeadersUtility.transfer( pnlLoad[i].getWellInfo() ),
            cmn.cmnLASFileUtility.transfer( pnlLoad[i].getFiles() ),
            las.lasFileDataUtility.transfer( pnlLoad[i].getLAS() ),
            horizon.strat.stratUtility.transfer( pnlLoad[i].getTops() ),
            rock.rockDataUtility.transfer( pnlLoad[i].getCoreData() ),
	        lith.rock.rockColumnUtility.transfer( pnlLoad[i].getRockColumn() ),
	        rock.phi.phiUtility.transfer( pnlLoad[i].getPorosity() ),
		    rock.sedimentary.sedimentaryUtility.transfer( pnlLoad[i].getSedStruct() ),
		    rock.fossil.fossilUtility.transfer( pnlLoad[i].getFossils() ),
	        rock.color.rockColorUtility.transfer( pnlLoad[i].getColor() ),
            iqstrat.iqstratRemarkUtility.transfer( pnlLoad[i].getGeo() ) );
		}

//	    pLoad.close();
//	    pLoad = null;

	    profile.plot();
      }
	}
  }


  /* ===================================================================== *
   * ---------------------- DISPLAY PROFILE PLOT ------------------------- *
   * ===================================================================== */

  /** Method plotProfile()
   *  <p> This method will plot the Profile Plot of selected wells
   */

  private void plotProfile()
  {
	if (profile != null)
	  profile.close();
	profile = null;

	profile = new xsectionControlFrame( notifier, stStruct );

	setData();
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (pnlMemory != null) pnlMemory.setMemory();

	if (event.getSource() == btnContinue) { WaitLoadData(); } //{ loadData(); }
  }

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    int    icnt = -1;
    int    iW   = -1;
    String sArg = new String( (String) arg);

    if (pnlMemory != null) pnlMemory.setMemory();

//	if (sArg.equals("Clear Well Data 0")) { iW = 0; }
//	if (sArg.equals("Clear Well Data 1")) { iW = 1; }
//	if (sArg.equals("Clear Well Data 2")) { iW = 2; }
//	if (sArg.equals("Clear Well Data 3")) { iW = 3; }

//    if (iW > -1)
//    {
//	  pnlLoad[iW].clear();
//    }

    if (pnlMemory != null) pnlMemory.setMemory();

    if (sArg.equals("SHOW MEMORY"))
    {
      if (pnlMemory != null) pnlMemory.setMemory();
    }

    if (sArg.equals("FREE MEMORY"))
    {
      if (pnlMemory != null) pnlMemory.freeMemory();
    }
  }
}

/** CLASS geminiLoadLAS3PanelObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class geminiLoadLAS3PanelObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 03/04/2013
 *  @author  John Victorine
 */
