/*
 * @guiMemoryPanel.java Version 1.1 02/07/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

/** Class guiMemoryPanel
 *  <p> This Class will display the memory usage.
 *
 *  @version 1.1 02/07/2013
 *  @author  John R. Victorine
 */

public class guiMemoryPanel extends JPanel implements ActionListener
{
  // Memory Progress Bars

  private JProgressBar pbUSE = null;
  private JProgressBar pbPC  = null;

  /** Constructor guiMemoryPanel()
   *  <p> This is the Constructor for this class.
   */

  public guiMemoryPanel()
  {
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlFreeMemory = new JPanel();
    JPanel pnlPCMemory   = new JPanel();

    // Textfield Panel Borders

    TitledBorder titledBorderUsage = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Memory Usage:");
    titledBorderUsage.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTotal = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Memory Usage - PC Total Available Memory");
    titledBorderTotal.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // Build Panel

    this.setLayout(new BorderLayout());
    this.setPreferredSize(new Dimension(10, 100));

    // Free Memory Panel

    pnlFreeMemory.setLayout(new BorderLayout());
    pnlFreeMemory.setBorder(titledBorderUsage);

    pbUSE = new JProgressBar(0, 100);
    pbUSE.setValue( util.utilMemory.getFreeUsage() );
    pbUSE.setStringPainted( true );

    // PC Memory Panel

    pnlPCMemory.setLayout(new BorderLayout());
    pnlPCMemory.setBorder(titledBorderTotal);

    pbPC = new JProgressBar(0, 100);
    pbPC.setValue( util.utilMemory.getPCUsage() );
    pbPC.setStringPainted( true );

    this.add(pnlFreeMemory,  BorderLayout.NORTH);
    pnlFreeMemory.add(pbUSE, BorderLayout.CENTER);

    this.add(pnlPCMemory,    BorderLayout.SOUTH);
    pnlPCMemory.add(pbPC,    BorderLayout.CENTER);
  }

  /** Method setMemory
   * <p> This method will show the memory usage after forcing the Java program
   *     to free memory and do garbage collection.
   */

  public void freeMemory()
  {
    util.utilMemory.free();
    if (pbPC  != null) pbPC.setValue( util.utilMemory.getPCUsage() );
    if (pbUSE != null) pbUSE.setValue( util.utilMemory.getFreeUsage() );
  }

  /** Method setMemory
   * <p> This method will show the memory usage after forcing the Java program
   *     to free memory and do garbage collection.
   */

  public void setMemory()
  {
    if (pbPC  != null) pbPC.setValue( util.utilMemory.getPCUsage() );
    if (pbUSE != null) pbUSE.setValue( util.utilMemory.getFreeUsage() );
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Memory Progress Bars

    pbUSE = null;
    pbPC  = null;
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
  }
}

/*
 *  @version 1.1 02/07/2013
 *  @author  John Victorine
 */
