/*
 * @stratListFrame.java Version 1.1 11/27/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package horizon.strat.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.Observable;

import javax.swing.*;
import javax.swing.event.*;

import horizon.strat.stratListStruct;
import horizon.strat.stratStruct;
import horizon.strat.gui.stratListTable;

/** Class stratListFrame
 *  <p> This Class will allow the user to select the status of the data
 *
 *  @version 1.1 11/27/2012
 *  @author  John R. Victorine
 */

public class stratListFrame extends JFrame implements ActionListener
{
  private Observable      notifier = null; // Observable notifier
  private stratListStruct stList   = null; // Stratigraphic Units List Data
  private stratListTable  pTable   = null; // Stratigraphic Units Table

  private static final int    _SYSTEMS = 14;
  private static final String SYSTEMS[] = {
	"All",
    "Quaternary",
    "Neogene",
    "Cretaceous",
    "Jurassic",
    "Permian",
    "Carboniferous",
    "Pennsylvanian",
    "Mississippian",
    "Devonian",
    "Silurian",
    "Ordovician",
    "Precambrian",
    "Cambrian"
  };

  private JRadioButton rb[] = null;

  private JButton btnSelect = new JButton();
  private JButton btnCancel = new JButton();

  /** Constructor stratListFrame()
   *  <p> This is the Constructor for this class.
   *  @param notifier = Observable notifier
   *  @param st       = the Stratigraphic Units List Structure
   */

  public stratListFrame(Observable notifier, stratListStruct st)
  {
    try
    {
      this.notifier = notifier;
      this.stList   = st;

      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel pnlTop      = new JPanel();
    JPanel pnlButtons  = new JPanel();
    JPanel pnl         = new JPanel();
    JScrollPane scroll = new JScrollPane();
    ButtonGroup group  = new ButtonGroup();

    this.setTitle("Select Horizon");
    this.getContentPane().setLayout(new BorderLayout());

    pnlTop.setLayout(new GridLayout(3,4));

    rb = new JRadioButton[_SYSTEMS];

    for (int i=0; i<_SYSTEMS; i++)
    {
      rb[i] = new JRadioButton();
      if (i==0) rb[i].setSelected(true);

      rb[i].setFont(new java.awt.Font("Dialog", 0, 11));
      rb[i].setText( SYSTEMS[i] );
      rb[i].addActionListener(this);

      group.add(rb[i]);
      pnlTop.add(rb[i], null);
	}

    pnlButtons.setBorder(BorderFactory.createEtchedBorder());

    btnSelect.setFont(new java.awt.Font("Dialog", 1, 11));
    btnSelect.setText("Select");
    btnSelect.addActionListener(this);

    btnCancel.setFont(new java.awt.Font("Dialog", 1, 11));
    btnCancel.setText("Cancel");
    btnCancel.addActionListener(this);

    pnl.setLayout(new BorderLayout());

    pTable = new stratListTable(stList);
    scroll = pTable.getScrollPane();

    // Attach Widgets to frame

    this.getContentPane().add(pnlTop,     BorderLayout.NORTH);

    this.getContentPane().add(pnlButtons, BorderLayout.SOUTH);
    pnlButtons.add(btnSelect,      null);
    pnlButtons.add(btnCancel,      null);

    this.getContentPane().add(pnl, BorderLayout.CENTER);
    pnl.add(scroll,                BorderLayout.CENTER);

    // Display the Frame

//    this.setSize(new Dimension(650, 350));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setSize(new Dimension(700, d.height-50));
//    this.setLocation((d.width  - this.getSize().width) / 2,
//                     (d.height - this.getSize().height) / 2);
    this.setLocation((d.width  - this.getSize().width) / 2, 5);

//    this.setResizable(false);
    this.setVisible(true);
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getData()
   *  <p> This method will return the Strat Unit Data Structure.
   *  @return stData = Strat Unit Data Structure.
   */

  public stratStruct getData()
  {
    stratStruct stData = null;

    if (pTable != null)
      stData = pTable.getRowData();

    return (stData);
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close this dialog and all other dialogs opened by
   *     this dialog
   */

  public void close()
  {
    notifier     = null;       // Observable notifier
    stList       = null;

    pTable       = null;
    btnSelect    = null;
    btnCancel    = null;

    if (rb != null)
    {
	  for (int i=0; i<_SYSTEMS; i++)
	    rb[i] = null;
	  rb = null;
	}

    dispose();
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    for (int i=0; i<_SYSTEMS; i++)
    {
	  if (event.getSource() == rb[i]) { pTable.setFilter( SYSTEMS[i] ); }
    }

    if (event.getSource() == btnSelect)
    {
  	  if (notifier != null)
  	    notifier.notifyObservers(new String("Table - State Stratigraphic Unit Selected"));
  	}

    if (event.getSource() == btnCancel) { close(); }
  }
}