/*
 * @iqstratIO_Files.java Version 1.0 12/06/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package iqstrat.io;

import java.io.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

import iqstrat.iqstratStruct;

import util.utilFileIO;
import util.utilImageIO;

/** Class iqstratIO_Files
 *  <p> This class will manage the plot output for the profile plot
 *
 *  @version 1.1 12/06/2012
 *  @author  John R. Victorine
 */

public class iqstratIO_Files
{
  /** Method createImage( iqstratStruct stStruct, BufferedImage image )
   * <p> this method will create an image from the Plot image
   *  @param stStruct = The Global IQSTRAT Data Structure
   *  @param image    = Profile Plot Buffered Image
   */

  public static void createImage( iqstratStruct stStruct, BufferedImage image )
  {
    String sOut  = "";
    String sPath = stStruct.sDirectory;

    sOut = sPath  + "/" + stStruct.sName + ".png";

    utilImageIO pImage = new utilImageIO( image );
    pImage.save( sOut );

    pImage.close();
    pImage = null;

    image = null;
    sOut  = null;
  }

  /** Method createHTML( iqstratHeadersStruct stHeader )
   * <p> this method will create an html file from the Plot image
   *  @param stHeader  = The Header Data Structure
   */

  public static void createHTML( iqstratStruct stStruct )
  {
    int           iError = util.utilFileIO.NOERROR;
    String        sOut   = stStruct.sName + ".html";
    String        sPath  = stStruct.sDirectory;
    String        sXML   = "";
    utilFileIO    pIO    = null;

    // Create HTML Document

    sXML = new String(sXML + "<html>\n");
    sXML = new String(sXML + "  <head>\n");
    sXML = new String(sXML + "    <title>Type Log Applet Cross Section Plot</title>\n");
    sXML = new String(sXML + "  </head>\n");

    sXML = new String(sXML + "  <body>\n");
    sXML = new String(sXML + "    <a href=\"http://www.kgs.ku.edu/PRS/Ozark/TYPE_LOG/PDF.html\"> " +
                             "Create a PDF Document of this PNG Image\n");
    sXML = new String(sXML + "    <p>\n");
    sXML = new String(sXML + "    <img src=\"" + sPath  + "\\" + stStruct.sName + ".png\">\n");
    sXML = new String(sXML + "  </body>\n");
    sXML = new String(sXML + "</html>\n");

    // Write HTML Document to Temporary File

    pIO  = new utilFileIO();
    pIO.Open(util.utilFileIO.DIRECTORY, util.utilFileIO.WRITE, sPath, sOut);
    pIO.Write(sXML);

    if (iError == util.utilFileIO.NOERROR)
      pIO.Close();

    pIO.delete();
    pIO = null;

    util.BrowserControl.displayURL(sPath + "\\" + sOut);

    sOut  = null;
    sPath = null;
    sXML  = null;
  }
}