/*
 * @iqstratTracksStruct.java Version 1.1 09/02/2007
 *
 * Copyright (c) 2007 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package iqstrat;

/** Class iqstratTracksStruct
 *  <p> This Class holds the Plot Tracks Data Structure.
 *
 *  @version 1.1 09/02/2007
 *  @author  John R. Victorine
 */

public class iqstratTracksStruct
{
//  public static final int PLOT_TITLES  = 100; // Plot Titles
//  public static final int DEPTH_TRACK  = 50;  // Depth Track Width
//  public static final int LABELSTART   = 25;  // Label Start

  public static final int PLOT_TITLES   = 210; // Plot Titles
  public static final int LABELSTART    = 135; // Label Start
  public static final int DEPTH_TRACK   = 50;  // Depth Track Width
  public static final int NO_SEPARATION = 5;   // no separation Width

  // Color Image Track Values

  // Resistivity/Porosity Track Macros

  public static final int _RT   = 0;
  public static final int _PHIT = 1;

  // -- Porosity

  public static final int _PU  = 16;
  public static final int PU[][] =
  { {204, 255, 204},
    {153, 255, 153},
    {102, 255, 102},
    { 51, 255,  51},
    {  0, 255,   0},
    {  0, 255, 153},
    { 51, 204, 153},
    {  0, 204, 255},
    { 51, 153, 255},
    {  0, 153, 255},
    {  0, 102, 204},
    { 51, 102, 204},
    {102, 102, 204},
    {102,  51, 204},
    {102,  51, 102},
    { 51,   0,  51}};

  // -- Conductivity

  public static final int _MMHOS_M  = 16;
  public static final int MMHOS_M[][] =
  { {255, 255, 204},
    {255, 255, 102},
    {255, 255,  51},
    {255, 255,   0},
    {255, 204, 102},
    {255, 204,  51},
    {255, 204,   0},
    {255, 153,  51},
    {255, 153,   0},
    {204, 102,   0},
    {204, 102,  51},
    {153,  51,   0},
    {153,  51,  51},
    {102,  51,   0},
    {102,  51,  51},
    { 51,   0,   0} };

  // Depth Scale Plot Variables

  public static final int    SCALE_1   = 0;   //   1 Feet / Inch
  public static final int    SCALE_2   = 1;   //   2 Feet / Inch
  public static final int    SCALE_5   = 2;   //   5 Feet / Inch
  public static final int    SCALE_10  = 3;   //  10 Feet / Inch
  public static final int    SCALE_20  = 4;   //  20 Feet / Inch
  public static final int    SCALE_50  = 5;   //  50 Feet / Inch
  public static final int    SCALE_100 = 6;   // 100 Feet / Inch
  public static final int    SCALE_200 = 7;   // 200 Feet / Inch

  public static final int SCALE[] = { 1, 2, 5, 10, 20, 50, 100, 200 };

  // Lithology/Texture Depth Scale Variables

  public static final int    _SCALE_HALF_FOOT = 0; // each cell is 1/2 ft deep
  public static final int    _SCALE_1_FOOT    = 1; // each cell is 1 ft deep
  public static final int    _SCALE_2_FOOT    = 2;
  public static final int    _SCALE_5_FOOT    = 3;
  public static final int    _SCALE_10_FOOT   = 4;
  public static final int    _SCALE_20_FOOT   = 5;

  public static final int    _SCALE_HALF_INCH = 6; // each cell is 1/2 in deep
  public static final int    _SCALE_1_INCH    = 7; // each cell is 1 in deep
  public static final int    _SCALE_2_INCH    = 8; // each cell is 2 in deep
  public static final int    _SCALE_QUARTER_FOOT = 9; // each cell is 1/4 ft deep
  public static final int    _SCALE_10TH_FOOT = 10; // each cell is 1/10 ft deep

  public static final double _SCALE_FACTOR[] =
      { 2.0, 1.0, 0.5, 0.2, 0.1, 0.05, 24.0, 12.0, 6.0, 4.0,  10.0};
  public static final double _SCALE[] =
    // 0.5 ft  1 ft   2 ft   5 ft   10 ft   20 ft  0.5 in   1 in    2 in
      { .5,    1.0,   2.0,   5.0,   10.0,   20.0, .04167, .08333, .16667, .25, .1 };

  // By Datum

  public static final int _BY_ELEV  = 0;
  public static final int _BY_DATUM = 1;
  public static final int _BY_LOG   = 2;

  // Source Data Types

  public static final int _SRC_LAS      = 0;
  public static final int _SRC_ROCK     = 1;
  public static final int _SRC_TOPS     = 2;
  public static final int _SRC_PFEFFER  = 3;
  public static final int _SRC_BRINE    = 4;
  public static final int _SRC_BIO      = 5;

  public static final int _SRC_ID       = 0;
  public static final int _SRC_COLUMN   = 1;

  public static final int TOTAL_SOURCES = 6;
  public static final String SOURCE[][] =
    {
     // SOURCE         Description
      { "LAS",     "Digital LAS File Curve Data" },
      { "ROCK",    "Rock Data" },
      { "TOPS",    "Horizons" },
      { "PfEFFER", "PfEFFER Data Tracks" },
      { "BRINE",   "Brine Data" },
      { "BIO",     "Bio Stratigraphy" },
    };

  public static final int COLORS[][] =  { { 204, 255, 153 },
                                          { 255, 204, 102 },
                                          { 204, 255, 255 },
                                          { 180, 200, 100 },
                                          { 221, 195, 200 },
                                          { 255, 140,   0 }};

  // Stratigraphic Track Types & Track Widths

  public static final String ICON = "ICON";

  public static final int TOTAL_TYPES = 15;
  public static final String TYPE[][] =
    {
    //   TYPE         Description                  Width
      { "LAS",     "LAS Plot Track Format",        "100" },
      { "LITH",    "Lithology Plot Track Format",   "25" },
      { "ROCK",    "Rock Lithology Column Format", "100" },
      { "PHI",     "Porosity Thin Track Format",    "30" },
      { "TOPS",    "Formation Tops Format",        "240" },
      { "STRAT",   "Formation Tops Format",        "300" },
      { "SEQ",     "Sequence Strat Format",         "25" },
      { "ENV",     "Depositional Environment",     "140" },
      { "GRAIN",   "Rock Texture",                 "140" },
      { "IMAGE",   "Rock Image Track Format",       "25" },
      { "ICON",    "Icon Image Track Format",      "150" },
      { "COLOR",   "Color Track Format",            "50" },
      { "DESC",    "Description Track Format",     "200" },
      { "TEXT",    "Abbreveation Track Format",     "50" },
      { "STRK",    "Lithology Streak Track Format", "20" },
      { "FLOW",    "PfEFFER Flow Units",           "100" },
    };

  // Colorlith Track Identifiers & Rock Column Identifiers

  public static final int _COLORLITH_RU   = 0;
  public static final int _COLORLITH_PD   = 1;
  public static final int _COLORLITH_SG   = 2;
  public static final int _LITHOLOGY_LAS  = 3;
  public static final int _LITHOLOGY_ROCK = 4;
  public static final int _COLORLITH_RN   = 5;

  // Thin Porosity Track Identifiers

  public static final int _THIN_NPHI      = 0;
  public static final int _THIN_NPHI_DPHI = 1;
  public static final int _THIN_SPHI      = 2;
  public static final int _THIN_DPHI      = 3;

  // Track Group Descriptions

  public static final int TOTAL_TRACKS_DESC = 9;
  public static final String TRACKS_DESC[][] =
  {
    { "LAS",     "Standard LAS Curve Format"  },
    { "CL",      "Color Image Track Curve Format"  },
    { "IMG",     "Rock Image"  },
    { "ROCK",    "Measured Rock Data"  },
    { "COMP",    "Computed Rock Data"  },
    { "OBS",     "Observational Rock Data"  },
    { "TOPS",    "Horizons" },
    { "PfEFFER", "PfEFFER Data" },
    { "BRINE",   "Measured Brine Data"  },
  };

  // Available Stratigraphic Plot Tracks

  public static final int _NONE                  = -1;
  // Gamma Lithology Track
  public static final int _LITHOLOGY_GR          = 0;
  // LAS Like Plot Tracks
  // -- Digital LAS File Tracks
  // .. Reference
  public static final int _LAS_REFERENCE         = 1;
  // .. Resistivity
  public static final int _LAS_INDUCTION         = 2;
  public static final int _LAS_MICRO_RESISTIVITY = 3;
  public static final int _LAS_RESISTIVITY       = 4;
  public static final int _LAS_LATERLOG          = 5;
  public static final int _LAS_NORMAL            = 6;
  public static final int _LAS_AHT               = 7;
  // .. Porosity
  public static final int _LAS_LITHO_DENSITY_3   = 8;
  public static final int _LAS_LITHO_DENSITY     = 9;
  public static final int _LAS_SONIC             = 10;
  // .. Temperature
  public static final int _LAS_TEMP              = 11;
  // .. Permeability
  public static final int _LAS_PERM              = 12;
  // .. Spectral Gamma Ray
  public static final int _LAS_SPECTRAL_GR       = 13;
  public static final int _LAS_SPECTRAL_RATIO    = 14;
  // .. Computed Data
  public static final int _LAS_RHOMAA_UMAA       = 15;
  public static final int _LAS_RHOMAA_NPHI       = 16;
  // .. Unknown Log Curves
  public static final int _LAS_LINEAR            = 17;
  public static final int _LAS_LOG               = 18;
  // .. Set Counts Limits
  public static final int _LAS_SET_GR_COUNTS     = 19;
  public static final int _LAS_SET_NEUT_COUNTS   = 20;
  // .. Colorlith
  public static final int _CLRLITH_RU            = 21;
  public static final int _CLRLITH_RN            = 22;
  // .. Rock Lithology Columns
  public static final int _ROCK_COLUMN           = 23;
  public static final int _ROCK_COLUMN_PD        = 24;
  // .. Porosity Thin track
  public static final int _THIN_PHI              = 25;
  // .. Porosity Image Track
  public static final int _POROSITY_IMAGER       = 26;
  public static final int _POROSITY_IMAGER_N     = 27;
  // .. Colorlith Spectral Gamma Ray
  public static final int _CLRLITH_SPGR          = 28;
  public static final int _CLRLITH_HALL          = 29;
  // .. Rock Texture by LAS Curves
  public static final int _LAS_TEXTURE           = 30;
  // .. Resistivity Image Track
  public static final int _RESISTIVITY_IMAGER    = 31;
  public static final int _RESISTIVITY_IMAGER_N  = 32;
  // Horizon Data Plot Tracks
  // Rock Observational Data Tracks
  // .. Rock Color
  public static final int _ROCK_COLOR            = 33;
  // .. Rock Lithology Columns
  public static final int _ROCK_MEASURED_SECTION = 34;
  // .. Sedimentary Structure
  public static final int _SED_STRUCTURE         = 35;
  // .. Fossils
  public static final int _ROCK_FOSSILS          = 36;
  // .. Porosity Type Tracks
  public static final int _ROCK_THIN_PHI         = 37;
  public static final int _ROCK_POROSITY         = 38;
  // .. Comments & Remark
  public static final int _REMARKS               = 39;
  // -- Stratigraphic Units
  public static final int _STRAT_UNITS_A         = 40;
  // TOTAL Plot Tracks
  public static final int TOTAL_TRACKS           = 41;

  // All Plot Tracks

  public static final int ALL_TRACKS_START       = _LITHOLOGY_GR;
  public static final int ALL_TRACKS_END         = _STRAT_UNITS_A;

  // LAS File Data Plot Tracks

  public static final int LAS_TRACKS_START       = _LITHOLOGY_GR;
  public static final int LAS_TRACKS_END         = _ROCK_COLOR;

  // Horizons Only Data Plot Tracks

  public static final int TOPS_TRACKS_START      = _STRAT_UNITS_A;
  public static final int TOPS_TRACKS_END        = _STRAT_UNITS_A;

  // Rock Data Plot Tracks

  public static final int ROCK_TRACKS_START      = _RESISTIVITY_IMAGER_N;
  public static final int ROCK_TRACKS_END        = _REMARKS;

  public static final int _GROUP       = 0;
  public static final int _SOURCE      = 1;
  public static final int _TYPE        = 2;
  public static final int _DESCRIPTION = 3;
  public static final int _STATUS      = 4;

  public static final String TRACK[][] =
  {
   // GROUP   SOURCE  TYPE    Description
    // - Gamma Lithology Track
    { "CL",  "LAS",  "LITH",  "Lithology - Gamma Ray",                    "YES"},
    // -- Digital LAS File Tracks
    // .. Reference
    { "LAS",  "LAS",  "LAS",  "LAS - Reference - GR,SP,CAL Logs",         "YES"},
    // .. Resistivity
    { "LAS",  "LAS",  "LAS",  "LAS - Induction Resistivity Logs",         "YES"},
    { "LAS",  "LAS",  "LAS",  "LAS - Micro Resistivity Logs",             "NO" },
    { "LAS",  "LAS",  "LAS",  "LAS - Resistivity Logs",                   "NO" },
    { "LAS",  "LAS",  "LAS",  "LAS - Laterlog Resistivity Logs",          "NO" },
    { "LAS",  "LAS",  "LAS",  "LAS - Normal Resistivity Logs",            "NO" },
    { "LAS",  "LAS",  "LAS",  "LAS - AHT Resistivity Logs",               "NO" },
    // .. Porosity
    { "LAS",  "LAS",  "LAS",  "LAS - Litho-Density - PE, NPHI, DPHI",     "NO" },
    { "LAS",  "LAS",  "LAS",  "LAS - Litho-Density - NPHI,RHOB,PE Logs",  "YES"},
    { "LAS",  "LAS",  "LAS",  "LAS - Sonic - SPHI,DT Logs",               "NO" },
    // .. Temperature
    { "LAS",  "LAS",  "LAS",  "LAS - Temperature Logs",                   "NO" },
    // .. Permeability
    { "LAS",  "LAS",  "LAS",  "LAS - Permeability",                       "NO" },
    // .. Spectral Gamma Ray
    { "LAS",  "LAS",  "LAS",  "LAS - Spectral Gamma Ray - Th,U,K Logs",   "NO" },
    { "LAS",  "LAS",  "LAS",  "LAS - Spectral Gamma Ray Ratios",          "NO" },
    // .. Computed Data
    { "LAS",  "LAS",  "LAS",  "LAS - Rhomaa-Umaa Computed Curves",        "NO" },
    { "LAS",  "LAS",  "LAS",  "LAS - Rhomaa-NPHI Curves",                 "NO" },
    // .. Unknown Log Curves
    { "LAS",  "LAS",  "LAS",  "LAS - Unknown Curves - Linear Track",      "NO" },
    { "LAS",  "LAS",  "LAS",  "LAS - Unknown Curves - Log Track",         "NO" },
    // .. Set Count Limits for Conversion
    { "LAS",  "LAS",  "LAS",  "*** Gamma Ray Counts to GR (API) ***",     "NO" },
    { "LAS",  "LAS",  "LAS",  "*** Neutron Counts to NPHI (PU) ***",      "NO" },
    // .. Colorlith
    { "CL",   "LAS",  "LITH", "Colorlith - Rhomaa-Umaa Track",            "YES"},
    { "CL",   "LAS",  "LITH", "Colorlith - Rhomaa-NPHI Track",            "NO" },
    // .. Rock Lithology Columns
    { "CL",   "LAS",  "ROCK", "Lithology - Rhomaa-Umaa Track",            "YES"},
    { "CL",   "LAS",  "ROCK", "Lithology - Rhomaa-NPHI Track",            "YES" },
    // .. Porosity Thin track
    { "CL",   "LAS",  "PHI",  "Thin Porosity Track",                      "YES"},
    // .. Porosity Image Track
    { "CL",   "LAS",  "LITH", "Colorlith - Porosity Imager",              "NO" },
    { "CL",   "LAS",  "LITH", "Colorlith - Porosity Imager Nonlinear",    "NO" },
    // .. Colorlith Spectral Gamma Ray
    { "CL",   "LAS",  "LITH", "Colorlith - Th/U - Th/K Track",            "NO" },
    { "CL",   "LAS",  "LITH", "Colorlith - Hall K-Th-U (RGB) Track",      "NO" },
    // .. Rock Texture by LAS Curves
    { "CL",   "LAS",  "GRAIN","Texture - by Gamma Ray",                   "YES"},
    // .. Resistivity Image Track
    { "CL",   "LAS",  "LITH", "Colorlith - Resistivity Imager",           "NO" },
    { "CL",   "LAS",  "LITH", "Colorlith - Resistivity Imager Nonlinear", "NO" },
    // -- Rock Observational Data Tracks
    // .. Rock Color
    { "OBS",  "ROCK", "COLOR","Color - Rock RGB Values",                  "YES"},
    // .. Rock Lithology Columns
    { "OBS",  "ROCK", "ROCK", "Lithology - Measured Sections",            "NO" },
    // .. Rock Texture
    // .. Sedimentary Structure
    { "OBS",  "ROCK", "ICON", "Sedimentary Structures",                   "YES"},
    // .. Fossils
    { "OBS",  "ROCK", "ICON", "Fossils",                                  "YES"},
    // .. Porosity Type Tracks
    { "OBS",  "ROCK", "PHI",  "Porosity Track",                           "YES"},
    { "OBS",  "ROCK", "PHI",  "Porosity Type",                            "YES"},
    // .. Comments & Remarks
    { "OBS",  "ROCK", "DESC", "Description",                              "YES"},
    // Horizon Data Plot Tracks
    // -- Stratigraphic Units
    { "TOPS", "TOPS", "STRAT","Horizons - Stratigraphic Units",           "YES" },
  };

  public static final int _GR    = las.lasStandardTools._GR;
  public static final int _CGR   = las.lasStandardTools._CGR;
  public static final int _SP    = las.lasStandardTools._SP;
  public static final int _CAL   = las.lasStandardTools._CAL;

  public static final int _PE    = las.lasStandardTools._PE;
  public static final int _RHOB  = las.lasStandardTools._RHOB;
  public static final int _DPHI  = las.lasStandardTools._DPHI;
  public static final int _DRHO  = las.lasStandardTools._DRHO;
  public static final int _NPHI  = las.lasStandardTools._NPHI;

  public static final int _DT    = las.lasStandardTools._DT;
  public static final int _SPHI  = las.lasStandardTools._SPHI;

  public static final int _COND  = las.lasStandardTools._COND;
  public static final int _CILD  = las.lasStandardTools._CILD;
  public static final int _CILM  = las.lasStandardTools._CILM;

  public static final int _RES   = las.lasStandardTools._RES;
  public static final int _RDEP  = las.lasStandardTools._RDEP;
  public static final int _RMED  = las.lasStandardTools._RMED;
  public static final int _RSHAL = las.lasStandardTools._RSHAL;

  public static final int _ILD   = las.lasStandardTools._ILD;
  public static final int _ILM   = las.lasStandardTools._ILM;
  public static final int _SFLU  = las.lasStandardTools._SFLU;

  public static final int _LL    = las.lasStandardTools._LL;
  public static final int _MLL   = las.lasStandardTools._MLL;
  public static final int _LL8   = las.lasStandardTools._LL8;

  public static final int _LN    = las.lasStandardTools._LN;
  public static final int _SN    = las.lasStandardTools._SN;

  public static final int _MNOR  = las.lasStandardTools._MNOR;
  public static final int _MSFL  = las.lasStandardTools._MSFL;
  public static final int _MINV  = las.lasStandardTools._MINV;

  public static final int _AHT10 = las.lasStandardTools._AHT10;
  public static final int _AHT20 = las.lasStandardTools._AHT20;
  public static final int _AHT30 = las.lasStandardTools._AHT30;
  public static final int _AHT60 = las.lasStandardTools._AHT60;
  public static final int _AHT90 = las.lasStandardTools._AHT90;

  public static final int _TH    = las.lasStandardTools._TH;
  public static final int _U     = las.lasStandardTools._U;
  public static final int _K     = las.lasStandardTools._K;

  public static final int _GRN   = las.lasStandardTools._GRN;
  public static final int _NEUT  = las.lasStandardTools._NEUT;

  public static final int _PERM  = las.lasStandardTools._PERM;

  public static final int _TEMP     = las.lasStandardTools._TEMP;
  public static final int _TEMPDIFF = las.lasStandardTools._TEMPDIFF;

  public static final int _LIN_1    = las.lasStandardTools._LIN_1;
  public static final int _LIN_2    = las.lasStandardTools._LIN_2;
  public static final int _LIN_3    = las.lasStandardTools._LIN_3;
  public static final int _LIN_4    = las.lasStandardTools._LIN_4;

  public static final int _LOG_1    = las.lasStandardTools._LOG_1;
  public static final int _LOG_2    = las.lasStandardTools._LOG_2;
  public static final int _LOG_3    = las.lasStandardTools._LOG_3;
  public static final int _LOG_4    = las.lasStandardTools._LOG_4;

  public static final int _RHOMAA   = las.lasStandardTools._RHOMAA;
  public static final int _UMAA     = las.lasStandardTools._UMAA;
  public static final int _PHIDIFF  = las.lasStandardTools._PHIDIFF;

  public static final int _THU      = las.lasStandardTools._THU;
  public static final int _THK      = las.lasStandardTools._THK;

  public static final int LAS_START       = _LAS_REFERENCE;
  public static final int LAS_TRACK_TOTAL = _LAS_SET_NEUT_COUNTS;

  public static final String LAS_TRACK_DESC[] = {
      "Reference",
      "Dual Induction",
      "Micro Resistivity",
      "Resistivity",
      "Laterlog",
      "Normal",
      "AHT Resistivity",
      "Neutron/Density",
      "Litho-Density",
      "Sonic",
      "Temperature",
      "Permeability",
      "Spectral GR",
      "Spectral GR Ratio",
      "Rhomaa-Umaa",
      "Rhomaa-NPHI",
      "Linear Track",
      "Semilog Track",
      "MODIFY GR",
      "MODIFY NPHI"
  };

  public static final int LAS_TRACK[][] =
  {
    {_GR,    _SP,       _CGR,    _CAL,   -1},
    {_ILD,   _ILM,      _LL8,    _SFLU,  -1},
    {_MNOR,  _MSFL,     _MINV,   -1,     -1},
    {_RES,   _RDEP,     _RMED,   _RSHAL, -1},
    {_LL,    _MLL,      _LL8,    -1,     -1},
    {_LN,    _SN,       -1,      -1,     -1},
    {_AHT10, _AHT30,    _AHT60,  _AHT90, -1},
    {_PE,    _NPHI,     _DPHI,   -1,     -1},
    {_PE,    _NPHI,     _RHOB,   _DPHI,  -1},
    {_SPHI,  _DT,       -1,      -1,     -1},
    {_TEMP,  _TEMPDIFF, -1,      -1,     -1},
    {_PERM,  -1,        -1,      -1,     -1},
    {_U,     _K,        _TH,     -1,     -1},
    {_THU,   _THK,      -1,      -1,     -1},
    {_RHOMAA,_UMAA,     -1,      -1,     -1},
    {_RHOMAA,_NPHI,     _PHIDIFF,-1,     -1},
    {_LIN_1, _LIN_2,    _LIN_3,  _LIN_4, -1},
    {_LOG_1, _LOG_2,    _LOG_3,  _LOG_4, -1},
    {_GR,    _GRN,      -1,      -1,     -1},
    {_NPHI,  _NEUT,     -1,      -1,     -1},
  };

  public static final double GAMMA_MAX  = 150.0;
  public static final double GAMMA_MIN  =   0.0;

  public static final double UMAA_MAX   = 13.8;
  public static final double UMAA_MIN   =  4.3;

  public static final double RHOMAA_MAX = 2.87;
  public static final double RHOMAA_MIN = 2.6;

  public static final double PHI_MAX    =  0.5;//0.3;
  public static final double PHI_MIN    =  0.0;

  public static final double DPHI_MAX   =  .08;
  public static final double DPHI_MIN   = -.08;

  public static final double THU_MAX    = 24.5;
  public static final double THU_MIN    =  0.57;

  public static final double THK_MAX    = 41.14;
  public static final double THK_MIN    =  1.02;

  public static final double U_MAX      = 30.0; // 0 to 40
  public static final double U_MIN      =  0.0;

  public static final double TH_MAX     = 20.0; // 0 to 30
  public static final double TH_MIN     =  0.0;

  public static final double K_MAX      = 5.0;  // 0 to 5
  public static final double K_MIN      = 0.0;

  public static final double MMHO_MIN   =    0.0;
  public static final double MMHO_MAX   = 2000.0;

  public static final double PHIT_MIN   = 0.0;
  public static final double PHIT_MAX   = 0.3;
}

/*
 *  @version 1.1 09/02/2007
 *  @author  John Victorine
 */

