/*
 * @kgsIORead.java Version 1.1 10/19/2007
 *
 * Copyright (c) 2007 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package kgs.io;

import java.awt.*;

import xml.ReadLASFileXMLFile;
import cmn.cmnLASFileListStruct;

import horizon.regions.io.ReadRegionsXMLFile;
import horizon.regions.regionsListStruct;

import horizon.strat.io.ReadStratUnitsXMLFile;
import horizon.strat.stratListStruct;

/** Class kgsIORead
 *  <p> This Class will retrieve the Oil & Gas Well Data from the KGS Database.
 *      This class will retrieve the following information,
 *
 *      - LAS File Information by API-Number
 *      - Formation Regions, Tops, Perforation zones, DST Measurements
 *      - Measured Core Data
 *      - Core Images
 *
 *  @version 1.1 10/19/2007
 *  @author  John R. Victorine
 */

public class kgsIORead
{
  /** Constructor kgsIORead()
   *  <p> This is the Constructor for this class.
   */

  public kgsIORead() { }

  /* ===================================================================== *
   * ---------------------------- XML Methods ---------------------------- *
   * ===================================================================== */

  /** Method ReadLASFileInformation()
   * <p> This method will retrieve the LAS File Information
   *     for a specific well from the database
   * @param  sAPI    = API-Number of the Well.
   * @param  iMethod = The Method used in retrieving the XML
   * @return st  = The LAS File Information Data
   */

  public static cmnLASFileListStruct ReadLASFileInformation(String sAPI,
                                                            int    iMethod)
  {
    cmnLASFileListStruct st = new cmnLASFileListStruct();

    ReadLASFileXMLFile pXML = new ReadLASFileXMLFile(iMethod);
    st = pXML.Process(cmn.cmnStruct.KGS_LAS_FILES + "sAPI=" + sAPI);

    return (st);
  }

  /** Method ReadLASFileInformation2()
   * <p> This method will retrieve the LAS File Information
   *     for a specific well from the database
   * @param  sAPI    = API-Number of the Well.
   * @param  iMethod = The Method used in retrieving the XML
   * @return st  = The LAS File Information Data
   */

  public static cmnLASFileListStruct ReadLASFileInformation2(String sKID,
                                                             int    iMethod)
  {
    cmnLASFileListStruct st = new cmnLASFileListStruct();

    ReadLASFileXMLFile pXML = new ReadLASFileXMLFile(iMethod);
    st = pXML.Process(cmn.cmnStruct.KGS_LAS_FILES + "sKID=" + sKID);

    return (st);
  }

  /** Method ReadLASFileInformationOC()
   * <p> This method will retrieve the LAS File Information
   *     for a specific well from the database
   * @param  sAPI    = API-Number of the Well.
   * @param  iMethod = The Method used in retrieving the XML
   * @return st  = The LAS File Information Data
   */

  public static cmnLASFileListStruct ReadLASFileInformationOC(String sKID,
                                                              int    iMethod)
  {
    cmnLASFileListStruct st = new cmnLASFileListStruct();

    ReadLASFileXMLFile pXML = new ReadLASFileXMLFile(iMethod);
    st = pXML.Process(cmn.cmnStruct.TYPE_LOG_LAS_FILES + "sKID=" + sKID);

    return (st);
  }

  /** Method ReadWellRegions()
   * <p> This method will retrieve the Well Regions for a specific well
   * @param  sAPI    = API-Number of the Well.
   * @param  iMethod = The Method used in retrieving the XML
   * @return st      = The Regions List Data Structure
   */

  public static regionsListStruct ReadWellRegions(String sAPI, int iMethod)
  {
    regionsListStruct st = null;

    ReadRegionsXMLFile pXML = new ReadRegionsXMLFile(iMethod);
    st = pXML.Process(cmn.cmnStruct.KGS_REGIONS + "sAPI="+sAPI);

    return (st);
  }

  /** Method ReadByPartialName()
   * <p> This method will retrieve Zeller Tops by Partial Phrase
   * @param  sName   = Partial Tops Name
   * @param  iMethod = The Method used in retrieving the XML
   * @return st      = The Stratigraphic Units List Data Structure
   */

  public static stratListStruct ReadByPartialName(String sName, int iMethod)
  {
    stratListStruct st = null;

    ReadStratUnitsXMLFile pXML = new ReadStratUnitsXMLFile(iMethod);
    st = pXML.Process( cmn.cmnStruct.GET_TOPS_XML + "sName="+sName );

    return (st);
  }

  /** Method ReadWellRegions()
   * <p> This method will retrieve the Well Stratigraphic Units for a
   *     specific well
   * @param  source  = Tops Source.
   * @param  sAPI    = API-Number of the Well.
   * @param  iMethod = The Method used in retrieving the XML
   * @return st      = The Stratigraphic Units List Data Structure
   */

  public static stratListStruct ReadWellRegions( String source,
                                                 String sAPI,
                                                 int iMethod )
  {
    stratListStruct st    = null;
    stratListStruct stKGS = null;

    ReadStratUnitsXMLFile pXML = new ReadStratUnitsXMLFile(iMethod);
    st = pXML.Process( cmn.cmnStruct.TYPE_LOG_SOURCE_TOPS +
                       "src="+source+"&sAPI="+sAPI);

//    stKGS = ReadKGSStratUnits( iMethod );
//    st    = horizon.strat.stratUtility.fillBaseDepth(st, stKGS);

    return (st);
  }

  /** Method ReadWellRegions()
   * <p> This method will retrieve the Well Stratigraphic Units for a
   *     specific well
   * @param  sKID    = Well Header KID.
   * @param  iMethod = The Method used in retrieving the XML
   * @return st      = The Stratigraphic Units List Data Structure
   */

  public static stratListStruct ReadWellRegions0( String sKID, int iMethod )
  {
    stratListStruct st    = null;
    stratListStruct stKGS = null;

    ReadStratUnitsXMLFile pXML = new ReadStratUnitsXMLFile(iMethod);
    st = pXML.Process( cmn.cmnStruct.TYPE_LOG_SOURCE_TOPS + "sKID=" + sKID);

    return (st);
  }

  /** Method ReadWellRegions2()
   * <p> This method will retrieve the Well Stratigraphic Units for a
   *     specific well
   * @param  sZeller = Zeller KID.
   * @param  sKID    = Well Header KID.
   * @param  iMethod = The Method used in retrieving the XML
   * @return st      = The Stratigraphic Units List Data Structure
   */

  public static stratListStruct ReadWellRegions2( String sZeller,
                                                  String sKID,
                                                  int iMethod )
  {
    stratListStruct st    = null;
    stratListStruct stKGS = null;

    ReadStratUnitsXMLFile pXML = new ReadStratUnitsXMLFile(iMethod);
    st = pXML.Process( cmn.cmnStruct.TYPE_LOG_SOURCE_TOPS +
                       "src="+sZeller+"&sKID="+sKID);
//System.out.println(cmn.cmnStruct.TYPE_LOG_SOURCE_TOPS + "src="+sZeller+"&sKID="+sKID);

    return (st);
  }

  /** Method ReadWellEvalRegions2()
   * <p> This method will retrieve the Evaluated Well Stratigraphic
   *     Units for a specific well
   * @param  sZeller = Zeller KID.
   * @param  sKID    = Well Header KID.
   * @param  iMethod = The Method used in retrieving the XML
   * @return st      = The Stratigraphic Units List Data Structure
   */

  public static stratListStruct ReadWellEvalRegions2( String sZeller,
                                                      String sKID,
                                                      int iMethod )
  {
    stratListStruct st    = null;
    stratListStruct stKGS = null;

    ReadStratUnitsXMLFile pXML = new ReadStratUnitsXMLFile(iMethod);
    st = pXML.Process( cmn.cmnStruct.TYPE_LOG_EVAL_TOPS2 +
                       "sZeller="+sZeller+"&sKID="+sKID);
//System.out.println(cmn.cmnStruct.TYPE_LOG_EVAL_TOPS2 + "sZeller="+sZeller+"&sKID="+sKID);
    return (st);
  }

  /** Method ReadKGSStratUnits()
   * <p> This method will retrieve the Stratigraphic Units for Kansas
   * @param  iMethod = The Method used in retrieving the XML
   * @return st      = The Stratigraphic Units List Data Structure
   *

  public static stratListStruct ReadKGSStratUnits( int iMethod )
  {
    stratListStruct st = null;

    ReadStratUnitsXMLFile pXML = new ReadStratUnitsXMLFile(iMethod);
    st = pXML.Process(cmn.cmnStruct.KGS_STRAT_UNITS);

    return (st);
  }
*/
}