/*
 * @kgsIOUser.java Version 1.0 10/05/2007
 *
 * Copyright (c) 2007 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package kgs.io;

import java.io.*;
import java.awt.*;
import javax.swing.*;

import kgs.kgsUserStruct;
import kgs.kgsUserListStruct;
import kgs.io.ReadUsersXMLFile;

/** Class kgsIOUser
 *  <p> This class will manage the User Login Data
 *
 *  @version 1.1 10/05/2007
 *  @author  John R. Victorine
 */

public class kgsIOUser
{
  /** Method getHomeDir()
   * <p> This method will retrieve the User's Home Directory
   * @return sDirectory = The User Home Directory
   */

  public static String getHomeDir()
  {
    String sDir = "";    // User Home Directory
    File   pDir = null;  // User Home Directory File Object

    try
    {
      pDir = new File( System.getProperty( "user.home" ) );
      sDir = new String( pDir.getCanonicalPath() );
    }
    catch (Exception e)
    {
      e.printStackTrace();
      String sError = new String("kgsIOUser.getHomeDir()\n "+e.toString());
      JOptionPane.showMessageDialog(
        (Component) null, sError, "ERROR", JOptionPane.ERROR_MESSAGE);
    }

    return (sDir);
  }

  /** Method login()
   * <p> This method will log into the KGS for validation of User.
   * @param  sEmail    = the user email address
   * @return stUser    = The User Data Structure
   */

  public static kgsUserStruct login(String sEmail)
  {
	kgsUserStruct     stUser = null;
    String            sPath  = cmn.cmnStruct.TYPE_LOG_USER + "sEmail=" + sEmail;
    ReadUsersXMLFile  pUser  = new ReadUsersXMLFile(kgs.io.ReadUsersXMLFile.URL);
    kgsUserListStruct st     = pUser.Process(sPath);

    if (st != null)
    {
	  if (st.iCount > 0)
        stUser = st.stItem[0];
    }

    return (stUser);
  }

  /** Method getSource()
   * <p> This method will log into the KGS for validation of User.
   * @param  sKEY    = the user KEY
   * @return stUser  = The User Data Structure
   */

  public static kgsUserStruct getSource(String sKEY)
  {
	kgsUserStruct     stUser = null;
    String            sPath  = cmn.cmnStruct.TYPE_LOG_USER + "sKEY=" + sKEY;
    ReadUsersXMLFile  pUser  = new ReadUsersXMLFile(kgs.io.ReadUsersXMLFile.URL);
    kgsUserListStruct st     = pUser.Process(sPath);
//System.out.println(sPath);

    if (st != null)
    {
	  if (st.iCount > 0)
        stUser = st.stItem[0];
    }

    return (stUser);
  }

  /** Method getTopsSource()
   * <p> This method will retrieve the User information for Well API-Number.
   * @param  sAPI   = the user Well API-Number
   * @return stUser = The User Data List Structure
   */

  public static kgsUserListStruct getTopsSource(String sAPI)
  {
    String            sPath  = cmn.cmnStruct.TYPE_LOG_USER + "sAPI=" + sAPI;
    ReadUsersXMLFile  pUser  = new ReadUsersXMLFile(kgs.io.ReadUsersXMLFile.URL);
    kgsUserListStruct stUser = pUser.Process(sPath);
//System.out.println(sPath);

    return (stUser);
  }

  /** Method getWellTopsSource()
   * <p> This method will retrieve the User information for Well Header KID.
   * @param  sKID   = the user Well Header KID
   * @return stUser = The User Data List Structure
   */

  public static kgsUserListStruct getWellTopsSource(String sKID)
  {
    String            sPath  = cmn.cmnStruct.TYPE_LOG_USER + "sKID=" + sKID;
    ReadUsersXMLFile  pUser  = new ReadUsersXMLFile(kgs.io.ReadUsersXMLFile.URL);
    kgsUserListStruct stUser = pUser.Process(sPath);
//System.out.println(sPath);

    return (stUser);
  }
}