/*
 * @kgsIOWrite.java Version 1.1 11/29/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package kgs.io;

import java.awt.*;
import java.awt.*;
import javax.swing.*;

import iqstrat.io.ReadErrorXMLFile;

/** Class kgsIOWrite
 *  <p> This Class will retrieve the Oil & Gas Well Data from the KGS Database.
 *      This class will retrieve the following information,
 *
 *      - LAS File Information by API-Number
 *      - Formation Regions, Tops, Perforation zones, DST Measurements
 *      - Measured Core Data
 *      - Core Images
 *
 *  @version 1.1 11/29/2012
 *  @author  John R. Victorine
 */

public class kgsIOWrite
{
  /** Constructor kgsIOWrite()
   *  <p> This is the Constructor for this class.
   */

  public kgsIOWrite() { }

  /** Method SaveTopsSourceData()
   * <p> This method will save the Tops Source Data to the Database
   * @param  sKID    = Well Header KID.
   * @param  src     = User Source KID (Owner of Top).
   * @param  sZeller = Tops Zeller KID.
   * @param  sName   = Name of Horizon.
   * @param  dTop    = Top Depth.
   * @param  dBase   = Base Depth.
   */

  public static void SaveTopsSourceData( String sKID, String src, String sZeller,
                                         String sName, double dTop, double dBase )
  {
    String sPath  = "";
    String sError = "";

    ReadErrorXMLFile pXML =
        new ReadErrorXMLFile(iqstrat.io.ReadErrorXMLFile.URL);

    sPath = cmn.cmnStruct.TYPE_LOG_SOURCE_SAVE +
              "sKID="     + sKID +
              "&src="     + src +
              "&sZeller=" + sZeller +
              "&sName="   + sName +
              "&dTop="    + dTop +
              "&dBase="   + dBase;
    sError = pXML.Process(sPath);

    if (sError.length() > 0)
      JOptionPane.showMessageDialog(
          (Component) null, sError, "ERROR", JOptionPane.ERROR_MESSAGE);
    else
      JOptionPane.showMessageDialog(
          (Component) null, "Data Successfully Saved",
          "INFORMATION", JOptionPane.INFORMATION_MESSAGE);
  }


  /** Method DeleteTopsSourceData()
   * <p> This method will save the Tops Source Data to the Database
   * @param  sKID    = Well Header KID.
   * @param  src     = User Source KID.
   * @param  sZeller = Tops Zeller KID.
   */

  public static void DeleteTopsSourceData( String sKID, String src, String sZeller )
  {
    String sPath  = "";
    String sError = "";

    ReadErrorXMLFile pXML =
        new ReadErrorXMLFile(iqstrat.io.ReadErrorXMLFile.URL);

    sPath = cmn.cmnStruct.TYPE_LOG_SOURCE_DELETE +
              "sKID="     + sKID +
              "&src="     + src +
              "&sZeller=" + sZeller;
    sError = pXML.Process(sPath);

    if (sError.length() > 0)
      JOptionPane.showMessageDialog(
          (Component) null, sError, "ERROR", JOptionPane.ERROR_MESSAGE);
    else
      JOptionPane.showMessageDialog(
          (Component) null, "Data Successfully Deleted",
          "INFORMATION", JOptionPane.INFORMATION_MESSAGE);
  }

  /** Method SaveTopsSourceData()
   * <p> This method will save the Tops Source Data to the Database
   * @param  sKID    = Well Header KID.
   * @param  src     = User Source KID as Evaluator (NOT OWNER of Top).
   * @param  sZeller = Tops Zeller KID.
   * @param  sName   = Name of Horizon.
   * @param  dTop    = Top Depth.
   * @param  dBase   = Base Depth.
   * @param  sEval   = The Evaluators Response: ACCEPT or REJECT
   */

  public static void SaveTopsEvalData( String sKID, String src, String sZeller,
                                       String sName, double dTop, double dBase, String sEval )
  {
    String sPath  = "";
    String sError = "";

    ReadErrorXMLFile pXML =
        new ReadErrorXMLFile(iqstrat.io.ReadErrorXMLFile.URL);

    sPath = cmn.cmnStruct.TYPE_LOG_EVAL_SAVE +
              "sKID="     + sKID +
              "&src="     + src +
              "&sZeller=" + sZeller +
              "&sName="   + sName +
              "&dTop="    + dTop +
              "&dBase="   + dBase +
              "&sEval="   + sEval;
    sError = pXML.Process(sPath);

    if (sError.length() > 0)
      JOptionPane.showMessageDialog(
          (Component) null, sError, "ERROR", JOptionPane.ERROR_MESSAGE);
    else
      JOptionPane.showMessageDialog(
          (Component) null, "Data Successfully Saved",
          "INFORMATION", JOptionPane.INFORMATION_MESSAGE);
  }


  /** Method DeleteTopsEvalData()
   * <p> This method will save the Tops Source Data to the Database
   * @param  sKID    = Well Header KID.
   * @param  src     = User Source KID.
   * @param  sZeller = Tops Zeller KID.
   */

  public static void DeleteTopsEvalData( String sKID, String src, String sZeller )
  {
    String sPath  = "";
    String sError = "";

    ReadErrorXMLFile pXML =
        new ReadErrorXMLFile(iqstrat.io.ReadErrorXMLFile.URL);

    sPath = cmn.cmnStruct.TYPE_LOG_EVAL_DELETE +
              "sKID="     + sKID +
              "&src="     + src +
              "&sZeller=" + sZeller;
    sError = pXML.Process(sPath);

    if (sError.length() > 0)
      JOptionPane.showMessageDialog(
          (Component) null, sError, "ERROR", JOptionPane.ERROR_MESSAGE);
    else
      JOptionPane.showMessageDialog(
          (Component) null, "Data Successfully Deleted",
          "INFORMATION", JOptionPane.INFORMATION_MESSAGE);
  }
}