/*
 * @lasToolsTable.java Version 1.1 06/20/2008
 *
 * Copyright (c) 2007 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package las.gui;

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;

import gui.guiTable;

/** Class lasToolsTable
 *  <p> This Class will display the Standard Tools Table.
 *
 *  @version 1.1 06/20/2008
 *  @author  John R. Victorine
 */

public class lasToolsTable
{
  private int      iRows       = 0;
  private int      iColumns    = 5;
  private String   sColumn[]   = { "Mnemonic",
                                   "Description",
                                   "Units",
                                   "Minimum",
                                   "Maximum" };
  private Object   oData[][]   = null;
  private guiTable pTable      = null;

  public static final int _LAS_TYPE    = 0;
  public static final int _CORE_TYPE   = 1;
  public static final int _TOPS_TYPE   = 2;
  public static final int _CNTRL_TYPE  = 4;
  public static final int _GEO_TYPE    = 13;

  private int iData = _LAS_TYPE;

  /** Constructor lasToolsTable()
   *  <p> This is the Constructor for this class.
   *  @param iData    = type of data to display
   */

  public lasToolsTable( int iData )
  {
    this.iData = iData;

    populateList();

    if (iRows > 0)
      pTable = new guiTable(0, iRows, iColumns, sColumn, oData);
    else
      pTable = new guiTable(0, iColumns, sColumn);

    pTable.resizeColumn(0, 8);
    pTable.resizeColumn(2, 8);
    pTable.resizeColumn(3, 8);
    pTable.resizeColumn(4, 8);

    if (iRows > 0)
      pTable.setSelectedRow(0);
  }

  /** Method close()
   * <p> This method will force java to allocate its memory
   */

  public void close()
  {
    sColumn = null;
    oData   = null;
    pTable  = null;
  }

  /** Method populateList
   *  <p> This method will load the Data Array for the guiTable.
   */

  private void populateList()
  {
    int    iTotal = 0;
    String sKID   = "";
    String sMnem  = "";
    String sUnit  = "";
    String sDesc  = "";
    double dMin   = 0.0;
    double dMax   = 0.0;

    if (iData == _LAS_TYPE)
    {
      iTotal = las.lasStandardTools.TOTAL_LAS;
    }
    else if (iData == _CORE_TYPE)
    {
      iTotal = rock.rockStandardTools.TOTAL_DATA;
    }
    else if (iData == _TOPS_TYPE)
    {
      iTotal = horizon.strat.stratStandardTools.TOTAL;
    }
    else if (iData == _CNTRL_TYPE)
    {
      iTotal = iqstrat.iqstratControlStandardTools.TOTAL;
    }
    else if (iData == _GEO_TYPE)
    {
      iTotal = iqstrat.iqstratRemarkStandardTools.TOTAL;
    }

    iRows = 0;
    oData = new Object[iTotal][iColumns];

    for (int i=0; i<iTotal; i++)
    {
      switch (iData)
      {
        case _LAS_TYPE:
          sKID  = new String(las.lasStandardTools.LAS_TOOLS[i][0]);
          sMnem = new String(las.lasStandardTools.LAS_TOOLS[i][1]);
          sDesc = new String(las.lasStandardTools.LAS_TOOLS[i][2]);
          sUnit = new String(las.lasStandardTools.LAS_TOOLS[i][3]);
          dMin  = cmn.cmnString.stringToDouble(
                    las.lasStandardTools.LAS_TOOLS[i][4]);
          dMax  = cmn.cmnString.stringToDouble(
                    las.lasStandardTools.LAS_TOOLS[i][5]);
          break;
        case _CORE_TYPE:
          sKID  = new String(rock.rockStandardTools.ROCK_TOOLS[i][6]);
          sMnem = new String(rock.rockStandardTools.ROCK_TOOLS[i][0]);
          sDesc = new String(rock.rockStandardTools.ROCK_TOOLS[i][1]);
          sUnit = new String(rock.rockStandardTools.ROCK_TOOLS[i][2]);
          dMin  = cmn.cmnString.stringToDouble(
                    rock.rockStandardTools.ROCK_TOOLS[i][4]);
          dMax  = cmn.cmnString.stringToDouble(
                    rock.rockStandardTools.ROCK_TOOLS[i][5]);
          break;
        case _TOPS_TYPE:
          sKID  = new String(horizon.strat.stratStandardTools.CURVES[i][0]);
          sMnem = new String(horizon.strat.stratStandardTools.CURVES[i][1]);
          sDesc = new String(horizon.strat.stratStandardTools.CURVES[i][2]);
          sUnit = new String(horizon.strat.stratStandardTools.CURVES[i][3]);
          dMin  = 0.0;
          dMax  = 0.0;
          break;
        case _CNTRL_TYPE:
          sKID  = new String(iqstrat.iqstratControlStandardTools.CURVES[i][0]);
          sMnem = new String(iqstrat.iqstratControlStandardTools.CURVES[i][1]);
          sDesc = new String(iqstrat.iqstratControlStandardTools.CURVES[i][2]);
          sUnit = new String("");
          dMin  = 0.0;
          dMax  = 0.0;
          break;
        case _GEO_TYPE:
          sKID  = new String(iqstrat.iqstratRemarkStandardTools.CURVES[i][0] );
          sMnem = new String(iqstrat.iqstratRemarkStandardTools.CURVES[i][1] );
          sDesc = new String(iqstrat.iqstratRemarkStandardTools.CURVES[i][2] );
          sUnit = new String(iqstrat.iqstratRemarkStandardTools.CURVES[i][3] );
          dMin  = 0.0;
          dMax  = 0.0;
          break;
      }

      oData[i][0] = new String( sMnem );
      oData[i][1] = new String( sDesc );
      oData[i][2] = new String( sUnit );
      oData[i][3] = new Double( dMin );
      oData[i][4] = new Double( dMax );

      iRows++;
    }
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getSelectedRow()
   *  <p> This method will return the selected row.
   *  @return Selected Row.
   */

  public int getSelectedRow() { return (pTable.getSelectedRow()); }

  /** Method getTotalRows()
   *  <p> This method will return the total number of rows.
   *  @return iRows - Total Rows.
   */

  public int getTotalRows() { return (iRows); }

  /** Method getScrollPane()
   *  <p> This method will return the scroll pane.
   *  @return jScrollPane - Scroll Pane containing the Table.
   */

  public JScrollPane getScrollPane() { return (pTable.getScrollPane()); }
}
