/*
 * @lasStandardTools.java Version 1.1 09/05/2007
 *
 * Copyright (c) 2007 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package las;

/** Class lasStandardTools
 *  <p> This Class holds the standard tool information to map the LAS log
 *      curves to standard names for plotting.
 */

public class lasStandardTools
{
  public static final int _GR       = 0;
  public static final int _CGR      = 1;
  public static final int _SP       = 2;
  public static final int _CAL      = 3;

  public static final int _PE       = 4;
  public static final int _RHOB     = 5;
  public static final int _DRHO     = 6;
  public static final int _DPHI     = 7;
  public static final int _NPHI     = 8;

  public static final int _SPHI     = 9;
  public static final int _DT       = 10;

  public static final int _COND     = 11;
  public static final int _CILD     = 12;
  public static final int _CILM     = 13;

  public static final int _RES      = 14;
  public static final int _RDEP     = 15;
  public static final int _RMED     = 16;
  public static final int _RSHAL    = 17;

  public static final int _ILD      = 18;
  public static final int _ILM      = 19;
  public static final int _SFLU     = 20;

  public static final int _LL       = 21;
  public static final int _MLL      = 22;
  public static final int _LL8      = 23;

  public static final int _LN       = 24;
  public static final int _SN       = 25;

  public static final int _MNOR     = 26;
  public static final int _MSFL     = 27;
  public static final int _MINV     = 28;

  public static final int _AHT10    = 29;
  public static final int _AHT20    = 30;
  public static final int _AHT30    = 31;
  public static final int _AHT60    = 32;
  public static final int _AHT90    = 33;

  public static final int _TH       = 34;
  public static final int _U        = 35;
  public static final int _K        = 36;

  public static final int _GRN      = 37;
  public static final int _NEUT     = 38;

  public static final int _PERM     = 39;

  public static final int _TEMP     = 40;
  public static final int _TEMPDIFF = 41;

  public static final int _LIN_1    = 42;
  public static final int _LIN_2    = 43;
  public static final int _LIN_3    = 44;
  public static final int _LIN_4    = 45;

  public static final int _LOG_1    = 46;
  public static final int _LOG_2    = 47;
  public static final int _LOG_3    = 48;
  public static final int _LOG_4    = 49;

  public static final int _RHOMAA   = 50;
  public static final int _UMAA     = 51;
  public static final int _PHIDIFF  = 52;

  public static final int _THU      = 53;
  public static final int _THK      = 54;

  public static final int TOTAL_LAS = 50;
  public static final int TOTAL     = 55;
  public static final int TOTAL_RES = 20;
  public static final int TOTAL_PHI =  3;

  public static final String LAS_TOOLS[][] =
  {
    {"1022012534",  "GR",   "Gamma Ray",             "API",     "0", "150"},
    {"1022012535",  "CGR",  "Gamma Ray Minus Uranium", "API", "0", "150"},
    {"1022012507",  "SP",   "Spontaneous Potential", "MV",      "0", "1"},
    {"1022012517",  "CAL",  "Caliper",               "IN",      "6", "12"},

    {"1022012598",  "PE",   "Photoelectric factor",   "BARNS/E","0", "20"},
    {"1022012499",  "RHOB", "Bulk Density",           "GM/CC",  "2", "3"},
    {"1022012501",  "DRHO", "Bulk Density Correction","GM/CC",  "-1.5", ".5"},
    {"1022012500",  "DPHI", "Density porosity",       "PU",     "-.1", ".3"},
    {"1022012503",  "NPHI", "Neutron porosity",       "PU",     "-.1", ".3"},

    {"1022012505",  "SPHI", "Sonic porosity",         "PU",     "-.1", ".3"},
    {"1022012504",  "DT",   "Acoustic transit time",  "USEC/FT","40", "140"},

    {"1022012508",  "COND", "Conductivity",                "MMHO/M", "0", "2000"},
    {"1022012509",  "CILD", "Deep Induction Conductivity", "MMHO/M", "0", "2000"},
    {"1022012510",  "CILM", "Medium Induction Conductivity","MMHO/M", "0", "2000"},

    {"1022012511",  "RES",  "Resistivity",          "OHM-M", ".1", "1000"},
    {"1022012600",  "RDEP", "Deep Resistivity",     "OHM-M", ".1", "1000"},
    {"1022012601",  "RMED", "Medium Resistivity",   "OHM-M", ".1", "1000"},
    {"1022012599",  "RSHAL","Shallow Resistivity",  "OHM-M", ".1", "1000"},

    {"1022012512",  "ILD",  "Deep Induction Resistivity", "OHM-M", ".1", "1000"},
    {"1022012513",  "ILM",  "Medium Induction Resistivity", "OHM-M", ".1", "1000"},
    {"1022012519",  "SFLU", "Spherically Focused Resistivity","OHM-M", ".1", "1000"},

    {"1022012520",  "LL",   "Deep Laterolog Resistivity",  "OHM-M", ".1", "1000"},
    {"1022012522",  "MLL",  "Micro Laterolog Resistivity", "OHM-M", ".1", "1000"},
    {"1022012515",  "LL8",  "Shallow Laterolog Resistivity","OHM-M", ".1", "1000"},

    {"1022012518",  "LN",   "Long Normal Resistivity",   "OHM-M", ".1", "1000"},
    {"1022012516",  "SN",   "Shallow Normal Resistivity","OHM-M", ".1", "1000"},

    {"1022012523",  "MNOR", "Micro Normal Resistivity", "OHM-M", ".1", "1000"},
    {"1022012524",  "MSFL", "Micro Spherically Focused Resistivity", "OHM-M", ".1", "1000"},
    {"1022012521",  "MINV", "Micro Inverse Resistivity", "OHM-M", ".1", "1000"},

    {"1022012514a", "AHT10", "Array Induction Resistivity-10", "OHM-M", ".1", "1000"},
    {"1022012514b", "AHT20", "Array Induction Resistivity-20", "OHM-M", ".1", "1000"},
    {"1022012514c", "AHT30", "Array Induction Resistivity-30", "OHM-M", ".1", "1000"},
    {"1022012514d", "AHT60", "Array Induction Resistivity-60", "OHM-M", ".1", "1000"},
    {"1022012514e", "AHT90", "Array Induction Resistivity-90", "OHM-M", ".1", "1000"},

    {"1022012496",  "THOR", "Thorium Concentration",       "PPM",       "-10", "30"},
    {"1022012497",  "URAN", "Uranium Concentration",       "PPM",       "0", "40"},
    {"1022012498",  "POTA", "Potassium Concentration",     "%",         "-10", "5"},

    {"1050000000",  "GRN",  "Gamma Ray Counts",            "GAPI",      "0",  "10" },
    {"1022012502",  "NEUT", "Neutron counts",              "COUNTS",    "0", "1" },

    {"1030216707",  "PERM",     "Permeability",            "MD",        "0.01", "10000.0" },

    {"1022012533",  "TEMP",     "Temperature",             "DEGF",      "0",    "1" },
    {"COMPUTED",    "TEMPDIFF", "Temperature Diff",        "DEGF",      "0",    "1" },

    {"200LIN1",     "LIN_1",    "Linear Track Curve 1",    "UNI",      "0",    "1" },
    {"200LIN2",     "LIN_2",    "Linear Track Curve 2",    "UNI",      "0",    "1" },
    {"200LIN3",     "LIN_3",    "Linear Track Curve 3",    "UNI",      "0",    "1" },
    {"200LIN4",     "LIN_4",    "Linear Track Curve 4",    "UNI",      "0",    "1" },

    {"200LOG1",     "LOG_1",    "Semilog Track Curve 1",   "UNL",      ".1",   "1000" },
    {"200LOG2",     "LOG_2",    "Semilog Track Curve 2",   "UNL",      ".1",   "1000" },
    {"200LOG3",     "LOG_3",    "Semilog Track Curve 3",   "UNL",      ".1",   "1000" },
    {"200LOG4",     "LOG_4",    "Semilog Track Curve 4",   "UNL",      ".1",   "1000" },

    {"COMPUTED",    "RHOMAA",  "Apparent Matrix Density",  "GM/CC",     "2", "3"},
    {"COMPUTED",    "UMAA",    "Apparent Photoelectric",   "BARNS/E",   "0", "20"},
    {"COMPUTED",    "PHIDIFF", "Neutron-Density Porosity", "PU",        "-.3", ".3"},

    {"COMPUTED",    "Th/U", "Thorium/Uranium Ratio",       "LOG_RATIO", ".1", "100"},
    {"COMPUTED",    "Th/K", "Thorium/Potassium Ratio",     "LOG_RATIO", ".1", "100"},
  };
}