/*
 * @lithMath.java Version 1.1 05/05/2008
 *
 * Copyright (c) 2008 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package lith.math;

/** Class lithMath
 *  <p> This Class will perform the basic math for the lithology plots.
 *
 *  @version 1.1 05/05/2008
 *  @author  John R. Victorine
 */

public class lithMath
{
  public static final int _QUARTZ      = math.mathLAS._QUARTZ;    // SILICATE  - Quartz
  public static final int _CALCITE     = math.mathLAS._CALCITE;   // CARBONATE - Calcite
  public static final int _DOLOMITE    = math.mathLAS._DOLOMITE;  // CARBONATE - Dolomite
  public static final int _ANHYDRITE   = 3;  // EVAPORITE - Anhydrite
  public static final int _GYPSUM      = 4;  // EVAPORITE - Gypsum
  public static final int _HALITE      = 5;  // EVAPORITE - Halite
  public static final int _HIGH_Z_CLAY = 6;  // CLAY      - ( Illite )
  public static final int _LOW_Z_CLAY  = 7;  // CLAY      - ( Smectite )
  public static final int _COAL        = 8;  // ORGANIC   - Coal
  public static final int _FELDSPAR    = 9;  // SILICATE  - Orthoclase
  public static final int _PLAGIOCLASE = 10; // IGNEOUS   - Anorthite
  public static final int _ULTRA_MAFIC = 11; // IGNEOUS   - Ultra Mafic
  public static final int _SIO2        = 12;  // SHALE      - ( Smectite )
  public static final int _SHALE       = 13;  // SHALE      - ( Smectite )

  public static final int _ROWS        = 14;  // Total Mineral

  public static final String MINERALS[] =
  {
    "Quartz",
    // Carbonates
    "Calcite",
    "Dolomite",
    //Evaporites
    "Anhydrite",
    "Gypsum",
    "Halite",
    // Clays
    "High Z Clay", // "Illite",
    "Low Z Clay",  // "Smectite"
    // Coal
    "Coal",
    //igneous
    "Feldspar",
    "Anorthite",
    "Ultramafic",
    "QUARTZ No Porosity",
    "SHALE"
  };

  public static final int _RHOB     = 0;   // Bulk Density Log Curve
  public static final int _RHOMAA   = 1;   // Computed Matrix Density
  public static final int _NPHI     = 2;   // Neutron Porosity Log Curve
  public static final int _UMAA     = 3;   // Computed Photoelectric Factor
  public static final int _PHI_DIFF = 4;   // Computed Porosity Difference
  public static final int _GR       = 5;   // Gamma Ray Log Curve
  public static final int _TH       = 6;   // Thorium Log Curve
  public static final int _U        = 7;   // Uranium Log Curve
  public static final int _K        = 8;   // Potassium Log Curve
  public static final int _DTC      = 9;   // Sonic
  public static final int _COLUMNS  = 10;  // Total Columns

  public static final double _MINERAL[][] =
  {
  //  RHOB  Rhomaa NPHI  Umaa    DelPHI   GR     Th     U      K     DT
  //(gm/cc) (gm/cc)(PU) (barns/cc)(PU)    (API)  (ppm) (ppm)   (%)
//    { 2.64,  2.65, 17.5,   4.8,   20.0,  14.83,  2.0,  0.7,  0.08, 55.5 }, // Quartz     - Silicate
    { 2.64,  2.65, 18.0,   4.8,   20.0,  14.83,  2.0,  0.7,  0.08, 55.5 }, // SS     - Silicate
//    { 2.71,  2.71,  1.0,  13.8,  -10.0,   4.9,   0.0,  0.0,  0.3,  47.6 }, // Calcite    - Carbonate
    { 2.71,  2.71,  0.1,  13.8,  -10.0,   4.9,   0.0,  0.0,  0.3,  47.6 }, // LM    - Carbonate
//    { 2.85,  2.87, 25.0,   9.0,   20.0,   1.14,  0.0,  0.0,  0.07, 43.5 }, // Dolomite   - Carbonate
    { 2.85,  2.88, 18.0,   9.0,   20.0,   1.14,  0.0,  0.0,  0.07, 43.5 }, // DOL   - Carbonate
    { 2.98,  2.9,  1.0,   14.9,  -10.0,   0.0,   0.0,  0.0,  0.0,  50.0 }, // ANH  - Evaporite
    { 2.35,  3.1,  51.0,  18.38, -30.0,   0.0,   0.0,  0.0,  0.0,  52.5 }, // GYP     - Evaporite
    { 2.04,  2.04, -2.0,  13.8,   40.0,   0.0,   0.0,  0.0,  0.0,  67.0 }, // NaCl     - Evaporite
    { 2.52,  3.006, 30.0, 13.0,    3.1, 234.34, 17.5, 6.95,  6.7,  55.5 }, // Illite     - Clay
    { 2.12,  3.08, 44.0,   7.0,   15.0, 132.16, 17.0, 4.85,  1.6,  55.5 }, // Smectite   - Clay
    { 1.24,  1.5,  60.0,   0.42,  40.0,   0.0,   0.0,  0.0,  0.0, 105.0 }, // Coal       - Organic
    { 2.52,  2.52,  1.5,   7.5,   30.0, 265.2,   5.0,  1.4, 14.0,  69.0 }, // Orthoclase - Silicate
    { 2.76,  2.74, 11.0,  10.76,  20.0,  34.0,   3.9,  1.5,  1.31, 51.5 }, // Plagioclase
    { 3.09,  3.02, 24.0,  13.95,  20.0,  15.0,   2.0, 0.35,  0.54, 49.4 }, // Ultra-Mafic
    { 2.64,  2.65, 1.0,    4.8,   20.0,  14.83,  2.0,  0.7,  0.08, 55.5 }, // Quartz     - Silicate
    { 2.12,  3.02, 44.0,   6.0,   15.0, 132.16, 17.0, 4.85,  1.6,  55.5 }, // Smectite   - Clay
  };

  // Fluid Constants

  public static final int _FRESH_H2O = math.mathLAS._FRESH;
  public static final int _SALTY_H2O = math.mathLAS._SALT;
  public static final int _OIL_MUD   = math.mathLAS._OIL;

  public static final double RHO_f[]   = {   1.0,   1.1, 0.985 }; // (gm/cc)
  public static final double DT_f[]    = { 189.0, 185.0, 204.5 }; // (usec/ft)
  public static final double U_f[]     = { 0.398, 1.36,  0.0 };   // (barns/cc)

  /* ----------------------------------------------------------------------- *
   * ------- Mineral Order depending on the type of Matrix to create ------- *
   * ----------------------------------------------------------------------- */

  // Dolomite Quartz Calcite Triangle

  public static final int _RU[]      = { _DOLOMITE, _CALCITE, _QUARTZ };
  public static final int _RU_COL[]  = { _RHOMAA, _UMAA };
  public static final int _RU_LITH[] =
  {
    lith.lithology.lithologySymbolsStruct._DOLOMITE,  // Blue
    lith.lithology.lithologySymbolsStruct._LIMESTONE, // Cyan
    lith.lithology.lithologySymbolsStruct._SANDSTONE  // Yellow
  };

  public static final int _PD_COL[]   = { _RHOMAA, _NPHI };
  public static final int _PDGR_COL[] = { _RHOMAA, _NPHI, _GR };
//  public static final int _PD_COL[]   = { _RHOMAA, _PHI_DIFF };
//  public static final int _PDGR_COL[] = { _RHOMAA, _PHI_DIFF, _GR };

  public static final int _RUo[]      = { _SHALE,  _DOLOMITE,
                                         _CALCITE, _QUARTZ };
  public static final int _RUo_LITH[] =
  {
    lith.lithology.lithologySymbolsStruct._SHALE,     // Light Gray
    lith.lithology.lithologySymbolsStruct._DOLOMITE,  // Blue
    lith.lithology.lithologySymbolsStruct._LIMESTONE, // Cyan
    lith.lithology.lithologySymbolsStruct._SANDSTONE  // Yellow
  };

  // Clay - Quartz Calcite Area

  public static final int _RUGR[] = { _LOW_Z_CLAY, _HIGH_Z_CLAY ,
                                      _CALCITE,    _QUARTZ };
  public static final int _RUGR_COL[]  = { _RHOMAA, _UMAA, _GR };
  public static final int _RUGR_LITH[] =
  {
    lith.lithology.lithologySymbolsStruct._CLAY,       // Dark Gray
    lith.lithology.lithologySymbolsStruct._SHALE,      // Light Gray
    lith.lithology.lithologySymbolsStruct._LIMESTONE,  // Cyan
    lith.lithology.lithologySymbolsStruct._SANDSTONE   // Yellow
  };

  // Arkose Silicate Carbonate Area

  public static final int _ARKOSE[] = { _HIGH_Z_CLAY, _CALCITE, _SIO2, _FELDSPAR };
  public static final int _ARKOSE_LITH[] =
  {
    lith.lithology.lithologySymbolsStruct._SHALE,     // Light Gray
    lith.lithology.lithologySymbolsStruct._LIMESTONE, // Cyan
    lith.lithology.lithologySymbolsStruct._SANDSTONE, // Yellow
    lith.lithology.lithologySymbolsStruct._FELDSPAR   // Violet
  };

  // Igneous Rock Area

  public static final int _IGNEOUS[] = { _ULTRA_MAFIC, _PLAGIOCLASE, _SIO2, _FELDSPAR };
  public static final int _IGNEOUS_LITH[] =
  {
    lith.lithology.lithologySymbolsStruct._ULTRAMAFIC, //_VOLCANIC,  // Orgate
    lith.lithology.lithologySymbolsStruct._PLAGIOCLASE, //_IGNEOUS,   // Pink
    lith.lithology.lithologySymbolsStruct._SANDSTONE, // Yellow
    lith.lithology.lithologySymbolsStruct._FELDSPAR   // Violet
  };

  // Dolomite Anydrite Calcite Triangle

  public static final int _ANHY[]      = { _DOLOMITE, _CALCITE, _ANHYDRITE };
  public static final int _ANHY_LITH[] =
  {
    lith.lithology.lithologySymbolsStruct._DOLOMITE,  // Blue
    lith.lithology.lithologySymbolsStruct._LIMESTONE, // Cyan
    lith.lithology.lithologySymbolsStruct._ANHYDRITE  // Pink
  };

  // Dolomite Gypsum Calcite Triangle

  public static final int _GYP[]      = { _DOLOMITE, _CALCITE, _GYPSUM };
  public static final int _GYP_LITH[] =
  {
    lith.lithology.lithologySymbolsStruct._DOLOMITE,  // Blue
    lith.lithology.lithologySymbolsStruct._LIMESTONE, // Cyan
    lith.lithology.lithologySymbolsStruct._GYPSUM     // Violet
  };

  // Calcite Quartz Halite Triangle

  public static final int _SALT_COAL[] = { _CALCITE, _QUARTZ, _HALITE, _COAL };
  public static final int _SALT_COAL_LITH[] =
  {
    lith.lithology.lithologySymbolsStruct._LIMESTONE, // Cyan
    lith.lithology.lithologySymbolsStruct._SANDSTONE, // Yellow
    lith.lithology.lithologySymbolsStruct._HALITE,    //
    lith.lithology.lithologySymbolsStruct._COAL       // Black
  };

  // Calcite Quartz Halite Triangle

  public static final int _SALT[]      = { _CALCITE, _QUARTZ, _HALITE };
  public static final int _SALT_LITH[] =
  {
    lith.lithology.lithologySymbolsStruct._LIMESTONE, // Cyan
    lith.lithology.lithologySymbolsStruct._SANDSTONE, // Yellow
    lith.lithology.lithologySymbolsStruct._HALITE     //
  };

  // Calcite Quartz Coal Triangle

  public static final int _BLK_COAL[]      = { _CALCITE, _QUARTZ, _COAL };
  public static final int _BLK_COAL_LITH[] =
  {
    lith.lithology.lithologySymbolsStruct._LIMESTONE, // Cyan
    lith.lithology.lithologySymbolsStruct._SANDSTONE, // Yellow
    lith.lithology.lithologySymbolsStruct._COAL       // Black
  };

  /* ======================================================================= *
   * -------------------------- LOG DATA MATRIX ---------------------------- *
   * ======================================================================= */

  /** Method getLog()
   * <p> This method will convert the Log curves to a matrix for the
   *     compositional analysis matrix.
   * @param  matrix = the matrix to use in computing DPHI
   * @param  fluid = Identifier of type of fluid to use
   * @param  dRHOB = Bulk density log value
   * @param  dPE   = Photoelectric Factor log value
   * @param  dNPHI = Neutron Porosity
   * @param  dGR   = Gammma Ray
   * @return dIn   = the Input matrix for the compositional analysis
   */

  public static double[] getLog( int matrix, int fluid,  double dRHOB,
                                 double dPE, double dNPHI, double dGR )
  {
    double dIn[] = {0.0, 0.0, 0.0, 0.0};
    double dDPHI = 0.0;
    double dPHI  = 0.0;

    dDPHI  = math.mathLAS.computePHI( dRHOB, math.mathLAS._DENSITY,
                                      matrix, fluid );
    dPHI   = math.mathLAS.computeAvgPHI( dNPHI,  dDPHI );

    dIn[0] = math.mathLAS.computeRhomaa(fluid, dRHOB, dPHI);
    if (dPE > 0.0)
      dIn[1] = math.mathLAS.computeUmaa(fluid, dRHOB, dPE, dPHI);
    else
      dIn[1] = 100.0*dNPHI;
    dIn[2] = dGR;
    dIn[3] = 1.0;

    return (dIn);
  }

  /* ----------------------------------------------------------------------- *
   * ------------------ Dolomite Quartz Calcite Triangle ------------------- *
   * ----------------------------------------------------------------------- */

  /** Method getRU()
   * <p> This method will convert the Log curves to a matrix for the
   *     compositional analysis matrix.
   * @param  matrix = the matrix to use in computing DPHI
   * @param  fluid = Identifier of type of fluid to use
   * @param  dRHOB = Bulk density log value
   * @param  dPE   = Photoelectric Factor log value
   * @param  dNPHI = Neutron Porosity
   * @return dIn   = the Input matrix for the compositional analysis
   */

  public static double[] getRU( int matrix, int fluid,  double dRHOB,
                                double dPE, double dNPHI )
  {
    double dIn[] = {0.0, 0.0, 0.0};
    double dDPHI = 0.0;
    double dPHI  = 0.0;

    dDPHI  = math.mathLAS.computePHI( dRHOB, math.mathLAS._DENSITY,
                                      matrix, fluid );
    dPHI   = math.mathLAS.computeAvgPHI( dNPHI,  dDPHI );

    dIn[0] = math.mathLAS.computeRhomaa(fluid, dRHOB, dPHI);
    dIn[1] = math.mathLAS.computeUmaa(fluid, dRHOB, dPE, dPHI);
    dIn[2] = 1.0;

    return (dIn);
  }

  /** Method getPD()
   * <p> This method will convert the Log curves to a matrix for the
   *     compositional analysis matrix.
   * @param  matrix = the matrix to use in computing DPHI
   * @param  fluid = Identifier of type of fluid to use
   * @param  dRHOB = Bulk density log value
   * @param  dNPHI = Neutron Porosity
   * @return dIn   = the Input matrix for the compositional analysis
   */

  public static double[] getPD( int matrix, int fluid,
                                double dRHOB, double dNPHI )
  {
    double dIn[] = {0.0, 0.0, 0.0};
    double dDPHI = 0.0;
    double dPHI  = 0.0;

    dDPHI  = math.mathLAS.computePHI( dRHOB, math.mathLAS._DENSITY,
                                      matrix, fluid );
    dPHI   = math.mathLAS.computeAvgPHI( dNPHI,  dDPHI );

    dIn[0] = math.mathLAS.computeRhomaa(fluid, dRHOB, dPHI);
    dIn[1] = 100.0*dNPHI;
//    dIn[0] = dRHOB;
//    dIn[1] = 100.0 * (dNPHI-dDPHI);
    dIn[2] = 1.0;

    return (dIn);
  }

  /* ----------------------------------------------------------------------- *
   * ---------------------- Clay Quartz Calcite Area ----------------------- *
   * ----------------------------------------------------------------------- */

  /** Method getRUGR()
   * <p> This method will convert the Log curves to a matrix for the
   *     compositional analysis matrix.
   * @param  matrix = the matrix to use in computing DPHI
   * @param  fluid = Identifier of type of fluid to use
   * @param  dRHOB = Bulk density log value
   * @param  dPE   = Photoelectric Factor log value
   * @param  dNPHI = Neutron Porosity
   * @param  dGR   = Gammma Ray
   * @return dIn   = the Input matrix for the compositional analysis
   */

  public static double[] getRUGR( int matrix, int fluid,  double dRHOB,
                                  double dPE, double dNPHI, double dGR )
  {
    double dIn[] = {0.0, 0.0, 0.0, 0.0};
    double dDPHI = 0.0;
    double dPHI  = 0.0;

    dDPHI  = math.mathLAS.computePHI( dRHOB, math.mathLAS._DENSITY,
                                      matrix, fluid );
    dPHI   = math.mathLAS.computeAvgPHI( dNPHI,  dDPHI );

    dIn[0] = math.mathLAS.computeRhomaa(fluid, dRHOB, dPHI);
    dIn[1] = math.mathLAS.computeUmaa(fluid, dRHOB, dPE, dPHI);
    dIn[2] = dGR;
    dIn[3] = 1.0;

    return (dIn);
  }

  /** Method getPDGR()
   * <p> This method will convert the Log curves to a matrix for the
   *     compositional analysis matrix.
   * @param  matrix = the matrix to use in computing DPHI
   * @param  fluid = Identifier of type of fluid to use
   * @param  dRHOB = Bulk density log value
   * @param  dNPHI = Neutron Porosity
   * @param  dGR   = Gammma Ray
   * @return dIn   = the Input matrix for the compositional analysis
   */

  public static double[] getPDGR( int matrix, int fluid,  double dRHOB,
                                  double dNPHI, double dGR )
  {
    double dIn[] = {0.0, 0.0, 0.0, 0.0};
    double dDPHI = 0.0;
    double dPHI  = 0.0;

    dDPHI  = math.mathLAS.computePHI( dRHOB, math.mathLAS._DENSITY,
                                      matrix, fluid );
    dPHI   = math.mathLAS.computeAvgPHI( dNPHI,  dDPHI );

    dIn[0] = math.mathLAS.computeRhomaa(fluid, dRHOB, dPHI);
    dIn[1] = 100.0*dNPHI;
//    dIn[0] = dRHOB;
//    dIn[1] = 100.0 * (dNPHI-dDPHI);
    dIn[2] = dGR;
    dIn[3] = 1.0;

    return (dIn);
  }

  /* ======================================================================= *
   * ---------------------- GET MINERAL N X N MATRIX ----------------------- *
   * ======================================================================= */

  /** Method getMatrix()
   * <p> This method will build the N X N Matrix for the compositional
   *     analysis matrix.
   * @param  iMinerals = the array of minerals
   * @param  iColumns  = data columns
   * @return dMat      = compositional Matrix
   */

  public static double[][] getMatrix(int iMinerals[], int iColumns[])
  {
    int i = 0;
    int j = 0;
    int len = iMinerals.length;
    double dMat[][] = null;

    dMat = new double[len][len];

    // Initialize the Matrix

    for (i=0; i<len; i++)
    {
      for (j=0; j<len; j++)
      {
        dMat[i][j] = 1.0;
      }
    }

    // Build Matrix [ Log Type ] X [ Mineral ]
    // Last Row is Unitary define at initizlization of Matrix

    for (i=0; i<len-1; i++)
    {
      for (j=0; j<len; j++)
      {
//System.out.println(i+" "+j+" "+iColumns[i]+" "+iMinerals[j]);
        dMat[i][j] = _MINERAL[iMinerals[j]][iColumns[i]];
      }
    }

    return (dMat);
  }

  /* ======================================================================= *
   * ----------------------- COMPOSITIONAL ANALYSIS ------------------------ *
   * ======================================================================= */

  /** Method composition()
   * <p> This method will compute the compositional anlaysis of a rock depending
   *     on the data available.
   *        [ % ] = [ L ] / [ M ]
   *
   * @param  dIn    = Array of log variables
   * @param  dMat   = N X N Matrix
   * @return dOut   = The output matrix
   */

  public static double[] composition( double dIn[], double dMat[][])
  {
    double dOut[]   = null;
    double dInv[][] = null;
    int    N        = 0;

    if ((dIn != null) && (dMat != null))
    {
      N    = dIn.length;
      dOut = new double[N];
      dInv = new double[N][N];

      dInv = lith.math.Inverse.invert(dMat);

      for (int i=0; i<N; i++)
      {
        for (int j=0; j<N; j++)
        {
          dOut[i] = dOut[i] + dInv[i][j] * dIn[j];
        }
      }
    }

    return (dOut);
  }

  /** Method computeComposition()
   * <p> This method will compute the percentage of Quartz, Calcite and Dolomite
   *     in the rock
   *     From: http://www.kgs.ku.edu/Gemini/Help/PfEFFER/Pfeffer-theory11.html
   *
   *      D = Dolomite %      Phin = Neutron Porosity
   *      Q = Quartz %        Rhob = Apparent Bulk Density
   *      C = Calcite %       U    = Apparent Photoelectric Factor
   *
   *      Neutron:  5.0 D  - 5.0 Q  + 0 C     + 100 Phi = Phin
   *      Density:  2.87 D + 2.65 Q + 2.71 C  + 1.0 Phi = Rhob
   *      UPhoto:   9.0 D  + 4.79 Q + 13.77 C + 0.5 Phi = U
   *      Unity:    1.0 D  + 1.0  Q + 1.0 C   + 1.0 Phi = 1.0
   *
   *       _                         _   _   _     _     _
   *      |  5.0  -5.0   0.0   100.0  | |  D  |   |  Phin |
   *      |  2.87 2.65   2.71    1.0  | |  Q  | = |  Rhob |
   *      |  9.0  4.79  13.77    0.5  | |  C  |   |  U    |
   *      |  1.0  1.0    1.0     1.0  | | 1.0 |   |  1.0  |
   *       -                         -   -   -     -     -
   *
   * @param  dNPHI = The neutron Porosity
   * @param  dRho  = The Bulk Density   (Rhomaa)
   * @param  dPe   = The Photo Electric (UMaa)
   * @return dComp = Array of Composition percentage
   */

  public static double[] computeComposition( double dNPHI,
                                             double dRho,
                                             double dPe )
  {
    double dIn[]    = { dNPHI, dRho, dPe, 1.0 };
    double dOut[]   = { 0.0,   0.0,  0.0, 0.0 };
    //                Dolomite,  Quartz, Limestone, H20
    double dMat[][] = {{ 1.0,     -2.0,    0.0,   100.0 },  // NPHI
                       { 2.87,     2.65,   2.71,    1.0 },  // RHOMAA
                       { 9.0,      4.79,  13.77,    0.5 },  // UMAA
                       { 1.0,      1.0,    1.0,     1.0 }}; // Unitary
    dMat = lith.math.Inverse.invert(dMat);

    for (int i=0; i<4; i++)
    {
      for (int j=0; j<4; j++)
      {
        dOut[i] = dOut[i] + dMat[i][j] * dIn[j];
      }
    }

    return (dOut);
  }

  /** Method compositionByCore( double dNPHI, double dRho )
   * <p> This method will compute the percentage of Quartz, Calcite and Dolomite
   *     in the rock
   * @param  dNPHI = The neutron Porosity
   * @param  dRho  = The Bulk Density   (Rhomaa)
   * @return dOut  = Th output matrix (Dolomite, Quartz, Limestone)
   */

  public static double[] compositionByCore( double dNPHI, double dRho )
  {
    double dDPHI    = math.mathLAS.computePHI( dRho,
                                               math.mathLAS._DENSITY,
                                               math.mathLAS._CALCITE,
                                               math.mathLAS._FRESH );
    double delta    = dNPHI - dDPHI;
    double dRhomaa  = dRho;

    double dIn[]    = { dNPHI, dRhomaa, delta, 1.0 };
    double dOut[]   = { 0.0,   0.0,  0.0, 0.0 };
    //                 Dolomite,  Quartz, Limestone, H20
    double dMat[][] = {{ 1.0,   -2.0,     0.0,     100.0 },  // NPHI
                       { 2.87,   2.65,    2.71,      1.0 },  // RHOMAA
                       { 20.0,   20.0,   -10.0,    100.0 },  // NPHI-DPHI
                       { 1.0,    1.0,     1.0,       1.0 }}; // Unitary

    dMat = lith.math.Inverse.invert(dMat);

    for (int i=0; i<4; i++)
    {
      for (int j=0; j<4; j++)
      {
        dOut[i] = dOut[i] + dMat[i][j] * dIn[j];
      }
    }

    return (dOut);
  }
  /** Method compositionByRock( double dNPHI, double dRho )
   * <p> This method will compute the percentage of Quartz, Calcite and Dolomite
   *     in the rock
   * @param  dNPHI = The neutron Porosity
   * @param  dRho  = The Bulk Density   (Rhomaa)
   * @return dOut  = Th output matrix (Dolomite, Quartz, Limestone)
   */

  public static double[] compositionByRock( double dNPHI, double dRho )
  {
    double dDPHI    = math.mathLAS.computePHI( dRho,
                                               math.mathLAS._DENSITY,
                                               math.mathLAS._CALCITE,
                                               math.mathLAS._FRESH );
    double delta    = dNPHI - dDPHI;
    double dRhomaa  = math.mathLAS.computeRhomaa(math.mathLAS._FRESH, dRho, dNPHI);

    double dIn[]    = { dNPHI, dRhomaa, delta, 1.0 };
    double dOut[]   = { 0.0,   0.0,  0.0, 0.0 };
    //                 Dolomite,  Quartz, Limestone, H20
    double dMat[][] = {{ 1.0,   -2.0,     0.0,     100.0 },  // NPHI
                       { 2.87,   2.65,    2.71,      1.0 },  // RHOMAA
                       { 20.0,   20.0,   -10.0,    100.0 },  // NPHI-DPHI
                       { 1.0,    1.0,     1.0,       1.0 }}; // Unitary

    dMat = lith.math.Inverse.invert(dMat);

    for (int i=0; i<4; i++)
    {
      for (int j=0; j<4; j++)
      {
        dOut[i] = dOut[i] + dMat[i][j] * dIn[j];
      }
    }

    return (dOut);
  }
}