/*
 * @rockColumnPlotTrack.java Version 1.1 08/07/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package lith.rock.plot;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;

import iqstrat.iqstratHeadersStruct;

import lith.rock.rockColumnListStruct;
import lith.rock.rockColumnStruct;
import lith.rock.plot.rockColumnPlotSymbol;

import parse.parseRockColumnSymbolsListStruct;

import xsection.xsectionDataListStruct;

public class rockColumnPlotTrack extends Canvas
{
  // Input Variables

  private double depthStart = 0.0;   // Starting Depth
  private double depthEnd   = 0.0;   // Ending Depth
  private int    iScale     = -1;    // Plot Scale

  // Global Variables

//  private rockColumnListStruct             stList    = null;
  private int                              _TOTAL      = 2;
  private rockColumnListStruct             stColumn[]  = null;

  private parseRockColumnSymbolsListStruct stSymbols = null;
  private rockColumnPlotSymbol             plotLith  = null;

  private xsectionDataListStruct           stXList = null;

  // -- By Datum

  private int    iDatum   = iqstrat.iqstratTracksStruct._BY_LOG; // Datum
  private String sDatum   = "";

  // Plot Variables

  private int iWidth      = 100;  // Width of Plot
  private int iHeight     = 100;  // Height of Plot with the Titles
  private int iLogHeight  = 100;  // Height of Plot without the Titles
  private int iIncrementY = 100;  // Increment Height

  public static final int PLOT_TITLES = iqstrat.iqstratTracksStruct.PLOT_TITLES;
  public static final int LABELSTART  = iqstrat.iqstratTracksStruct.LABELSTART;

  /** CONSTRUCTOR rockColumnPlotTrack()
   *  <p> This is the constructor for this class.
   *  @param depthStart = The starting depth of plot
   *  @param depthEnd   = The ending depth of plot
   *  @param iScale     = The scale
   *  @param stSymbols  = Texture Symbols List Data Structure
   */

  public rockColumnPlotTrack( double depthStart,
                              double depthEnd,
                              int    iScale,
                              parseRockColumnSymbolsListStruct stSymbols )
  {
    this.stSymbols = stSymbols;
    plotLith       = new rockColumnPlotSymbol( stSymbols );

    setPlotHeight(iScale, depthEnd, depthStart);

    this.setBackground(Color.white);
  }

  /** Method close()
   * <p> This method will set object to null to force JAVA to reallocate memory
   */

  public void close()
  {
    stSymbols = null;

    if (stColumn != null)
    {
      for (int i=0; i<_TOTAL; i++)
      {
  	    if (stColumn[i] != null)
          stColumn[i].delete();
        stColumn[i]  = null;
      }
    }
    stColumn  = null;

    if (plotLith != null)
      plotLith.close();
    plotLith = null;
  }

  /* =============================================================== *
   * ------------------------- SET METHODS ------------------------- *
   * =============================================================== */

  /** Method setPlotHeight( int iScale, double dMaximum, double dMinimum )
   *  <p> This method will set the Plot Height.
   * @param iScale   = The Depth Track Scale identifier
   * @param dMaximum = The Maximum Depth Value
   * @param dMinimum = The Minimum Depth Value
   */

  public void setPlotHeight( int iScale, double dMaximum, double dMinimum )
  {
    this.depthStart = dMinimum;
    this.depthEnd   = dMaximum;
    this.iScale     = iScale;

    iLogHeight = (int) ((100 * Math.abs( dMaximum - dMinimum )) /
                        iqstrat.iqstratTracksStruct.SCALE[iScale]);
    iHeight    = PLOT_TITLES + iLogHeight;

    if (stXList != null)
    {
      if (stColumn != null)
      {
        for (int i=0; i<_TOTAL; i++)
        {
    	  if (stColumn[i] != null)
            stColumn[i].delete();
          stColumn[i]  = null;
        }
      }
      stColumn  = null;

      stColumn    = new rockColumnListStruct[_TOTAL];
      for (int i=0; i<this._TOTAL; i++)
      {
        stColumn[i] = new rockColumnListStruct();
        if (stXList.stItem[i].stColumn != null)
          stColumn[i] = setRockColumn( iScale, stXList.stItem[i].stColumn );
	  }

//      stColumn    = new rockColumnListStruct[2];
//      stColumn[0] = new rockColumnListStruct();
//      stColumn[1] = new rockColumnListStruct();

//      if (stXList.stItem[0].stColumn != null)
//        stColumn[0] = setRockColumn( iScale, stXList.stItem[0].stColumn );
//      if (stXList.stItem[1].stColumn != null)
//        stColumn[1] = setRockColumn( iScale, stXList.stItem[1].stColumn );
////    stColumn = lith.rock.rockColumnUtility.getData( stList, iScale );
    }
  }

  /** Method setRockColumn()
   * <p> This method will set the rock Column that will be shown
   * @param iDScale = The Feet / Inch Scale
   * @param st      = Rock column determined by user from rock data.
   */

  public static rockColumnListStruct setRockColumn( int iDScale, rockColumnListStruct st )
  {
	rockColumnListStruct stC = null;

    stC = lith.rock.rockColumnUtility.getData( st, iDScale );

    return (stC);
  }

  /** Method setData()
   * <p> This method will set the cross section data
   * @param stXList = Cross Section Data
   */

  public void setData( xsectionDataListStruct stXList )
  {
    this.stXList = stXList;
    this._TOTAL  = stXList.iCount;
    stColumn    = new rockColumnListStruct[this._TOTAL];
    for (int i=0; i<this._TOTAL; i++)
      stColumn[i] = setRockColumn( iScale, stXList.stItem[i].stColumn );
//    stColumn[0] = setRockColumn( iScale, stXList.stItem[0].stColumn );
//    stColumn[1] = setRockColumn( iScale, stXList.stItem[1].stColumn );
  }

  /** Method setDatum( int iDatum, String sDatum  )
   * <p> This method will set the datum by identifier button
   * @param iDatum = datum by identifier
   * @param sDatum = horizon name
   */

  public void setDatum( int iDatum, String sDatum )
  {
    this.iDatum = iDatum;
    this.sDatum = new String( sDatum );
  }

  /* =============================================================== *
   * ------------------------ DRAW METHODS ------------------------- *
   * =============================================================== */

  /** Method drawGrid()
   * <p> This method will draw the depth scale
   * @param g           = Graphics Parameter.
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   * @param iXData      = The well/outcrop number in Cross Section List Data
   */

  public void drawGrid( Graphics g, int iStartTrack,  int iWidthTrack, int iXData )
  {
	int    i,j;
    int    jXinc      = 0;
    int    iIncrement = iWidthTrack/10;
    int    iEndTrack  = iStartTrack+iWidthTrack;
    int    iY1        = 0;
    int    iY2        = 0;

    g.setColor(Color.black);

    Font fsb = new Font("Serif", Font.BOLD, 10);
    FontMetrics fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    // Cross Section Data Variables

    int                  iDataType = iqstrat.iqstratHeadersStruct.WELL;
    double               dElev     = 0.0;
    iqstratHeadersStruct stHeader  = null;

    if (stXList != null)
    {
      if (stXList.stItem[iXData] != null)
      {
        stHeader = stXList.stItem[iXData].stHeader;
        if (stHeader != null)
        {
          iDataType = stHeader.iType;
        }
      }
    }

    for (i=0; i<=iLogHeight; i+=iIncrementY)
    {
      if (iDataType == iqstrat.iqstratStruct._OUTCROP)
        iY1 = iHeight-i;
      else
        iY1 = i+PLOT_TITLES;

      jXinc = iIncrementY / 5;
      for (j=0; j<5; j++)
      {
        if (iDataType == iqstrat.iqstratStruct._OUTCROP)
          iY2 = iHeight-(i+(j*jXinc));
        else
          iY2 = i+(j*jXinc)+PLOT_TITLES;

        if ((iY2 >= PLOT_TITLES)  && (iY2 < iHeight))
        {
          g.setColor(Color.lightGray);
          g.drawLine(iStartTrack, iY2, iEndTrack, iY2);
        }
      }

      g.setColor(Color.gray);
      g.drawLine(iStartTrack, iY1, iEndTrack, iY1);
    }

//    g.setColor(Color.lightGray);
//    g.drawLine(iEndTrack, LABELSTART, iEndTrack, PLOT_TITLES+iHeight);

    g.setColor(Color.black);

    g.drawString("Lithology",   iStartTrack+3*iIncrement, LABELSTART+14);
    g.drawString("Rock Column", iStartTrack+2*iIncrement, LABELSTART+28);

    g.drawLine(iStartTrack, LABELSTART,  iEndTrack,   LABELSTART);
    g.drawLine(iStartTrack, PLOT_TITLES, iEndTrack,   PLOT_TITLES);
  }

  /** Method drawColumn( Graphics g, int iStartTrack, int iWidthTrack )
   * <p> This method draw the texture from the Gamma Ray after determining if
   *     it is a carbonate or a siliclastic rock.
   * @param g           = Graphics Parameter.
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   * @param iXData      = The well/outcrop number in Cross Section List Data
   */

  public void drawColumn( Graphics g, int iStartTrack, int iWidthTrack, int iXData )
  {
	int i,j,k,m,n;
    rockColumnListStruct st         = null;
    double               depth1     = 0.0;
    double               depth2     = 0.0;
    double               diff       = 0;
    double               dRatio     = 0.0;
    int                  iCount     = 0;
    int                  iDepth1    = 0;
    int                  iDepth2    = 0;
    int                  iDepth     = 0;
    int                  iDiff      = 0;
    String               sID        = "";
    int                  iLith      = -1;
    String               sIDd       = "";
    int                  iLithd     = -1;
    int                  iRows      = 0;
    int                  iCells     = 0;
    int                  iCellY     = 0;
    int                  iCellX     = 0;
    int                  iCellYd    = 0;
    int                  iCellXd    = 0;
    int                  iRGB[]     = { 255, 255, 255 };
    int                  iRGBd[]    = { 255, 255, 255 };
    int                  iTypes[]   = { -1, -1, -1 };
    int                  iOrder     = -1;
    int                  iPlot      = 0;
    int                  iPlotd     = 0;
    int                  iContinue  = 0;

    // Cross Section Data Variables

    int                  iDataType = iqstrat.iqstratHeadersStruct.WELL;
    double               dElev     = 0.0;
    iqstratHeadersStruct stHeader  = null;

    if (stXList != null)
    {
      if (stXList.stItem[iXData] != null)
      {
        dElev    = xsection.xsectionDataUtility.getElevation(
                     iDatum, sDatum, stXList.stItem[iXData] );
        stHeader = stXList.stItem[iXData].stHeader;
        if (stHeader != null)
        {
          iDataType = stHeader.iType;
        }
      }
    }

    st = stColumn[iXData];

    if (st != null)
    {
      if (st.iCount > 0)
      {
        for (k=0; k<st.iCount; k++)
        {
      	  if (iDatum == iqstrat.iqstratTracksStruct._BY_LOG)
		  {
            depth1 = st.stItem[k].depthStart;
            depth2 = st.stItem[k].depthEnd;
		  }
		  else
		  {
            depth1 = dElev - st.stItem[k].depthStart;
            depth2 = dElev - st.stItem[k].depthEnd;

            if (iDataType == iqstrat.iqstratHeadersStruct.OUTCROP)
            {
              depth1 = dElev + st.stItem[k].depthStart;
              depth2 = dElev + st.stItem[k].depthEnd;
            }
		  }

          dRatio  = (double) iLogHeight * (depthStart - depth1) /
                                          (depthStart - depthEnd);
          iDepth1 = PLOT_TITLES + (int) dRatio;
          if (iDataType == iqstrat.iqstratStruct._OUTCROP)
            iDepth1 = iHeight - (int) dRatio;

          dRatio  = (double) iLogHeight * (depthStart - depth2) /
                                          (depthStart - depthEnd);
          iDepth2 = PLOT_TITLES + (int) dRatio;
          if (iDataType == iqstrat.iqstratStruct._OUTCROP)
            iDepth2 = iHeight - (int) dRatio;

          iDepth = Math.abs( iDepth2 - iDepth1 );

//          if ((iDepth1 >= PLOT_TITLES) && (iDepth2 > PLOT_TITLES) &&
//              (iDepth1 < iHeight)      && (iDepth2 < iHeight))
          {
  		    sID    = st.stItem[k].sPrimary;
            iLith  = parse.parseRockColumnSymbolsUtility.getID( sID, stSymbols );

            iTypes = parse.parseRockColumnSymbolsUtility.getCapType( sID, stSymbols );

            if (iLith > -1)
            {
              iCells = stSymbols.stItem[iLith].iWeather;
              iCellY = stSymbols.stItem[iLith].iRows;
              iCellX = stSymbols.stItem[iLith].iColumns;
              iRGB   = stSymbols.stItem[iLith].iRGB;
              iPlot  = stSymbols.stItem[iLith].iOrder;

			  iRows  = iDepth / iCellY;
              iDiff  = iDepth - iRows *iCellY;

              for (i=0; i<iRows; i++)
              {
                if ((iDepth1+i*iCellY >= PLOT_TITLES) && (iDepth1+i*iCellY < iHeight))
                {
                  for (j=0; j<iCells; j++)
                  {
                    plotLith.drawBlock( g, iRGB[0], iRGB[1], iRGB[2],
                                        iStartTrack+j*iCellX, iDepth1 + i*iCellY,
                                        iCellX, iCellY);
                    plotLith.drawSymbolMask(
                           g, iCellY, plotLith.getOrder(iCount),
                           parse.parseRockColumnSymbolsUtility.getSymbol(sID, stSymbols),
                           iStartTrack+j*iCellX, iDepth1 + i*iCellY );
			      }

                  if (iTypes != null)
                  {
                    if (iTypes[0] != -1)
                    {
				      iOrder = plotLith.getOrder(iCount);

                      plotLith.drawSymbolMask(
                             g, iCellY, iOrder, iRGB,
                             parse.parseRockColumnSymbolsUtility.getCaps(sID, stSymbols),
                             iStartTrack+iCells*iCellX, iDepth1 + i*iCellY );
				    }
			      }

   	              if (st.stItem[k].iSecondary > 0)
			      {
			        for (m=0; m<st.stItem[k].iSecondary; m++)
			        {
  		              sIDd    = st.stItem[k].secondary[m];
                      iLithd  = parse.parseRockColumnSymbolsUtility.getID( sIDd, stSymbols );

                      if (iLithd > -1)
                      {
                        iCellYd = stSymbols.stItem[iLithd].iRows;
                        iCellXd = stSymbols.stItem[iLithd].iColumns;
                        iPlotd  = stSymbols.stItem[iLithd].iOrder;
                        if ((iPlot == 0) && ((iPlotd == 1) || (iPlotd ==2)))
                          iRGBd = stSymbols.stItem[iLithd].iRGB;
                        else
                          iRGBd = iRGB;

                        for (j=0; j<iCells; j++)
                        {
						  iContinue = 1;

						  if (iContinue == 1)
                            plotLith.drawSymbolMask(
                                 g, iCellYd, plotLith.getOrder(iCount), iRGBd,
                                 parse.parseRockColumnSymbolsUtility.getSymbol(sIDd, stSymbols),
                                 iStartTrack+j*iCellXd, iDepth1 + i*iCellYd );
			            }
			          }
			        }
			  	  }

                  if (iPlot > 3) //Force a replot of lithologies that have special symbols
                  {
                    for (j=0; j<iCells; j++)
                    {
                      plotLith.drawSymbolMask(
                             g, iCellY, plotLith.getOrder(iCount),
                             parse.parseRockColumnSymbolsUtility.getSymbol(sID, stSymbols),
                             iStartTrack+j*iCellX, iDepth1 + i*iCellY );
			        }
			      }
			      iCount++;
			    }
			  }

              if ((iDepth1+iRows*iCellY >= PLOT_TITLES) && (iDepth1+iRows*iCellY < iHeight))
              {
			    if (iDiff > 0)
			    {
                  for (j=0; j<iCells; j++)
                  {
                    plotLith.drawBlock( g, iRGB[0], iRGB[1], iRGB[2],
                                        iStartTrack+j*iCellX, iDepth1 + iRows*iCellY,
                                        iCellX, iDiff);
                    plotLith.drawSymbolMask(
                           g, iDiff, plotLith.getOrder(iCount),
                           parse.parseRockColumnSymbolsUtility.getSymbol(sID, stSymbols),
                           iStartTrack+j*iCellX, iDepth1 + iRows*iCellY );
			      }

                  if (iTypes != null)
                  {
                    if (iTypes[0] != -1)
                    {
				      iOrder = plotLith.getOrder(iCount);
				      if (iTypes[2] != -1)
				        iOrder = 2;

                      plotLith.drawSymbolMask(
                             g, iDiff, iOrder, iRGB,
                             parse.parseRockColumnSymbolsUtility.getCaps(sID, stSymbols),
                             iStartTrack+iCells*iCellX, iDepth1 + iRows*iCellY );
				    }
				  }

   	              if (st.stItem[k].iSecondary > 0)
			      {
			        for (m=0; m<st.stItem[k].iSecondary; m++)
			        {
  		              sIDd    = st.stItem[k].secondary[m];
                      iLithd  = parse.parseRockColumnSymbolsUtility.getID( sIDd, stSymbols );

                      if (iLithd > -1)
                      {
                        iCellYd = stSymbols.stItem[iLithd].iRows;
                        iCellXd = stSymbols.stItem[iLithd].iColumns;
                        iPlotd  = stSymbols.stItem[iLithd].iOrder;
                        if ((iPlot == 0) && ((iPlotd == 1) || (iPlotd ==2)))
                          iRGBd = stSymbols.stItem[iLithd].iRGB;
                        else
                          iRGBd = iRGB;

                        for (j=0; j<iCells; j++)
                        {
						  iContinue = 1;

                          if (iDiff > iCellYd) iDiff = iCellYd;

						  if (iContinue == 1)
                            plotLith.drawSymbolMask(
                                 g, iDiff, plotLith.getOrder(iCount), iRGBd,
                                 parse.parseRockColumnSymbolsUtility.getSymbol(sIDd, stSymbols),
                                 iStartTrack+j*iCellXd, iDepth1 + iRows*iCellYd );
			            }
			          }
			        }
			  	  }

                  if (iPlot > 3) // Force a replot of lithologies that have special symbols
                  {
                    for (j=0; j<iCells; j++)
                    {
                      plotLith.drawSymbolMask(
                             g, iDiff, plotLith.getOrder(iCount),
                             parse.parseRockColumnSymbolsUtility.getSymbol(sID, stSymbols),
                             iStartTrack+j*iCellX, iDepth1 + i*iCellY );
			        }
			      }
			    }
		      }
			}
		  }
	    }
	  }
	}
  }

  /** Method draw( Graphics g, int iSelected, int iStartTrack, int iWidthTrack )
   * <p> This method will create a LAS Plot Track
   * @param g           = Graphics Parameter.
   * @param iSelected   = The Track that was selected
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   * @param iXData      = The well/outcrop number in Cross Section List Data
   */

  public void draw(Graphics g, int iSelected, int iStartTrack, int iWidthTrack, int iXData )
  {
    int iColor = -1;

	if (iSelected == iqstrat.iqstratTracksStruct._ROCK_MEASURED_SECTION)
	{
      iColor = iqstrat.iqstratTracksStruct._SRC_ROCK;

      g.setColor(new Color(iqstrat.iqstratTracksStruct.COLORS[iColor][0],
                           iqstrat.iqstratTracksStruct.COLORS[iColor][1],
                           iqstrat.iqstratTracksStruct.COLORS[iColor][2]));
      g.fillRect(iStartTrack, LABELSTART, iWidthTrack, 20);

      drawGrid( g, iStartTrack, iWidthTrack, iXData );
      drawColumn( g, iStartTrack, iWidthTrack, iXData );
	}
  }

  /** Method paint( Graphics g )
   * <p> This method will paint Porosity & Permeability Plot
   * @param g = Graphics Parameter.
   */

  public void paint( Graphics g )
  {
    g.setColor(Color.white);
    g.fillRect(0, 0, iWidth, iHeight);
  }
}