/*
 * @xsectionControlFrame.java Version 1.1 10/31/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package xsection.gui;

import java.net.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.Observer;
import java.util.Observable;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import cmn.cmnLASFileListStruct;               // Common LAS File List Structure

import gui.guiSearchDirectoryFrame;            // File Search Dialog
import gui.guiMemoryPanel;                     // Memory Panel

import iqstrat.iqstratStruct;                  // Global IQSTRAT Data Structure
import iqstrat.iqstratHeadersStruct;           // IQSTRAT Headers Data Structure
import iqstrat.iqstratControlStruct;           // Well Plot Control Data Structure
import iqstrat.iqstratRemarkListStruct;        // Geologist Descriptions List Data Strucutre

import kgs.kgsUserStruct;                      // KGS User Data Structure

import las.lasFileDataStruct;                  // LAS File Data Structure

//import las3.gui.las3FileFrame;                 // LAS 3.0 Save

import horizon.strat.stratListStruct;          // Stratigraphic Units List Structure
//import horizon.strat.gui.stratPlotFrame;       // User Defined Stratigraphic Units Dialog

import lith.lithology.lithologyListStruct;     // Lithology List Structure
import lith.lithology.lithologySymbolsListStruct; // Lithology Symbols List Structure
import lith.rock.rockColumnListStruct;         // Rock Column Data List Structure

import rock.rockDataListStruct;                // Rock Data List Data Structure

import rock.fossil.fossilListStruct;           // Fossil List Data Structure
import rock.sedimentary.sedimentaryListStruct; // Sed Structure List Data Structure
import rock.phi.phiListStruct;                 // Porosity List Data Strurture
import rock.color.rockColorListStruct;         // Rock Color List Data Structure

import xsection.xsectionDataListStruct;        // Cross Section List Structure
import xsection.xsectionDataStruct;            // Cross Section Data Structure
import xsection.gui.xsectionDataPanel;         // Cross Section Track Data Panel
import xsection.gui.xsectionPlotFrame;         // Cross Section Plot Frame
import xsection.gui.xsectionTypeLogTopsFrame;  // Cross Section Modify Tops Frame

/** Class xsectionControlFrame
 *  <p> This Class will create the Lithology Control Plot Frame.
 *
 *  @version 1.1 10/31/2012
 *  @author  John R. Victorine
 */

public class xsectionControlFrame extends JFrame implements ActionListener, Observer
{
  // Input Variables

  private Observable    pNotifier = null; // Observable Object this Frame
  private iqstratStruct stStruct  = null; // Global PROFILE Data Structure

  // Global Variables

  private Observable    notifier = null; // Observalbles Object
//  private int           iAction  = cmn.cmnStruct._LAS;

  // -- Depth Range of Data (initial value)

  private int    iScale     = iqstrat.iqstratTracksStruct.SCALE_100; // Scale

  private static final int _RESET  = 0;
  private static final int _MODIFY = 1;

  private double dStart     = 0.0;
  private double dEnd       = 2000.0;

  private double dStartOrig = 0.0;
  private double dEndOrig   = 2000.0;

  // -- By Datum

  private int    iDatum     = iqstrat.iqstratTracksStruct._BY_LOG; // Datum
  private String sDatum     = "";  // Datum by Horizon Name

  // -- LAS Plot Track Width Size

  private int    iLASSize[] = { xsection.gui.xsectionAvailablePanel._SINGLE,
                                xsection.gui.xsectionAvailablePanel._SINGLE,
                                xsection.gui.xsectionAvailablePanel._SINGLE,
                                xsection.gui.xsectionAvailablePanel._SINGLE };

  // -- Plot Data

  private xsectionDataListStruct stList    = null; // Cross Section List Structure

  //--  Panel Data

  private guiSearchDirectoryFrame  pSearch   = null; // Search for Directory
  private guiMemoryPanel           pnlMemory = new guiMemoryPanel(); // Memory Panel

  private xsectionDataPanel        pTrack[]  = null; // Cross Section Track Data Panel
  private xsectionPlotFrame        plotFrame = null; // Cross Section Plot Frame
  private xsectionTypeLogTopsFrame pTops     = null; // Modify Tops Frame
//  private las3FileFrame            p3File    = null; // LAS 3 File Data Frame

//  private stratPlotFrame           pChart    = null; // User Define Strat Units Dialog

  // Plot Screen Location & Size

  private int iFrameX       = -1;  // Plot Frame X-Coordintate
  private int iFrameY       = -1;  // Plot Frame Y-Coordintate
  private int iFrameWidth   = -1;  // Plot Frame Width
  private int iFrameHeight  = -1;  // Plot Frame Height

  // Screen Width

  public static final int iControlWidth = 325; // Control Panel Width
  private int iScreenWidth  = 0;   // Screen Width
  private int iScreenHigh   = 0;   // Scree Hieght

  // Selected Plot Tracks

//  private int iSelected_0[] = null; // Reference Well
//  private int iSelected_1[] = null; // Edit Well

  // Global Frame Widgets

  // -- Depth Scale Buttons

  private JLabel       lblScale    = new JLabel();

  // -- Depth Range Widgets

  private JTextField   txtStart    = new JTextField(); // Starting Depth
  private JTextField   txtEnd      = new JTextField(); // Ending Depth

  private JButton      btnReset    = new JButton();  // Reset To Original Depth
  private JButton      btnModify   = new JButton();  // Modify Depth Range

  // -- Level of Confidence Widgets

  private JRadioButton rbOFF       = new JRadioButton();
  private JRadioButton rbON        = new JRadioButton();

  // -- Datum By Widgets

  private JRadioButton rbByLog     = new JRadioButton();
  private JRadioButton rbByElev    = new JRadioButton();
  private JRadioButton rbByHorizon = new JRadioButton();

  private JTextField   txtHorizon  = new JTextField();

  // -- Tabbed Pane

  private JTabbedPane tabbedPane = new JTabbedPane();

  // -- Menus

  // .. File Button Menu

  private JMenuItem mSave = null; // Save Profile Data
  private JMenuItem mPDF  = null; // Create a PDF of Profile Plot
  private JMenuItem mExit = null; // Exit Profile Plot

  // -- Menus

  // .. Create a new Member Button Menu

  private JMenuItem mStrat  = null; // Create a new member

  // .. Help Button Menu

  private JMenuItem mHelp = null; // Help

  // .. Depth Scale Menu

  private static final String scale[] =
  {
      "1 ft/in",   "2 ft/in",  "5 ft/in",
     "10 ft/in",  "20 ft/in", "50 ft/in",
    "100 ft/in", "200 ft/in"
  };

  private JRadioButtonMenuItem  m1   = null;  // Display Plot at   1 ft / in
  private JRadioButtonMenuItem  m2   = null;  // Display Plot at   2 ft / in
  private JRadioButtonMenuItem  m5   = null;  // Display Plot at   5 ft / in
  private JRadioButtonMenuItem  m10  = null;  // Display Plot at  10 ft / in
  private JRadioButtonMenuItem  m20  = null;  // Display Plot at  20 ft / in
  private JRadioButtonMenuItem  m50  = null;  // Display Plot at  50 ft / in
  private JRadioButtonMenuItem  m100 = null;  // Display Plot at 100 ft / in
  private JRadioButtonMenuItem  m200 = null;  // Display Plot at 200 ft / in

  /** Constructor xsectionControlFrame()
   *  <p> This is the Constructor for this class.
   *  @param stStruct   = Global Color Lithology Data Structure
   */

  public xsectionControlFrame(Observable pNotifier, iqstratStruct stStruct)
  {
    try
    {
	  this.pNotifier = pNotifier;
      this.stStruct  = stStruct;

      dStart     = stStruct.depthStart;
      dEnd       = stStruct.depthEnd;

      dStartOrig = stStruct.depthStart;
      dEndOrig   = stStruct.depthEnd;

      jbInit();

      addWindowListener( new xsectionControlFrame_WindowListener() );

      pnlMemory.setMemory();
//      setInitialConfiguration( iConfig );

//      plot();  // Display the plot panel
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel pnlTop          = new JPanel();
    JPanel pnlRange        = new JPanel();
    JPanel pnlScale        = new JPanel();
    JPanel pnlEnterDepth   = new JPanel();
    JPanel pnlDepthButtons = new JPanel();
    JPanel pnlStart        = new JPanel();
    JPanel pnlEnd          = new JPanel();

    JPanel pnlLevel        = new JPanel();

    JPanel pnlDatum        = new JPanel();
    JPanel pnlHorizon      = new JPanel();
    JPanel pnlByDatum      = new JPanel();

	JPanel pnlCenter       = new JPanel();

    ButtonGroup groupDatum = new ButtonGroup();
    ButtonGroup groupLevel = new ButtonGroup();

    TitledBorder titledBorderDepth = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Depth Range");

    TitledBorder titledBorderStart = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Start Depth");

    TitledBorder titledBorderEnd = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "End Depth");

    TitledBorder titledBorderLevel = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Show Top Confidence Level");

    TitledBorder titledBorderDatum = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Datum");

    TitledBorder titledBorderHorizon = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Horizon:");

    // Build Menus

    JMenuBar menuBar    = new JMenuBar();
    JMenu    menuFile   = new JMenu("File");
    JMenu    menuScale  = new JMenu("Depth Scale");
    JMenu    menuNew    = new JMenu("Adding New Strat Units");
    JMenu    menuHelp   = new JMenu("Help");
    ButtonGroup groupDepth = new ButtonGroup();

    // -- File Menu Items

    mSave = new JMenuItem("Save Data");
    mSave.addActionListener(this);

    mPDF = new JMenuItem("Create PNG Image of Plot");
    mPDF.addActionListener(this);

    mExit = new JMenuItem("Exit");
    mExit.addActionListener(this);

//    menuFile.add( mSave );
//    menuFile.addSeparator();
    menuFile.add( mPDF );
    menuFile.addSeparator();
    menuFile.add( mExit );

    // -- Adding New Stratigraphic Units Menu Item

    mStrat = new JMenuItem("Add Stratigraphic Unit");
    mStrat.addActionListener(this);
    menuNew.add(mStrat);

    // -- Help Menu Item

    mHelp = new JMenuItem("Help");
    mHelp.addActionListener(this);
    menuHelp.add(mHelp);

    // -- Depth Scale Menu Items

    m1   = new JRadioButtonMenuItem("1 ft / in");
    m1.setHorizontalTextPosition(JMenuItem.RIGHT);
    m1.addActionListener(this);
    groupDepth.add(m1);

    m2   = new JRadioButtonMenuItem("2 ft / in");
    m2.setHorizontalTextPosition(JMenuItem.RIGHT);
    m2.addActionListener(this);
    groupDepth.add(m2);

    m5   = new JRadioButtonMenuItem("5 ft / in");
    m5.setHorizontalTextPosition(JMenuItem.RIGHT);
    m5.addActionListener(this);
    groupDepth.add(m5);

    m10  = new JRadioButtonMenuItem("10 ft / in");
    m10.setHorizontalTextPosition(JMenuItem.RIGHT);
    m10.addActionListener(this);
    groupDepth.add(m10);

    m20  = new JRadioButtonMenuItem("20 ft / in");
    m20.setHorizontalTextPosition(JMenuItem.RIGHT);
    m20.addActionListener(this);
    groupDepth.add(m20);

    m50  = new JRadioButtonMenuItem("50 ft / in");
    m50.setHorizontalTextPosition(JMenuItem.RIGHT);
    m50.addActionListener(this);
    groupDepth.add(m50);

    m100 = new JRadioButtonMenuItem("100 ft / in");
    m100.setSelected(true);
    m100.setHorizontalTextPosition(JMenuItem.RIGHT);
    m100.addActionListener(this);
    groupDepth.add(m100);

    m200 = new JRadioButtonMenuItem("200 ft / in");
    m200.setHorizontalTextPosition(JMenuItem.RIGHT);
    m200.addActionListener(this);
    groupDepth.add(m200);

    menuScale.add(m1);
    menuScale.add(m2);
    menuScale.add(m5);
    menuScale.add(m10);
    menuScale.add(m20);
    menuScale.add(m50);
    menuScale.add(m100);
    menuScale.add(m200);

    // -- Attach menu items to Menu Bar

    menuBar.add(menuFile);
    menuBar.add(menuScale);
//    menuBar.add(menuNew);
    menuBar.add(menuHelp);

    this.setJMenuBar(menuBar);

    // Create an Observable

    notifier = new xsectionControlFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Frame Widgets

    this.setTitle("Plot Control");
    this.getContentPane().setLayout(new BorderLayout());

    // -- Top Panel

    pnlTop.setLayout(new BorderLayout());

    // -- Depth Range Panel

    pnlRange.setBorder(titledBorderDepth);
    pnlRange.setLayout(new BorderLayout());

    // .. Enter Depth Panel

    pnlEnterDepth.setLayout(new GridLayout());

    // .... Panel Depth Scale

    pnlScale.setLayout(new BorderLayout());

    lblScale.setFont(new java.awt.Font("Dialog", 1, 13));
    lblScale.setHorizontalAlignment(SwingConstants.CENTER);
    lblScale.setText("Depth Scale: "+ scale[iScale]);

    // .... Starting Depth Panel

    pnlStart.setBorder(titledBorderStart);
    pnlStart.setLayout(new BorderLayout());

    txtStart.setText("" + dStart);
    txtStart.setHorizontalAlignment(SwingConstants.TRAILING);
    txtStart.addFocusListener(new xsectionControlFrameFocusAdapter(this));

    // .... Ending Depth Panel

    pnlEnd.setBorder(titledBorderEnd);
    pnlEnd.setLayout(new BorderLayout());

    txtEnd.setText("" + dEnd);
    txtEnd.setHorizontalAlignment(SwingConstants.TRAILING);
    txtEnd.addFocusListener(new xsectionControlFrameFocusAdapter(this));

    // .. Depth Range Buttons

    pnlDepthButtons.setBorder(BorderFactory.createEtchedBorder());
    pnlDepthButtons.setLayout(new GridLayout());

    btnReset.setFont(new java.awt.Font("Dialog", 1, 11));
    btnReset.setText("Reset");
    btnReset.addActionListener(this);

    btnModify.setFont(new java.awt.Font("Dialog", 1, 11));
    btnModify.setText("Modify");
    btnModify.addActionListener(this);

    // -- Level of Confidence Panel

    pnlLevel.setLayout(new GridLayout());
    pnlLevel.setBorder(titledBorderLevel);

    rbOFF.setFont(new java.awt.Font("Dialog", 1, 11));
    rbOFF.setSelected(true);
    rbOFF.setText("Off");
    rbOFF.addActionListener(this);

    rbON.setFont(new java.awt.Font("Dialog", 1, 11));
    rbON.setText("On");
    rbON.addActionListener(this);

    // -- Datum By Panel

    pnlDatum.setLayout(new BorderLayout());
    pnlDatum.setBorder(titledBorderDatum);

    // -- Datum By Panel

    pnlByDatum.setLayout(new GridLayout());

    rbByLog.setFont(new java.awt.Font("Dialog", 1, 11));
    rbByLog.setHorizontalAlignment(SwingConstants.LEFT);
    rbByLog.setSelected(true);
    rbByLog.setText("By Log Depth");
    rbByLog.addActionListener(this);

    rbByElev.setFont(new java.awt.Font("Dialog", 1, 11));
    rbByElev.setHorizontalAlignment(SwingConstants.LEFT);
    rbByElev.setText("By Elevation");
    rbByElev.addActionListener(this);

    rbByHorizon.setFont(new java.awt.Font("Dialog", 1, 11));
    rbByHorizon.setHorizontalAlignment(SwingConstants.LEFT);
    rbByHorizon.setText("By Horizon");
    rbByHorizon.addActionListener(this);

    // .. Datum By Horizon Name Panel

    pnlHorizon.setBorder(titledBorderHorizon);
    pnlHorizon.setLayout(new BorderLayout());

    txtHorizon.setText( sDatum );
    txtHorizon.setEditable( false );
    txtHorizon.addFocusListener(new xsectionControlFrameFocusAdapter(this));

    // -- Center Panel

//    pnlCenter.setLayout(new GridLayout(2,1));
    pnlCenter.setLayout(new BorderLayout());

    // Tabbed Pane

    tabbedPane.setFont(new java.awt.Font("Dialog", 1, 14));
    tabbedPane.setTabPlacement( JTabbedPane.TOP );
    tabbedPane.setTabLayoutPolicy( JTabbedPane.SCROLL_TAB_LAYOUT );

    pTrack = new xsectionDataPanel[stStruct.iWells]; //2];
//    pTrack[0] = new xsectionDataPanel( notifier, stStruct, 1 );
//    pTrack[1] = new xsectionDataPanel( notifier, stStruct, 2 );

    for (int i=0; i<stStruct.iWells; i++)
    {
      pTrack[i] = new xsectionDataPanel( notifier, stStruct, i );
      tabbedPane.add(pTrack[i],  "(" + i + ")");
	}

    tabbedPane.setSelectedIndex(0);

    // Add Widgets to Frame

    this.getContentPane().add(pnlTop,    BorderLayout.NORTH);
    pnlTop.add(pnlRange,                 BorderLayout.NORTH);
    pnlRange.add(pnlScale,               BorderLayout.NORTH);
    pnlScale.add(lblScale,               BorderLayout.NORTH);

    pnlRange.add(pnlEnterDepth,          BorderLayout.CENTER);
    pnlEnterDepth.add(pnlStart,          null);
    pnlStart.add(txtStart,               BorderLayout.CENTER);

    pnlEnterDepth.add(pnlEnd,            null);
    pnlEnd.add(txtEnd,                   BorderLayout.CENTER);

    pnlRange.add(pnlDepthButtons,        BorderLayout.SOUTH);
    pnlDepthButtons.add(btnReset,        null);
    pnlDepthButtons.add(btnModify,       null);

    pnlTop.add(pnlLevel,                 BorderLayout.CENTER);
    pnlLevel.add(rbOFF,                  null);
    pnlLevel.add(rbON,                   null);

    groupLevel.add(rbOFF);
    groupLevel.add(rbON);

    pnlTop.add(pnlDatum,                 BorderLayout.SOUTH);
    pnlDatum.add(pnlByDatum,             BorderLayout.NORTH);
    pnlByDatum.add(rbByLog,              null);
    pnlByDatum.add(rbByHorizon,          null);
    pnlByDatum.add(rbByElev,             null);

    pnlDatum.add(pnlHorizon,             BorderLayout.CENTER);
    pnlHorizon.add(txtHorizon,           BorderLayout.NORTH);

    groupDatum.add(rbByLog);
    groupDatum.add(rbByElev);
    groupDatum.add(rbByHorizon);

    this.getContentPane().add(pnlCenter, BorderLayout.CENTER);
    pnlCenter.add(tabbedPane,            BorderLayout.CENTER);
//    pnlCenter.add(pTrack[0],             null);
//    pnlCenter.add(pTrack[1],             null);

    this.getContentPane().add(pnlMemory, BorderLayout.SOUTH);

    // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    iScreenWidth = d.width;
    iScreenHigh  = d.height-50;

    this.setSize(new Dimension(iControlWidth, d.height-50)); //600
//    this.setSize(new Dimension(iControlWidth, 600));
    this.setLocation(1,1);

//    this.setResizable(false);
    this.setVisible(true);
  }

  /** METHOD iqstratControlPlotFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class xsectionControlFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    pNotifier = null; // Observable Object this Frame

    // Global Variables

    notifier = null;

    // -- By Datum

    sDatum   = null;  // Datum by Horizon Name

    // -- Plot Data

    iLASSize = null;

    if (stList != null)
      stList.delete();
    stList = null; // Cross Section List Structure

    // --  Panel Data

    if (pTrack != null)
    {
      for (int i=0; i<stStruct.iWells; i++)
      {
	    if (pTrack[i] != null)
	      pTrack[i].close();
        pTrack[i] = null;
      }
	}
    pTrack = null;

    if (plotFrame != null)
      plotFrame.close();
    plotFrame = null; // Cross Section Plot Frame

	CloseEditTops();

    if (pSearch != null)
      pSearch.close();
    pSearch = null;

//    if (p3File != null)
//      p3File.close();
//    p3File = null;

//    if (pChart != null)
//      pChart.close();
//    pChart = null; // User Define Strat Units Dialog

    // -- Depth Range Widgets

    txtStart     = null; // Starting Depth
    txtEnd       = null; // Ending Depth

    btnReset     = null; // Reset To Original Depth
    btnModify    = null; // Modify Depth Range

    // -- Tabbed Pane

    tabbedPane   = null;

    // -- Level of Confidence Widgets

    rbOFF        = null;
    rbON         = null;

    // -- Datum By Widgets

    rbByLog      = null;
    rbByElev     = null;
    rbByHorizon  = null;

    txtHorizon   = null;

    // Menus

    mSave  = null; // Save Profile Data
    mPDF   = null; // PDF of Profile Plot
    mExit  = null; // Exit
    mStrat = null; // New Stratigraphic Units
    mHelp  = null; // Help

    // -- Depth Scale Buttons

    lblScale   = null;

    m1     = null; //   1 foot / inch
    m2     = null; //   2 feet / inch
    m5     = null; //   5 feet / inch
    m10    = null; //  10 feet / inch
    m20    = null; //  20 feet / inch
    m50    = null; //  50 feet / inch
    m100   = null; // 100 feet / inch
    m200   = null; // 200 feet / inch

    if (pnlMemory != null)
      pnlMemory.close();
    pnlMemory = null;

    stStruct  = null;

    if (pNotifier != null)
      pNotifier.notifyObservers( new String( "FREE MEMORY" ) );

    dispose();
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData()
   * <p> This method will set the extra data structures
   * @param iWell       = Well Number; 0=Reference, 1=Editable Tops
   * @param stHeader    = Header Information Data Structure
   * @param stFiles     = LAS File Information List Strcuture
   * @param stLAS       = LAS File Data Structure
   * @param stStrat     = Stratigraphic Units Data List Structure
   * @param stCore      = Measured Core Data
   * @param stColumn    = Rock Lithology Column Data List Structure
   * @param stPHI       = Rock Porosity Type Data List Structure
   * @param stSedStruct = Sedimentary Structure Data List Structure
   * @param stFossil    = General Fossil Data List Structure
   * @param stColor     = Rock Color Data List Structure
   * @param stRemarks   = Measured Section Description Data List Structure
   */

  public void setData( int                     iWell,
	                   iqstratHeadersStruct    stHeader,
                       cmnLASFileListStruct    stFiles,
                       lasFileDataStruct       stLAS,
                       stratListStruct         stStrat,
                       // Measured Section Data
                       rockDataListStruct      stCore,
                       rockColumnListStruct    stColumn,
                       phiListStruct           stPHI,
                       sedimentaryListStruct   stSedStruct,
                       fossilListStruct        stFossil,
                       rockColorListStruct     stColor,
                       iqstratRemarkListStruct stRemarks )
  {
	int iTest = -1;

    if (pnlMemory != null) pnlMemory.freeMemory();

	if (stList == null)
	{
	  stList           = new xsectionDataListStruct();
	  stList.iCount    = stStruct.iWells; //2;
	  stList.stItem    = new xsectionDataStruct[stStruct.iWells]; //2];
	  for (int i=0; i<stStruct.iWells; i++)
	  {
	    stList.stItem[i] = new xsectionDataStruct();
	  }
	}

	stList.stItem[iWell].stHeader      = stHeader;    // Header Information
    stList.stItem[iWell].stControl     = new iqstratControlStruct();

    // LAS Data

	stList.stItem[iWell].stLAS          = stLAS;       // LAS File Data
    stList.stItem[iWell].stFiles        = stFiles;     // LAS File Information Data

    stList.stItem[iWell].stLASLithology = computeLithologyFromLAS(
                                            stList.stItem[iWell].stLAS,
                                            stStruct.stSymbols );

    // Horizons Data

	stList.stItem[iWell].stStrat        = stStrat;     // Stratigraphic Units

    // Rock Data - Measured Sections

    stList.stItem[iWell].stCore         = stCore;      // Core Data

	stList.stItem[iWell].stColumn       = stColumn;    // Rock Column

	stList.stItem[iWell].stPHI          = stPHI;       // Porosity Type Data
	stList.stItem[iWell].stFossil       = stFossil;    // Rock Fossils
	stList.stItem[iWell].stSedimentary  = stSedStruct; // Rock Sedimentary
	stList.stItem[iWell].stColor        = stColor;     // Rock Color

	stList.stItem[iWell].stRemarks      = stRemarks;   // Remarks (Raw Data)

    // Set Depth & Scale of Plot

    setDepthScale( stLAS, stRemarks );

    // Build the Well Info Panel & Available Plot Tracks Panel

    if (pTrack[iWell] != null)
    {
	  pTrack[iWell].setData( iWell, stHeader);
      pTrack[iWell].setData( stLAS,
                             stStrat,
                             stCore,
                             // Measured Section Data
                             stColumn,
                             stPHI,
                             stSedStruct,
                             stFossil,
                             stColor,
                             stRemarks );

      stList.stItem[iWell].stControl.iTracks = pTrack[iWell].getSelected();

      iTest = pTrack[iWell].getThinCurve();
      if (iTest > -1)
      {
        stList.stItem[iWell].stControl.iThin    = cmn.cmnStruct._YES;
        stList.stItem[iWell].stControl.iTHINPHI = pTrack[iWell].getThinCurve();
	  }
	}

    if (pnlMemory != null) pnlMemory.freeMemory();
  }

  /** Method setDepthScale()
   *  <p> This method will set the depth range & scale depending
   *      on the data loaded.
   * @param stLAS       = LAS File Data Structure
   * @param stRemarks   = Measured Section Description Data List Structure
   */

  private void setDepthScale( lasFileDataStruct       stLAS,
                              iqstratRemarkListStruct stRemarks )
  {
	double diff = 0.0;

    if (stRemarks != null)
    {
      if (dStartOrig == dEndOrig)
      {
        dStartOrig = stRemarks.depthStart;
        dEndOrig   = stRemarks.depthEnd;
      }

      if (dStartOrig > stRemarks.depthStart)
        dStartOrig = stRemarks.depthStart;

      if (dEndOrig < stRemarks.depthEnd)
        dEndOrig   = stRemarks.depthEnd;
	}

	if (stLAS != null)
	{
      if (dStartOrig == dEndOrig)
      {
        dStartOrig = stLAS.depthStart;
        dEndOrig   = stLAS.depthEnd;
	  }

      if (dStartOrig > stLAS.depthStart)
        dStartOrig = stLAS.depthStart;

      if (dEndOrig < stLAS.depthEnd)
        dEndOrig   = stLAS.depthEnd;
	}

    diff = dEndOrig - dStartOrig;
    if (diff < 10)
      setScale( iqstrat.iqstratTracksStruct.SCALE_1 );
    else if (diff < 20)
      setScale( iqstrat.iqstratTracksStruct.SCALE_2 );
    else if (diff < 50)
      setScale( iqstrat.iqstratTracksStruct.SCALE_5 );
    else if (diff < 100)
      setScale( iqstrat.iqstratTracksStruct.SCALE_10 );
    else if (diff < 200)
      setScale( iqstrat.iqstratTracksStruct.SCALE_20 );
    else if (diff < 500)
      setScale( iqstrat.iqstratTracksStruct.SCALE_50 );
    else
      setScale( iqstrat.iqstratTracksStruct.SCALE_100 );

    dStart = dStartOrig;
    dEnd   = dEndOrig;

    txtStart.setText("" + dStart);
    txtEnd.setText(""   + dEnd);
  }

  /** Method setScale( int iScale )
   * <p> This method will change the depth Scale.
   * @param iScale = The Depth Scale
   */

  private void setScale( int iScale )
  {
    m1.setSelected(false);
    m2.setSelected(false);
    m5.setSelected(false);
    m10.setSelected(false);
    m20.setSelected(false);
    m50.setSelected(false);
    m100.setSelected(false);
    m200.setSelected(false);

    switch (iScale)
    {
      case iqstrat.iqstratTracksStruct.SCALE_200:
        m200.setSelected(true);
        break;
      case iqstrat.iqstratTracksStruct.SCALE_100:
        m100.setSelected(true);
        break;
      case iqstrat.iqstratTracksStruct.SCALE_50:
        m50.setSelected(true);
        break;
      case iqstrat.iqstratTracksStruct.SCALE_20:
        m20.setSelected(true);
        break;
      case iqstrat.iqstratTracksStruct.SCALE_10:
        m10.setSelected(true);
        break;
      case iqstrat.iqstratTracksStruct.SCALE_5:
        m5.setSelected(true);
        break;
      case iqstrat.iqstratTracksStruct.SCALE_2:
        m2.setSelected(true);
        break;
      case iqstrat.iqstratTracksStruct.SCALE_1:
        m1.setSelected(true);
        break;
    }

    changeScale( iScale );

    if (plotFrame != null) plotFrame.setScale( iScale, dEnd, dStart );
  }

  /** Method changeScale()
   * <p> This method will change the depth Scale.
   * @param iScale = The Depth Scale
   */

  private void changeScale(int iScale)
  {
    this.iScale = iScale;

    lblScale.setText("Depth Scale: "+ scale[iScale]);

    if (plotFrame != null) plotFrame.setScale( iScale, dEnd, dStart );
  }

  /** Method setDepth()
   * <p> This method will set the Depth Range
   */

  private void setDepth()
  {
    double dTemp = 0.0;

    dStart = cmn.cmnString.stringToDouble(txtStart.getText());
    dEnd   = cmn.cmnString.stringToDouble(txtEnd.getText());

//    if (dStart > dEnd)
//    {
//      dTemp  = dEnd;
//      dEnd   = dStart;
//      dStart = dTemp;
//    }

    // Set the Starting & Ending Depth in the Control Panel

//    txtStart.setText("" + dStart);
//    txtEnd.setText(""   + dEnd);

    if (plotFrame != null) plotFrame.setScale(iScale, dEnd, dStart);
  }

  /** Method setDefaultDepth()
   * <p> This method will reset the depth to the original LAS Depth
   */

  private void setDefaultDepth()
  {
    txtStart.setText("" + dStartOrig);
    txtEnd.setText(""   + dEndOrig);

    dStart = dStartOrig;
    dEnd   = dEndOrig;

    if (plotFrame != null) plotFrame.setScale(iScale, dEnd, dStart);
  }


  /** Method setDatum( int iDatum )
   * <p> This method will set the datum by identifier button
   * @param iDatum = datum by identifier
   */

  private void setDatum( int iDatum )
  {
    String sByDatum = new String("");
    double dElev    = 0.0;

    lasFileDataStruct       stLAS     = null;
    iqstratRemarkListStruct stRemarks = null;

    this.iDatum = iDatum;

    txtHorizon.setEditable( false );
    txtHorizon.setText( sByDatum );

    if (iDatum == iqstrat.iqstratTracksStruct._BY_DATUM)
    {
      sByDatum = new String( sDatum );

      txtHorizon.setText( sDatum );
      txtHorizon.setEditable( true );
    }

    dStartOrig = 0.0;
    dEndOrig   = 0.0;

    for (int i=0; i<stList.iCount; i++)
    {
	  stLAS      = stList.stItem[i].stLAS;
	  stRemarks  = stList.stItem[i].stRemarks;

      if (iDatum == iqstrat.iqstratTracksStruct._BY_LOG)
      {
        if (stRemarks != null)
        {
          if (dStartOrig == dEndOrig)
          {
            dStartOrig = stRemarks.depthStart;
            dEndOrig   = stRemarks.depthEnd;
          }

          if (dStartOrig > stRemarks.depthStart)
            dStartOrig = stRemarks.depthStart;

          if (dEndOrig < stRemarks.depthEnd)
            dEndOrig   = stRemarks.depthEnd;
	    }

	    if (stLAS != null)
	    {
          if (dStartOrig == dEndOrig)
          {
            dStartOrig = stLAS.depthStart;
            dEndOrig   = stLAS.depthEnd;
	      }

          if (dStartOrig > stLAS.depthStart)
            dStartOrig = stLAS.depthStart;

          if (dEndOrig < stLAS.depthEnd)
            dEndOrig   = stLAS.depthEnd;
	    }
	  }
	  else
	  {
        dElev = xsection.xsectionDataUtility.getElevation(
                   iDatum, sDatum, stList.stItem[i] );

        if (stRemarks != null)
        {
          if (dStartOrig == dEndOrig)
          {
            dStartOrig = dElev - stRemarks.depthStart;
            dEndOrig   = dElev - stRemarks.depthEnd;
          }

          if (dStartOrig > dElev - stRemarks.depthStart)
            dStartOrig = dElev - stRemarks.depthStart;

          if (dEndOrig < dElev - stRemarks.depthEnd)
            dEndOrig   = dElev - stRemarks.depthEnd;
	    }

	    if (stLAS != null)
	    {
          if (dStartOrig == dEndOrig)
          {
            dStartOrig = dElev - stLAS.depthStart;
            dEndOrig   = dElev - stLAS.depthEnd;
	      }

          if (dStartOrig < dElev - stLAS.depthStart)
            dStartOrig = dElev - stLAS.depthStart;

          if (dEndOrig > dElev - stLAS.depthEnd)
            dEndOrig   = dElev - stLAS.depthEnd;
	    }
	  }
	}

	dStart = dStartOrig;
	dEnd   = dEndOrig;

	txtStart.setText("" + dStart);
	txtEnd.setText(""   + dEnd);

    if (plotFrame != null)
    {
      plotFrame.setDatum( iDatum, sDatum );
	  plotFrame.setScale( iScale, dEnd, dStart );
	}
  }

  /* ===================================================================== *
   * ---------------------------- COMPUTE -------------------------------- *
   * ===================================================================== */

  /** Method computeLithologyFromLAS()
   * <p> This method will compute the Lithology from measured LAS File data.
   *  @param  stLASFileData   = LAS File Data
   *  @param  stSymbols       = Lithology Symbols Data List Structure
   *  @return stRockLithology = Rock Lithology Data List Data Structure
   */

  public static lithologyListStruct computeLithologyFromLAS(
      lasFileDataStruct stLAS,
      lithologySymbolsListStruct stSymbols )
  {
    lithologyListStruct st   = null;
    String              sKEY = "0";

    if (stLAS != null)
    {
      st = lith.lithology.lithologyUtility.buildNew(
             iqstrat.iqstratTracksStruct._SCALE_HALF_FOOT,
             stLAS.depthStart,
             stLAS.depthEnd);

      sKEY = new String(cmn.cmnString.UniqueName());
      for (int i=0; i<st.iCount; i++)
      {
        st.stItem[i].sKEY = new String(sKEY + "_" + i);
      }

      st = lith.lithology.lithologyUtility.computeLithology( stLAS, stSymbols, st );
    }

    return (st);
  }

  /* ===================================================================== *
   * ----------------------------- PLOT ---------------------------------- *
   * ===================================================================== */

  /** Method plot()
   *  <p> This method will plot the cross section
   */

  public void plot()
  {
    if (plotFrame != null)
      plotFrame.close();
    plotFrame = null; // Cross Section Plot Frame

	stStruct.depthStart = dStart;
	stStruct.depthEnd   = dEnd;
	stStruct.iScale     = iScale;

	plotFrame = new xsectionPlotFrame( notifier, stStruct, iControlWidth );

    setDatum( iDatum );

	plotFrame.setData( stList );
	plotFrame.setScale( iScale, dEnd, dStart );
  }

  /* ===================================================================== *
   * --------------------------- EDIT TOPS ------------------------------- *
   * ===================================================================== */

  /** Method DisplayEditTops()
   *  <p> This method will display the Edit Tops Dialog
   *  @param iWell   = the well selected
   *  @param sZeller = Zeller KID
   *  @param depth   = the Depth Clicked
   */

  private void DisplayEditTops(int iWell, String sZeller, double depth)
  {
	CloseEditTops();

	pTops = new xsectionTypeLogTopsFrame( notifier, stStruct, stList,
	                                      iWell, sZeller, depth );
	if (iWell == 1) { plotFrame.setWorkingDepth( 1, depth ); }

    // Clear Temporary Fields

    stStruct.iCopy     = 0;
    stStruct.sTopsName = new String("");
  }

  /** Method CloseEditTops()
   *  <p> This method will close the Edit Tops Dialog
   */

  private void CloseEditTops()
  {
	if (pTops != null)
	  pTops.close();
	pTops = null;
  }

  /* ===================================================================== *
   * ------------------------ DISPLAY DIALOGS ---------------------------- *
   * ===================================================================== */

  /** Method displayChart()
   * <p> This method will display either the Kansas Stratigraphic Chart
   */

  private void displayChart()
  {
//    if (pChart != null)
//      pChart.close();
//    pChart = null;

//    pChart = new stratPlotFrame( notifier, stStruct.stICS, stStruct.stKGS );
  }

  /** Method save()
   * <p> This method will save the data to the PC and/or KGS database.
   *

  private void save()
  {
	xsectionDataStruct st = null;

    if (p3File != null)
      p3File.close();
    p3File = null;

    if (stList != null)
    {
	  if (stList.iCount == 1)
	  {
		if (stList.stItem[0] != null)
		{
		  st = stList.stItem[0];
          p3File = new las3FileFrame( notifier,       // Observables Object
                                      stStruct,       // Global IQSTRAT Data Struct
                                      st.stFiles,     // LAS File Information
                                      st.stHeader,    // Header Information
                                      st.stLAS,       // LAS File Data
                                      st.stCore,      // Core Data
                                      st.stStrat,     // Stratigraphic Units List
                                      st.stControl,   // Plot Control Data
                                      st.stRemarks ); // Remarks Data List Structure
	    }
	  }
    }
  }
*/
  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
	String sPath = "";

    if (pnlMemory != null) pnlMemory.freeMemory();

    if (event.getSource() == btnReset)  { setDefaultDepth(); }
    if (event.getSource() == btnModify) { setDepth(); }

    // - Level of Confidence Widgets

    if (event.getSource() == rbOFF)
    {
	  if (plotFrame != null) { plotFrame.setLevel(cmn.cmnStruct.OFF); }
	}

    if (event.getSource() == rbON)
    {
	  if (plotFrame != null) { plotFrame.setLevel(cmn.cmnStruct.ON); }
	}

    // - Datum By Widgets

    if (event.getSource() == rbByLog)
    {
      setDatum( iqstrat.iqstratTracksStruct._BY_LOG );
    }
    if (event.getSource() == rbByElev)
    {
      setDatum( iqstrat.iqstratTracksStruct._BY_ELEV );
    }
    if (event.getSource() == rbByHorizon)
    {
      setDatum( iqstrat.iqstratTracksStruct._BY_DATUM );
    }

    // Menus

    // -- Help Button Menu

    if (event.getSource() == mHelp )
    {
      util.BrowserControl.displayURL( cmn.cmnStruct.HELP );
	}

    // -- Depth Scale Menu

    if (event.getSource() == m200)
    {
      changeScale(iqstrat.iqstratTracksStruct.SCALE_200);
    }
    if (event.getSource() == m100)
    {
      changeScale(iqstrat.iqstratTracksStruct.SCALE_100);
    }
    if (event.getSource() == m50)
    {
      changeScale(iqstrat.iqstratTracksStruct.SCALE_50);
    }
    if (event.getSource() == m20)
    {
      changeScale(iqstrat.iqstratTracksStruct.SCALE_20);
    }
    if (event.getSource() == m10)
    {
      changeScale(iqstrat.iqstratTracksStruct.SCALE_10);
    }
    if (event.getSource() == m5)
    {
      changeScale(iqstrat.iqstratTracksStruct.SCALE_5);
    }
    if (event.getSource() == m2)
    {
      changeScale(iqstrat.iqstratTracksStruct.SCALE_2);
    }
    if (event.getSource() == m1)
    {
      changeScale(iqstrat.iqstratTracksStruct.SCALE_1);
    }

    // -- File Button Menu

    if (event.getSource() == mPDF  )
    {
	  sPath               = kgs.io.kgsIOUser.getHomeDir();
      stStruct.sDirectory = new String( sPath );
      stStruct.sName      = new String( "Type_Log" );

      pSearch = new guiSearchDirectoryFrame(notifier, sPath, "Type_Log", ".png");
    }

    if (event.getSource() == mSave) {} //  { save(); }

    if (event.getSource() == mStrat  ) { displayChart(); }

    if (pnlMemory != null) pnlMemory.freeMemory();

    // Exit

    if (event.getSource() == mExit)  { close(); }

    if (pNotifier != null)
      pNotifier.notifyObservers( new String( "FREE MEMORY" ) );
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");

    if (pnlMemory != null) pnlMemory.freeMemory();

    if (e.getSource() == txtStart)
    {
      iNumeric = 1;
      sTemp = txtStart.getText();
      sMessage = new String( "Start Depth Value is a Numeric Field" );
    }

    if (e.getSource() == txtEnd)
    {
      iNumeric = 1;
      sTemp = txtEnd.getText();
      sMessage = new String( "End Depth Value is a Numeric Field" );
    }

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);
      }
      else
      {
        if (e.getSource() == txtStart)
        {
          dStart = cmn.cmnString.stringToDouble(txtStart.getText());
        }

        if (e.getSource() == txtEnd)
        {
          dEnd = cmn.cmnString.stringToDouble(txtEnd.getText());
        }
      }
    }
    else
    {
      if (e.getSource() == txtHorizon)
      {
		if (xsection.xsectionDataUtility.isCommon( txtHorizon.getText(), stList ))
		{
          sDatum = new String(txtHorizon.getText());
          setDatum( iDatum );
	    }
	    else
	    {
  		  sMessage = "Horizon must be saved in both Wells to Datum By.";
		  JOptionPane.showMessageDialog(
		          (Component) null, sMessage, "ERROR", JOptionPane.ERROR_MESSAGE);
		}
      }
    }

    if (pnlMemory != null) pnlMemory.freeMemory();

    if (pNotifier != null)
      pNotifier.notifyObservers( new String( "FREE MEMORY" ) );
  }

  /* ===================================================================== *
   * ----------------------- OBSERVABLE ACTIONS -------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
	int i,j;
    String sArg        = new String((String) arg);
    int    iWell       = -1;
    String sWell       = "0";
    String sZeller     = "0";
    String sTops       = "";
    double dSelect     = 0.0;
    String sError      = "";
    int    iCont       = 0;
   	stratListStruct st = null;

    if (pnlMemory != null) pnlMemory.freeMemory();

	if (sArg.equals("Available Track Panel Changed"))
	{
	  for (i=0; i<stStruct.iWells; i++)
	  {
		plotFrame.setSelected( i, pTrack[i].getSelected());
	  }
	}

    if (sArg.equals("Get LAS Track Size"))
    {
	  for (i=0; i<stStruct.iWells; i++)
	  {
  	    iLASSize[i] = pTrack[i].getLASTrackSize();
	  }
//	  iLASSize[0] = pTrack[0].getLASTrackSize();
//	  iLASSize[1] = pTrack[1].getLASTrackSize();

	  plotFrame.setLASTrackSize( iLASSize );
	}

    if (sArg.equals("Not Owner"))
    {
	  iWell   = 1;
	  sZeller = pTops.getZellerKID();
	  dSelect = pTops.getWorkingDepth();

	  DisplayEditTops( iWell, sZeller, dSelect );
	}

	if (sArg.equals("Edit Tops"))
	{
	  iWell   = plotFrame.getWell();
	  sZeller = plotFrame.getZeller();
	  dSelect = plotFrame.getDepth();

//System.out.println("EditTops <"+sZeller+"> <"+dSelect+"> "+ iWell);

	  DisplayEditTops( iWell, sZeller, dSelect );
	}

	if (sArg.equals("Datum By"))
	{
//   	  if (pTops != null)
//   	  {
   	    sTops = plotFrame.getZeller();
//		sTops = pTops.getDatumBy();

		if (xsection.xsectionDataUtility.isCommon( sTops, stList ))
		{
		  iDatum = iqstrat.iqstratTracksStruct._BY_DATUM;
		  rbByHorizon.setSelected( true );

//		  sDatum = new String( pTops.getDatumBy() );
		  sDatum = new String( sTops );
          setDatum( iDatum );
	    }
	    else
	    {
  		  sError = "Horizon must be saved in both Wells to Datum By.";
		  JOptionPane.showMessageDialog(
		          (Component) null, sError, "ERROR", JOptionPane.ERROR_MESSAGE);
		}
//	  }
	}

    if (sArg.equals("Set Working Depth"))
    {
	  if (pTops != null)
	  {
		plotFrame.setWorkingDepth( 1, pTops.getWorkingDepth() );
	  }
	}

	if (sArg.equals("Free Working Depth"))
	{
	  plotFrame.setWorkingDepth( 0, 0.0 );
	  plotFrame.setEvalDepth( 0, 0.0 );
	}

    if (sArg.equals("Set Evaluator Depth"))
    {
	  if (pTops != null)
	  {
		plotFrame.setEvalDepth( 1, pTops.getEvalDepth() );
	  }
	}

    // Copy & Paste Icon Button Action

    if (sArg.equals("Set Copy Depth"))
    {
	  if (pTops != null)
	  {
		sZeller = pTops.getZellerKID();
		sWell   = new String( stList.stItem[1].stHeader.sWELLKID );
	    st      = kgs.io.kgsIORead.ReadWellRegions2( sZeller, sWell,
 	                horizon.strat.io.ReadStratUnitsXMLFile.URL);

 	    iCont = 0;
        if (st != null)
        {
		  if (st.iCount > 0)
		  {
			iCont = 1;
		  }
	    }

        if (iCont == 0)
        {
		  pTops.setNote();

		  stStruct.iCopy     = 1;
          stStruct.sTopsName = new String( pTops.getName() );

		  plotFrame.setCopyDepth( 1, pTops.getZellerKID(), pTops.getName() );
		}
		else
		{
		  sError = "Can not copy Horizon, " + pTops.getName() + " already exists in Edit Well.";
		  JOptionPane.showMessageDialog(
		            (Component) null, sError, "ERROR", JOptionPane.ERROR_MESSAGE);
		}
	  }
	}

	if (sArg.equals("Source Tops Modified"))
	{
	  stratListStruct stStrat = null;

      if (stList != null)
      {
		if (stList.iCount > 1)
		{
		  if (stList.stItem[1] != null)
		  {
	        if (stList.stItem[1].stStrat != null)
	          stList.stItem[1].stStrat.delete();
	        stList.stItem[1].stStrat = null;

            stStrat = kgs.io.kgsIORead.ReadWellRegions0( stList.stItem[1].stHeader.sWELLKID,
                                                         horizon.strat.io.ReadStratUnitsXMLFile.URL);

            if (stStrat != null)
            {
              stStrat = horizon.strat.stratUtility.addICSArray( stStrat, stStruct.stICS );

              stStrat.sTOPS  = new String("Kansas Geological Survey Type Log"); // Formation Source
              stStrat.sTOPDR = new String("Log Depth");    // Tops Depth Reference
              stStrat.iSource = las3.las3Constants._SOURCE_KGS;
	        }

	        stList.stItem[1].stStrat = horizon.strat.stratUtility.transfer( stStrat );
		  }
		}
      }

      plotFrame.setData( stList );

	  pTops.close();
	}

	if (sArg.equals("Evaluator Tops Modified"))
	{
	  pTops.close();
	}

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        stStruct.sDirectory = new String( pSearch.getPath() );
        stStruct.sName      = new String( pSearch.getFile() );

        iqstrat.io.iqstratIO_Files.createImage( stStruct, plotFrame.getImage() );
        iqstrat.io.iqstratIO_Files.createHTML( stStruct );

        stStruct.sDirectory = new String( "" );
        stStruct.sName      = new String( "" );

        if (pSearch != null)
          pSearch.close();
        pSearch = null;
      }
	}

    if (sArg.equals("Directory Path Cancel"))
    {
      if (pSearch != null)
        pSearch.close();
      pSearch = null;
	}

    if (sArg.equals("Close Cross Section Plot")) { close();	}

    if (pnlMemory != null) pnlMemory.freeMemory();

    if (pNotifier != null)
      pNotifier.notifyObservers( new String( "FREE MEMORY" ) );
  }
}

/** CLASS xsectionControlFrameFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class xsectionControlFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  xsectionControlFrame adaptee;

  xsectionControlFrameFocusAdapter(xsectionControlFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/** CLASS xsectionControlFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class xsectionControlFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 10/31/2012
 *  @author  John Victorine
 */



