/*
 * @xsectionDataPanel.java Version 1.1 10/31/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package xsection.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import iqstrat.iqstratStruct;                  // GLOBAL IQSTRAT Data Structure
import iqstrat.iqstratHeadersStruct;           // IQSTRAT Headers Data Structure
import iqstrat.iqstratRemarkListStruct;        // Geologist Descriptions List Data Strucutre

import las.lasFileDataStruct;                  // LAS File Data Structure

import horizon.strat.stratListStruct;          // Stratigraphic Units List Structure

import lith.rock.rockColumnListStruct;         // Rock Column Data List Structure

import rock.rockDataListStruct;                // Rock Data List Data Structure

import rock.fossil.fossilListStruct;           // Fossil List Data Structure
import rock.sedimentary.sedimentaryListStruct; // Sed Structure List Data Structure
import rock.phi.phiListStruct;                 // Porosity List Data Strurture
import rock.color.rockColorListStruct;         // Rock Color List Data Structure

import xsection.xsectionDataStruct;            // Cross Section Data Structure
import xsection.gui.xsectionAvailablePanel;    // Available Plot Tracks

/** Class xsectionDataPanel
 *  <p> This file will display the data & summary of the data loaded.
 *
 *  @version 1.1 10/31/2012
 *  @author  John R. Victorine
 */

public class xsectionDataPanel extends JPanel implements ActionListener, Observer
{
  // Input Variables

  private Observable    pNotifier = null; // Observable Object this Frame
  private iqstratStruct stStruct  = null; // Global IQSTRAT Data Structure
  private int           iWell     = -1;   // Well Number
  // Global Variables

  private Observable    notifier  = null; // Observable Object this Frame

  private xsectionDataStruct     stData    = null; // Cross Section Data Structure
  private xsectionAvailablePanel pTracks   = null; // Available Plot Tracks

  private int iSelected[] = null; // Selected Tracks

  // LAS Data

  public lasFileDataStruct       stLAS       = null; // LAS File Data

  // Horizons Data

  public stratListStruct         stStrat     = null; // Stratigraphic Units

  // Rock Data - Measured Sections

  public rockDataListStruct      stCore      = null; // Core Data

  public rockColumnListStruct    stColumn    = null; // Rock Column

  public phiListStruct           stPHI       = null; // Porosity Type Data
  public fossilListStruct        stFossil    = null; // Rock Fossils
  public sedimentaryListStruct   stSedStruct = null; // Rock Sedimentary
  public rockColorListStruct     stColor     = null; // Rock Color
  public iqstratRemarkListStruct stRemarks   = null; // Remarks

  // Global Widgets

  private TitledBorder titledBorderPanel = null;

  // -- Well Information Plot Data

  private JLabel lblName   = new JLabel();
  private JLabel lblAPI    = new JLabel();
  private JLabel lblStatus = new JLabel();
  private JLabel lblLat    = new JLabel();
  private JLabel lblLong   = new JLabel();
  private JLabel lblTD     = new JLabel();
  private JLabel lblElev   = new JLabel();

  /** Constructor xsectionDataPanel()
   *  <p> This is the Constructor for this class.
   *  @param pNotifier = Observables passed
   *  @param stStruct  = Global IQSTRAT Data Structure
   */

  public xsectionDataPanel(Observable pNotifier, iqstratStruct stStruct, int iWell)
  {
    try
    {
	  this.pNotifier = pNotifier;
      this.stStruct  = stStruct;
      this.iWell     = iWell;

      jbInit();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create the Login Panel.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlTop          = new JPanel();

    JPanel pnlHeader       = new JPanel();
    JPanel pnlInfo         = new JPanel();
    JPanel pnlInfoPosition = new JPanel();
    JPanel pnlInfoXY       = new JPanel();
    JPanel pnlInfoZ        = new JPanel();

    JPanel pnlID            = new JPanel();
    JPanel pnlName          = new JPanel();
    JPanel pnlStatus        = new JPanel();

    JPanel pnlCenter        = new JPanel();

    titledBorderPanel = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Data:");
    titledBorderPanel.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderWell = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Header Information:");
    titledBorderWell.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // Create an Observable

    notifier = new xsectionDataPanelObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver( this );

    // Build Frame

    this.setLayout(new BorderLayout());
    this.setBorder(titledBorderPanel);

    // Top Panel

    pnlTop.setLayout(new BorderLayout());
    pnlTop.setPreferredSize(new Dimension(10, 100));

    // -- Header Information Plot Labels, & Cross Section Depth Range Panels

    pnlHeader.setLayout(new BorderLayout());

    // ---- Header Information Plot Labels Panel

    pnlInfo.setBorder(titledBorderWell);
    pnlInfo.setLayout(new BorderLayout());

    // .. Well Information Panel

    pnlID.setLayout(new GridLayout(2,1));

    // .... Well Name Panel

    pnlName.setLayout(new BorderLayout());

    lblName.setText("Name: ");

    // .... Well API-Number & Well Status Panel

    pnlStatus.setLayout(new GridLayout());

    lblAPI.setText("API: ");
    lblStatus.setText("Status:");

    // .. Well Position Panel

    pnlInfoPosition.setLayout(new BorderLayout());

    // .... XY Well Position Panel

    pnlInfoXY.setLayout(new GridLayout());

    lblLat.setText("Lat: ");
    lblLong.setText("Long: ");

    // .... Z Well Position Panel

    pnlInfoZ.setLayout(new GridLayout());

    lblTD.setText("Depth:");
    lblElev.setText("Elev (GL):");

    // -- Center Panel

    pnlCenter.setLayout(new BorderLayout());

    pTracks = new xsectionAvailablePanel( notifier );

    // Add Widgets to Panel

    this.add( pnlTop,              BorderLayout.NORTH );
    pnlTop.add(pnlHeader,          BorderLayout.CENTER);
    pnlHeader.add(pnlInfo,         BorderLayout.CENTER);

    pnlInfo.add(pnlID,             BorderLayout.NORTH);
    pnlID.add(pnlName,             null);
    pnlName.add(lblName,           BorderLayout.WEST);

    pnlID.add(pnlStatus,           null);
    pnlStatus.add(lblAPI,          BorderLayout.WEST);
    pnlStatus.add(lblStatus,       BorderLayout.CENTER);

    pnlInfo.add(pnlInfoPosition,   BorderLayout.CENTER);
    pnlInfoPosition.add(pnlInfoXY, BorderLayout.NORTH);
    pnlInfoXY.add(lblLat,          null);
    pnlInfoXY.add(lblLong,         null);

    pnlInfoPosition.add(pnlInfoZ,  BorderLayout.CENTER);
    pnlInfoZ.add(lblTD,            null);
    pnlInfoZ.add(lblElev,          null);

    this.add( pnlCenter,           BorderLayout.CENTER );
    pnlCenter.add(pTracks,         BorderLayout.CENTER);
  }

  /** Method close()
   *  <p> This method will release the memory allocated for global variables
   */

  public void close()
  {
    // Input Variables

    pNotifier   = null; // Observable Object this Frame
    stStruct    = null; // Global IQSTRAT Data Structure

    // Global Variables

    notifier    = null; // Observable Object this Frame

    stData      = null; // Cross Section Data Structure

	stLAS       = null;
	stStrat     = null;
	stCore      = null;
	stColumn    = null;
	stPHI       = null;
	stSedStruct = null;
	stFossil    = null;
	stColor     = null;
	stRemarks   = null;

    // Global Widgets

    titledBorderPanel = null;

    // -- Well Information Plot Data

    lblName     = null;
    lblAPI      = null;
    lblStatus   = null;
    lblLat      = null;
    lblLong     = null;
    lblTD       = null;
    lblElev     = null;
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getWell()
   *  <p> This method will identify the well order, i.e. reference or edit.
   *  @param iWell = the well
   */

  public int getWell() { return (iWell); }

  /** Method getSelected()
   * <p> This method will retrieve the selected tracks
   * @param the selected plot tracks
   */

  public int[] getSelected() { return (pTracks.getSelected()); }

  /** Method getLASTrackSize()
   *  <p> This method will return the LAS Plot Track Width Size
   *  @return the LAS Plot Track Width Size Indicator
   */

  public int getLASTrackSize() { return (pTracks.getLASTrackSize()); }

  /** Method getThinCurve()
   * <p> This method will retrieve the default Porosity Thin Track setting.
   */

  public int getThinCurve() { return (pTracks.getThinCurve()); }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData()
   *  <p> This method will set the Well Header Label Fields
   *  @param iWell = Well Order
   *  @param st    = Header Information Data Structure
   */

  public void setData( int iWell, iqstratHeadersStruct st )
  {
    if (st != null)
    {
	  switch (iWell)
	  {
		case 0:
	      titledBorderPanel.setTitle( "Reference Well:" );
	      break;

	    case 1:
	      titledBorderPanel.setTitle( "Edit Well:" );
	      break;
      }

      lblName.setText( "Name: "     + st.sName );
      lblAPI.setText(st.sAPI  + " ");
      lblStatus.setText( "Status: " + st.status );
      lblLat.setText( "Lat: "       + st.dLatitude );
      lblLong.setText( "Long: "     + st.dLongitude );
      lblTD.setText( "Depth: "      + st.depth );

      lblElev.setText("Elev: ");
      if (st.dGL > 0.0)
        lblElev.setText("Elev (GL): " + st.dGL);
      else if (st.dKB > 0.0)
        lblElev.setText("Elev (KB): " + st.dKB);
      else if (st.dDF > 0.0)
        lblElev.setText("Elev (DF): " + st.dDF);

//      if (pTopsEntry != null) pTopsEntry.setData(st);
    }
    else
    {
      lblName.setText( "Name: " );
      lblAPI.setText( " " );
      lblStatus.setText( "Status: " );
      lblLat.setText( "Lat: " );
      lblLong.setText( "Long: " );
      lblTD.setText( "Depth: " );

      lblElev.setText("Elev: ");
    }
  }

  /** Method setData()
   *  <p> This method will set the default panel
   *  @param stLAS       = LAS File Data Structure
   *  @param stStrat     = Stratigraphic Units Data List Structure
   *  @param stColumn    = Rock Lithology Column Data List Structure
   *  @param stPHI       = Rock Porosity Type Data List Structure
   *  @param stSedStruct = Sedimentary Structure Data List Structure
   *  @param stFossil    = General Fossil Data List Structure
   *  @param stColor     = Rock Color Data List Structure
   *  @param stRemarks   = Measured Section Description Data List Structure
   */

  public void setData( lasFileDataStruct       stLAS,
                       stratListStruct         stStrat,
                       rockDataListStruct      stCore,
                       // Measured Section Data
                       rockColumnListStruct    stColumn,
                       phiListStruct           stPHI,
                       sedimentaryListStruct   stSedStruct,
                       fossilListStruct        stFossil,
                       rockColorListStruct     stColor,
                       iqstratRemarkListStruct stRemarks )
  {
	this.stLAS       = stLAS;
	this.stStrat     = stStrat;
	this.stCore      = stCore;
	this.stColumn    = stColumn;
	this.stPHI       = stPHI;
	this.stSedStruct = stSedStruct;
	this.stFossil    = stFossil;
	this.stColor     = stColor;
	this.stRemarks   = stRemarks;

    pTracks.setByAction( stLAS,
                         stStrat,
                         stCore,
                         stColumn,
                         stPHI,
                         stFossil,
                         stSedStruct,
                         stColor,
                         stRemarks );

//    pTracks.setNotify();
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Gemini Start Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
  }

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

	if (sArg.equals("Available Track Panel Changed"))
	{
      if (pNotifier != null)
        pNotifier.notifyObservers(new String("Available Track Panel Changed"));
//        pNotifier.notifyObservers(new String("" + iWell + " - Available Track Panel Changed"));
	}

    if (sArg.equals("Available Track Panel - Group Changed"))
    {
      pTracks.setByGroup( stLAS,
                          stStrat,
                          stCore,
                          stColumn,
                          stPHI,
                          stFossil,
                          stSedStruct,
                          stColor,
                          stRemarks );

      if (pNotifier != null)
        pNotifier.notifyObservers(new String("Available Track Panel Changed"));
	}

    if (sArg.equals("Available Track Panel - Data Changed"))
    {
      pTracks.setByGroup( stLAS,
                          stStrat,
                          stCore,
                          stColumn,
                          stPHI,
                          stFossil,
                          stSedStruct,
                          stColor,
                          stRemarks );

      if (pNotifier != null)
        pNotifier.notifyObservers(new String("Available Track Panel Changed"));
	}

	if (sArg.equals("Get LAS Track Size"))
	{
      if (pNotifier != null)
        pNotifier.notifyObservers(new String("Get LAS Track Size"));
	}
  }
}

/** CLASS xsectionDataPanelObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class xsectionDataPanelObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 10/31/2012
 *  @author  John Victorine
 */
