/*
 * @xsectionPlotFrame.java Version 1.1 01/14/2009
 *
 * Copyright (c) 2009 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package xsection.gui;

import java.util.Observable;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import iqstrat.iqstratStruct;           // Global IQSTRAT Data Structure

import xsection.xsectionStruct;         // Global Cross Section Data Structure
import xsection.xsectionDataListStruct; // Cross Section List Structure
import xsection.gui.xsectionPlotPanel;  // Cross Section Plot Panel

/** Class xsectionPlotFrame
 *  <p> This Class is the lead in to the IQSTRAT Program
 *
 *  @version 1.1 01/14/2009
 *  @author  John R. Victorine
 */

public class xsectionPlotFrame extends JFrame implements ActionListener
{
  // Input Variables

  private Observable     notifier  = null; // Observables Object
  private iqstratStruct  stStruct = null; // Global IQSTRAT Data Structure

  // Global Frame Variables

  // -- Screen Width

  private int iControlWidth = 250;  // Crontol Width
  private int iScreenWidth  = 0;    // Screen Width
  private int iScreenHigh   = 0;    // Screen Height

  // -- Cross Section Plot Panel

  private xsectionPlotPanel pnlPlot = null; // Plot Panel

  // -- Mouse Cursors

  private Cursor normalCursor = null;
  private Cursor mouseCursor  = null;

  // Global Frame Widgets

  private JLabel lblOrder   = null; // Cursor Order Number
  private JLabel lblWell    = null; // Cursor Well Name
  private JLabel lblAPI     = null; // Cursor API-Number
  private JLabel lblDepth   = null; // Cursor Log Depth
  private JLabel lblElev    = null; // Cursor Elevation Depth
  private JLabel lblTops    = null; // Cursor Tops Information
/*
  private JLabel lblOrder[]   = null; // Cursor Order Number
  private JLabel lblWell[]    = null; // Cursor Well Name
  private JLabel lblAPI[]     = null; // Cursor API-Number
  private JLabel lblDepth[]   = null; // Cursor Log Depth
  private JLabel lblElev[]    = null; // Cursor Elevation Depth
  private JLabel lblTops[]    = null; // Cursor Tops Information
*/
  /** Constructor xsectionPlotFrame( Observable     notifier,
   *                                 iqstratStruct  stStruct,
   *                                 int            iWidth )
   *  <p> This is the Constructor for this class.
   *  @param notifier     = Observable
   *  @param stIQSTRAT    = Global IQSTRAT Data Structure
   *  @param stStruct     = Global Cross Section Data Structure
   *  @param iWidth       = Control Dialog Width
   */

  public xsectionPlotFrame( Observable     notifier,
                            iqstratStruct  stStruct,
                            int            iWidth )
  {
    try
    {
      this.notifier      = notifier;
      this.stStruct      = stStruct;
      this.iControlWidth = iWidth;

      normalCursor = new Cursor(Cursor.DEFAULT_CURSOR);
      mouseCursor  = new Cursor(Cursor.HAND_CURSOR);

      jbInit();

      addWindowListener( new xsectionPlotFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel pnlTop   = new JPanel();
	JPanel pnlRef   = new JPanel();
	JPanel pnlARef  = new JPanel();
	JPanel pnlDRef  = new JPanel();
//	JPanel pnlEdit  = new JPanel();
//	JPanel pnlDEdit = new JPanel();

    TitledBorder titledBorderRef = new TitledBorder(
        BorderFactory.createEmptyBorder(),
        "Well Cursor Location: Click on Selected Top in Profile Plot to Datum By");
//        "Reference Well: Click on Selected Top in Profile Plot to Datum By or Copy & Past");
    titledBorderRef.setTitleJustification(TitledBorder.LEFT);
    titledBorderRef.setTitleFont(new java.awt.Font("Dialog", 1, 14));
    titledBorderRef.setTitleColor(Color.blue);

//    TitledBorder titledBorderEdit = new TitledBorder(
//        BorderFactory.createEmptyBorder(),
//        "Edit Top Well: Click on Profile Plot to Add New Top or Select Top To Evaluate or Modify");
//    titledBorderEdit.setTitleJustification(TitledBorder.LEFT);
//    titledBorderEdit.setTitleFont(new java.awt.Font("Dialog", 1, 14));
//    titledBorderEdit.setTitleColor(Color.blue);

    TitledBorder titledBorderOrder = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "#");
    titledBorderOrder.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderName = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Well Name:");
    titledBorderName.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderAPI = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "API:");
    titledBorderAPI.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderLog = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Depth (log):");
    titledBorderLog.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderElev = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Depth (elev):");
    titledBorderElev.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTops = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Horizon Name:");
    titledBorderTops.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    this.setTitle("Cross Section Plot");
    this.getContentPane().setLayout(new BorderLayout());

    // Top Panel

    pnlTop.setLayout(new GridLayout());
/*
    lblDepth = new JLabel[2]; // Cursor Log Depth
    lblElev  = new JLabel[2]; // Cursor Elevation Depth
    lblTops  = new JLabel[2]; // Cursor Tops Information

    for (int i=0; i<2; i++)
    {
      lblDepth[i] = new JLabel(); // Cursor Log Depth
      lblDepth[i].setBorder(titledBorderLog);
      lblDepth[i].setText( "0.0" );
//      lblDepth[i].setHorizontalAlignment(SwingConstants.EAST);
      lblDepth[i].setFont(new java.awt.Font("Dialog", 1, 11));

      lblElev[i]  = new JLabel(); // Cursor Elevation Depth
      lblElev[i].setBorder(titledBorderElev);
      lblElev[i].setText( "0.0" );
//      lblElev[i].setHorizontalAlignment(SwingConstants.EAST);
      lblElev[i].setFont(new java.awt.Font("Dialog", 1, 11));

      lblTops[i]  = new JLabel(); // Cursor Tops Information
      lblTops[i].setBorder(titledBorderTops);
      lblTops[i].setText( "" );
//      lblTops[i].setHorizontalAlignment(SwingConstants.CENTER);
      lblTops[i].setFont(new java.awt.Font("Dialog", 1, 11));
	}
*/

    lblOrder = new JLabel(); // Cursor Well Order Number
    lblOrder.setBorder(titledBorderOrder);
    lblOrder.setText( "" );
    lblOrder.setFont(new java.awt.Font("Dialog", 1, 11));

    lblAPI = new JLabel(); // Cursor Well API-Number
    lblAPI.setBorder(titledBorderAPI);
    lblAPI.setText( "" );
    lblAPI.setFont(new java.awt.Font("Dialog", 1, 11));

    lblWell = new JLabel(); // Cursor Well Name
    lblWell.setBorder(titledBorderName);
    lblWell.setText( "" );
    lblWell.setFont(new java.awt.Font("Dialog", 1, 11));

    lblDepth = new JLabel(); // Cursor Log Depth
    lblDepth.setBorder(titledBorderLog);
    lblDepth.setText( "0.0" );
    lblDepth.setFont(new java.awt.Font("Dialog", 1, 11));

    lblElev  = new JLabel(); // Cursor Elevation Depth
    lblElev.setBorder(titledBorderElev);
    lblElev.setText( "0.0" );
    lblElev.setFont(new java.awt.Font("Dialog", 1, 11));

    lblTops  = new JLabel(); // Cursor Tops Information
    lblTops.setBorder(titledBorderTops);
    lblTops.setText( "" );
    lblTops.setFont(new java.awt.Font("Dialog", 1, 11));

    // -- Reference Well

    pnlRef.setLayout(new GridLayout());
    pnlRef.setBorder(titledBorderRef);
    pnlARef.setLayout(new BorderLayout());
    pnlDRef.setLayout(new GridLayout());

    // -- Edit Well

//    pnlEdit.setLayout(new GridLayout());
//    pnlEdit.setBorder(titledBorderEdit);
//    pnlDEdit.setLayout(new GridLayout());

    // Center Panel

    pnlPlot = new xsectionPlotPanel( stStruct,
                                     stStruct.depthStart,
                                     stStruct.depthEnd,
                                     stStruct.iScale,
                                     normalCursor,
                                     mouseCursor );
    pnlPlot.setObservable( notifier );
    pnlPlot.setCommon( stStruct.stCMN );
    pnlPlot.setTextfields( lblOrder, lblWell, lblAPI, lblTops, lblDepth, lblElev );

    // Attach Widgets

    this.getContentPane().add(pnlTop,  BorderLayout.NORTH);
    pnlTop.add(pnlRef,                 null);
    pnlRef.add(pnlARef,                null);
    pnlARef.add(lblOrder,              BorderLayout.WEST);
    pnlARef.add(lblAPI,                BorderLayout.EAST);
    pnlARef.add(lblWell,               BorderLayout.CENTER);
    pnlRef.add(pnlDRef,                null);
    pnlDRef.add(lblElev,               null);
    pnlDRef.add(lblDepth,              null);
    pnlRef.add(lblTops,                null);

//    pnlDRef.add(lblElev[0],            null);
//    pnlDRef.add(lblDepth[0],           null);
//    pnlRef.add(lblTops[0],             null);

//    pnlTop.add(pnlEdit,                null);
//    pnlEdit.add(pnlDEdit,              null);
//    pnlDEdit.add(lblElev[1],           null);
//    pnlDEdit.add(lblDepth[1],          null);
//    pnlEdit.add(lblTops[1],            null);

    this.getContentPane().add(pnlPlot, BorderLayout.CENTER);

    // Display the Frame

    Dimension d  = Toolkit.getDefaultToolkit().getScreenSize();
    iScreenWidth = d.width;
    iScreenHigh  = d.height-50;

    this.setSize(iScreenWidth-iControlWidth, iScreenHigh);
    this.setLocation(iControlWidth+1, 1);

    this.setVisible(true);
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Variables passed to this panel

    stStruct  = null; // Global IQSTRAT Data Structure
    notifier  = null; // Observables Object

    if (pnlPlot != null)
      pnlPlot.close();
    pnlPlot = null;

    // Mouse Cursors

    normalCursor = null;
    mouseCursor  = null;

    // Global Frame Widgets

    lblOrder     = null; // Cursor Order Number
    lblWell      = null; // Cursor Well Name
    lblAPI       = null; // Cursor API-Number
    lblDepth     = null; // Cursor Log Depth
    lblElev      = null; // Cursor Elevation Depth
    lblTops      = null; // Cursor Tops Information

    dispose();
  }

  /** METHOD xsectionPlotFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class xsectionPlotFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
//      close();
//      System.exit(0);
      if (notifier != null)
        notifier.notifyObservers(new String("Close Cross Section Plot"));
    }
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)  { }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (pnlPlot != null) image = pnlPlot.getImage();

    return (image);
  }

  /** Method getWell()
   *  <p> This method will identify which well was selected
   *  @return the Well Order Number
   */

  public int getWell() { return (pnlPlot.getWell()); }

  /** Method getZeller()
   *  <p> This method will pass the Zeller KID selected
   *  @return the Zeller KID
   */

  public String getZeller() { return (pnlPlot.getZeller()); }

  /** Method double getDepth()
   *  <p> This method will pass the depth selected
   *  @return depthSelect = the depth Selected
   */

  public double getDepth() { return (pnlPlot.getDepth()); }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setWorkingDepth()
   *  <p> This method will set the Working Depth()
   *  @param iEdit    = Set the Set Working Depth Indicator
   *  @param depthTop = Set the Working Top Depth
   */

  public void setWorkingDepth(int iEdit, double depthTop)
  {
	if (pnlPlot != null) pnlPlot.setWorkingDepth( iEdit, depthTop );
  }

  /** Method setCopyDepth()
   *  <p> This method will set the plot to show the copy &
   *      paste tops information
   *  @param iCopy    = Set the Copy Indicator
   *  @param sZeller  = Set the Zeller Tops KID
   *  @param sName    = Set the Name of the Top
   */

  public void setCopyDepth(int iCopy, String sZeller, String sName)
  {
	if (pnlPlot != null) pnlPlot.setCopyDepth( iCopy, sZeller, sName );
  }

  /** Method setEvalDepth()
   *  <p> This method will set the Working Depth()
   *  @param iEval     = Set the Set Evaluator Depth Indicator
   *  @param depthEval = Set the evaluator Top Depth
   */

  public void setEvalDepth(int iEval, double depthEval)
  {
	if (pnlPlot != null) pnlPlot.setEvalDepth( iEval, depthEval );
  }

  /** Method setScale( int iScale, double dMaximum, double dMinimum )
   * <p> This method will set the Plot scale in the
   *     Cross Plot Data List Structure
   * @param iScale   = The Depth Track Scale identifier
   * @param dMaximum = The Maximum Depth Value
   * @param dMinimum = The Minimum Depth Value
   */

  public void setScale( int iScale, double dMaximum, double dMinimum )
  {
    pnlPlot.setScale( iScale, dMaximum, dMinimum );
    pnlPlot.updateUI();
  }

  /** Method setData( xsectionDataListStruct stXList )
   * <p> This method will set the cross section data
   * @param stXList = Cross Section Data
   */

  public void setData( xsectionDataListStruct stXList )
  {
    pnlPlot.setData( stXList );
  }

  /** Method setLASTrackSize(int iLASSize[])
   * <p> This method will set the LAS File Track Width
   * @param iLASSize = Array of LAS File Track Width
   */

  public void setLASTrackSize(int iLASSize[])
  {
    pnlPlot.setLASTrackSize( iLASSize );
  }

  /** Method setDatum( int iDatum, String sDatum  )
   * <p> This method will set the datum by identifier button
   * @param iDatum = datum by identifier
   * @param sDatum = horizon name
   */

  public void setDatum( int iDatum, String sDatum )
  {
    pnlPlot.setDatum( iDatum, sDatum );
  }

  /** Method setSelected()
   * <p> This method will change the selected tracks that are plotted
   *  @param iWell     = The well order number
   *  @param iSelected = The Tracks that will be displayed in the selected well
   */

  public void setSelected( int iWell, int iSelected[] )
  {
	if (pnlPlot != null) { pnlPlot.setSelected( iWell, iSelected ); }
  }

  /** Method setLevel( int iLevel )
   *  <p> This method will load the indicator to turn on the confidence
   *      level color map iLevel=1 or turn off iLevel=0. Default is 0
   *  @param iLevel = the confidence level plot switch
   */

  public void setLevel(int iLevel )
  {
	if (pnlPlot != null){ pnlPlot.setLevel(iLevel); }
  }

  /** Method refresh()
   * <p> This method will set the ICS stratigraphic Units List Structure
   */

  public void refresh() { pnlPlot.refresh(); }

}

/*
 *  @version 1.1 01/14/2009
 *  @author  John Victorine
 */
