/*
 * @xsectionPlot.java Version 1.1 02/20/2008
 *
 * Copyright (c) 2008 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package xsection.plot;

import java.net.*;
import java.lang.*;
import java.io.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.Observable;
import javax.swing.*;
import javax.imageio.*;

import xsection.xsectionDataListStruct;        // Cross Section List Structure

import cmn.cmnStruct;                          // GLOBAL Common Data Structure

import iqstrat.iqstratStruct;                  // Global IQSTRAT Data Structure
import iqstrat.iqstratHeadersStruct;           // IQSTRAT Headers Data Structure

import las.plot.lasPlotTrack;                  // LAS Plot Track Canvas
import las.plot.lasPlotThinPhiTrack;           // Porosity Thin Track Canvas

import lith.plot.lithPlotColorlith;            // Colorlith Plot Track Canvas
import lith.lithology.plot.lithologyPlotTrack; // Lithology Plot Track Canvas
import lith.lithology.plot.lithologyPlotSymbolLegend; // LAS Rock Track Legend
import lith.rock.plot.rockColumnPlotLegend;    // Rock Column Legend Track Canvas
import lith.rock.plot.rockColumnPlotTrack;     // Rock Column Plot Track Canvas

import rock.rockDataListStruct;                // Rock Data List Data Structure
import rock.plot.rockPlotTrack;                // Core Data Plot Track Canvas

import rock.color.plot.rockColorPlotTrack;     // Rock Color Plot Track Canvas

import rock.phi.plot.phiPlotTrack;             // Porosity Type Plot Track Canvas
import rock.phi.phiListStruct;                 // Porosity Type List Structure

import rock.fossil.plot.fossilPlotTrack;       // General Fossil Plot Track Canvas
import rock.fossil.fossilListStruct;           // General Fossil List Structure

import rock.sedimentary.plot.sedimentaryPlotTrack;  // Sedimentary Structure Plot Canvas
import rock.sedimentary.sedimentaryListStruct;      // Sedimentary Struct List Structure

import horizon.strat.stratListStruct;          // Stratigraphic Units List Structure
import horizon.strat.plot.stratPlotTrack;

import plot.plotWellStatusStruct;              // Well Status Data Structure
import plot.plotMapping;                       // Plot Mapping Canvas

/** CLASS xsectionPlot
 *  <p> This Class will create the Cross Section Plot.
 *
 *  @version 1.1 02/20/2008
 *  @author  John R. Victorine
 */

public class xsectionPlot extends Canvas
    implements MouseListener, MouseMotionListener
{
  private iqstratStruct stStruct  = null; // Global IQSTRAT Data Structure
  private Observable    notifier  = null; // Observable Class
  private cmnStruct     stCMN     = null; // Global Common Data Structure

  private Cursor     normalCursor = null; // Normal Arrow Cursor
  private Cursor     mouseCursor  = null; // Mouse Cursor

  private double     depthStart   = 0.0;  // Starting Depth
  private double     depthEnd     = 0.0;  // Ending Depth
  private int        iScale       = -1;   // Feet / Inch Scale

  private int        iWell        = -1;   // Well Number
  private String     sZeller      = "0";  // Zeller Tops ID Number
  private String     sName        = "";   // Tops Name
  private double     depthSelect  = 0.0;  // Depth Selected

  private int        iEval        = 0;    // Evaluator Indicator
  private double     depthEval    = 0.0;  // Evaluator Depth

  private int        iCopy        = 0;    // Copy Tops Indicator
  private int        iEdit        = 0;    // Edit Tops
  private double     depthTop     = 0.0;  // Depth of working Depth

  private int        iLASSize[]   = { xsection.gui.xsectionAvailablePanel._SINGLE,
                                      xsection.gui.xsectionAvailablePanel._SINGLE,
                                      xsection.gui.xsectionAvailablePanel._SINGLE,
                                      xsection.gui.xsectionAvailablePanel._SINGLE };

  private int        iStart[][]   = null; // Array of Well Plot Tracks start point
  private int        iWide[]      = null; // Array of Well Plot Track Width
  private int        iBio[]       = null; // Fossil Track Width Array
  private int        iSed[]       = null; // Sedimentary Structure Track Widths

  private BufferedImage imgFOSSIL = null;
  private BufferedImage imgSED    = null;
  private BufferedImage imgPHI    = null;

//  private int                    iTrack = -1;   // Well/Outcrop Track Selected
  private xsectionDataListStruct stXList  = null; // Cross Section List Structure
  private plotMapping            plotMap  = null; // Well Status Plot

  // By Datum

  private int    iDatum     = iqstrat.iqstratTracksStruct._BY_LOG; // Datum
  private String sDatum     = "";        // Datum by Horizon Text

  // Separation of Wells

//  private int iSeparation   = 100; //50;   // Minimum Separation between each well
  private int iSeparation   = 50;
  private int iStartDepth   = 0;

  // Selected Plot Tracks

//  private int iSelected_0[] = null; // Reference Well
//  private int iSelected_1[] = null; // Edit Well

  // Plot Tracks

  private lasPlotTrack         pLAS         = null; // LAS Plot Track
  private lasPlotThinPhiTrack  pThinPhi     = null; // Thin Porosity Track

  private lithPlotColorlith    pColorlith   = null; // Colorlith & Image Track
  private lithologyPlotTrack   pLithology   = null; // Rock Lithology Track
  private lithologyPlotSymbolLegend pLegend = null; // Plot Legend

  private rockColumnPlotTrack  pColumn      = null; // Measured Section Rock Column Track
  private rockColumnPlotLegend pRockLegend  = null; // Rock Column Plot Legend

  private rockPlotTrack        plotLASRock  = null; // Rock Data LAS Plot
  private phiPlotTrack         pPhi         = null; // Porosity Type Track
  private fossilPlotTrack      pFossils     = null; // Fossils Plot Track
  private sedimentaryPlotTrack pSedimentary = null; // Sedimentary Track
  private rockColorPlotTrack   pRockColor   = null; // Rock Color Track

  private stratPlotTrack       pStrat       = null;  // Stratigraphics Units

  // Mouse Cursor Location Widgets

  private JLabel lblOrder  = null; // Cursor Order Number
  private JLabel lblWell   = null; // Cursor Well Name
  private JLabel lblAPI    = null; // Cursor API-Number
  private JLabel lblDepth  = null; // Cursor Log Depth
  private JLabel lblElev   = null; // Cursor Elevation Depth
  private JLabel lblTops   = null; // Cursor Tops Information
/*
  private JLabel lblOrder[]  = null; // Cursor Order Number
  private JLabel lblWell[]   = null; // Cursor Well Name
  private JLabel lblAPI[]    = null; // Cursor API-Number
  private JLabel lblDepth[]  = null; // Cursor Log Depth
  private JLabel lblElev[]   = null; // Cursor Elevation Depth
  private JLabel lblTops[]   = null; // Cursor Tops Information
*/
  // Plot Variables

  private String sTitle1          = "";
  private String sTitle2          = "";

  private double dIncrementDepth  = 0.0;
  private int    iStartDepthLabel = 10;

  private int    iWidth           = 650;  // Width of Plot
  private int    iHeight          = 100;  // Height of Plot with the Titles
  private int    iLogHeight       = 100;  // Height of Plot without the Titles
  private int    iIncrementY      = 100;  // Increment Height
  private int    iLegend          = 125;  // Height of LAS Lithology Legend
  private int    jLegend          = 200;  // Width of Rock Legend

  public static final int PLOT_TITLES = iqstrat.iqstratTracksStruct.PLOT_TITLES;
  public static final int LABELSTART  = iqstrat.iqstratTracksStruct.LABELSTART;
  public static final int DEPTH_TRACK = iqstrat.iqstratTracksStruct.DEPTH_TRACK;

  // Mouse Action Variables

//  private double dStart = 0.0;
//  private double dEnd   = 0.0;

  /** CONSTRUCTOR xsectionPlot( iqstratStruct  stStruct,
   *                            double depthStart,
   *                            double depthEnd,
   *                            int    iScale,
   *                            Cursor normalCursor,
   *                            Cursor mouseCursor  )
   *  <p> This is the constructor for this class.
   *  @param stStruct     = Global IQSTRAT Data Structure
   *  @param depthStart   = The starting depth of plot
   *  @param depthEnd     = The ending depth of plot
   *  @param iScale       = The scale
   *  @param normalCursor = Normal Cursor
   *  @param mouseCursor  = Hand Cursor
   */

  public xsectionPlot( iqstratStruct stStruct,
                       double depthStart,
                       double depthEnd,
                       int    iScale,
                       Cursor normalCursor,
                       Cursor mouseCursor )
  {
    this.stStruct     = stStruct;
    this.normalCursor = normalCursor;
    this.mouseCursor  = mouseCursor;

    setPlotWidth();
    setPlotHeight(iScale, depthEnd, depthStart);

    plotMap      = new plotMapping();

    pLAS         = new lasPlotTrack( depthStart, depthEnd, iScale );
    pThinPhi     = new lasPlotThinPhiTrack( depthStart, depthEnd, iScale );

    pColorlith   = new lithPlotColorlith( depthStart, depthEnd, iScale );
    pLithology   = new lithologyPlotTrack(
                         depthStart, depthEnd, iScale, stStruct.stSymbols );

    pLegend      = new lithologyPlotSymbolLegend( stStruct.stSymbols );
    pRockLegend  = new rockColumnPlotLegend( stStruct.stParseRock );

    pColumn      = new rockColumnPlotTrack(
                         depthStart, depthEnd, iScale, stStruct.stParseRock);

    plotLASRock  = new rockPlotTrack(depthStart, depthEnd, iScale);

    pPhi         = new phiPlotTrack(
                         depthStart, depthEnd, iScale, stStruct.stPorosity );
    pFossils     = new fossilPlotTrack(
                         depthStart, depthEnd, iScale, stStruct.stFossils );
    pSedimentary = new sedimentaryPlotTrack(
                         depthStart, depthEnd, iScale, stStruct.stSedStruct );
    pRockColor   = new rockColorPlotTrack( depthStart, depthEnd, iScale );

    pStrat       = new stratPlotTrack(
                         depthStart, depthEnd, iScale, stStruct.stICS );

    addMouseListener(this);
    addMouseMotionListener(this);

    this.setBackground(Color.white);
  }

  /** CONSTRUCTOR xsectionPlot( iqstratStruct  stStruct,
   *                            double depthStart,
   *                            double depthEnd,
   *                            int    iScale )
   *  <p> This is the constructor for this class.
   *  @param stStruct     = Global IQSTRAT Data Structure
   *  @param depthStart   = The starting depth of plot
   *  @param depthEnd     = The ending depth of plot
   *  @param iScale       = The scale
   */

  public xsectionPlot( iqstratStruct  stStruct,
                       double depthStart,
                       double depthEnd,
                       int    iScale  )
  {
	this.stStruct = stStruct;

    setPlotWidth();
    setPlotHeight(iScale, depthEnd, depthStart);

    plotMap      = new plotMapping();

    pLAS         = new lasPlotTrack( depthStart, depthEnd, iScale );
    pThinPhi     = new lasPlotThinPhiTrack( depthStart, depthEnd, iScale );

    pColorlith   = new lithPlotColorlith( depthStart, depthEnd, iScale );
    pLithology   = new lithologyPlotTrack(
                         depthStart, depthEnd, iScale, stStruct.stSymbols );

    pLegend      = new lithologyPlotSymbolLegend( stStruct.stSymbols );
    pRockLegend  = new rockColumnPlotLegend( stStruct.stParseRock );

    pColumn      = new rockColumnPlotTrack(
                         depthStart, depthEnd, iScale, stStruct.stParseRock);

    plotLASRock  = new rockPlotTrack(depthStart, depthEnd, iScale);

    pPhi         = new phiPlotTrack(
                         depthStart, depthEnd, iScale, stStruct.stPorosity );
    pFossils     = new fossilPlotTrack(
                         depthStart, depthEnd, iScale, stStruct.stFossils );
    pSedimentary = new sedimentaryPlotTrack(
                         depthStart, depthEnd, iScale, stStruct.stSedStruct );
    pRockColor   = new rockColorPlotTrack( depthStart, depthEnd, iScale );

    pStrat       = new stratPlotTrack(
                         depthStart, depthEnd, iScale, stStruct.stICS );

    addMouseListener(this);
    addMouseMotionListener(this);

    this.setBackground(Color.white);
  }

  /** Method close()
   * <p> This method will set object to null to force JAVA to reallocate memory
   */

  public void close()
  {
    notifier     = null;
    stCMN        = null;

    normalCursor = null;
    mouseCursor  = null;

//    pTopsEntry   = null;
    imgFOSSIL = null;
    imgSED    = null;
    imgPHI    = null;

    iStart       = null;
    iWide        = null;
    iBio         = null;
    iSed         = null;
    stXList      = null;

    sDatum       = null;
    sZeller      = null;  // Zeller Tops ID Number
    sName        = null;

    // Mouse Cursor Location Widgets

    lblOrder     = null; // Cursor Order Number
    lblWell      = null; // Cursor Well Name
    lblAPI       = null; // Cursor API-Number
    lblTops      = null;
    lblDepth     = null;
    lblElev      = null;

    // Plot Title

    sTitle1      = null;
    sTitle2      = null;

    if (plotMap != null)
      plotMap.close();
    plotMap = null;

    if (pLAS != null)
      pLAS.close();
    pLAS = null;

    if (pThinPhi != null)
      pThinPhi.close();
    pThinPhi = null;

    if (pColorlith != null)
      pColorlith.close();
    pColorlith = null;

    if (pLithology != null)
      pLithology.close();
    pLithology = null;

    if (pColumn != null)
      pColumn.close();
    pColumn = null;

    if (plotLASRock != null)
      plotLASRock.close();
    plotLASRock = null;

    if (pPhi != null)
      pPhi.close();
    pPhi = null;

    if (pFossils != null)
      pFossils.close();
    pFossils     = null;

    if (pSedimentary != null)
      pSedimentary.close();
    pSedimentary = null;

    if (pRockColor != null)
      pRockColor.close();
    pRockColor   = null;

    if (pStrat != null)
      pStrat.close();
    pStrat = null;

    if (pLegend != null)
      pLegend.close();
    pLegend = null;

    if (pRockLegend != null)
      pRockLegend.close();
    pRockLegend = null;
  }

  /* =============================================================== *
   * ------------------------- GET METHODS ------------------------- *
   * =============================================================== */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int getPlotWidth()  { return (iWidth+jLegend); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (iHeight+iLegend); }

  /** METHOD getURL()
   *  <p> This method will locate the URL for the specific file.
   * @param  filename = The Filename to be converted to URL
   * @return url      = The URL of the Filename.
   */

  protected URL getURL(String filename)
  {
     URL url         = null;
     URL urlCodeBase = null;

     if (urlCodeBase == null)
     {
       urlCodeBase = stCMN.urlDirCodeBase;
     }

     try
     {
       url = new URL(urlCodeBase, filename);
     }
     catch (java.net.MalformedURLException e)
     {
       String Message = new String("rockImagesPanel.getURL(): \n" +
                                  e.getMessage() + "\n" +
                                  "Couldn't create image: " + filename);
       System.out.println(e.getMessage());
       JOptionPane.showMessageDialog((Component) null,
                                     Message,
                                     "ERROR",
                                     JOptionPane.ERROR_MESSAGE);
       return null;
     }

     return url;
  }

  /** Method getTrackWidth( int iXData, int iTrack )
   * <p> This method will retrieve the track width.
   * @param  iXData      = The Well ID
   * @param  iTrack      = the track
   * @return iTrackWidth = The width of the track
   */

  private int getTrackWidth( int iXData, int iTrack )
  {
    int iTrackWidth = 0;

    for (int j=0; j<iqstrat.iqstratTracksStruct.TOTAL_TYPES; j++)
    {
      if (iTrack > -1)
      {
        if (iqstrat.iqstratTracksStruct.TRACK[iTrack][2].equals(
            iqstrat.iqstratTracksStruct.TYPE[j][0]))
        {
          if ( iqstrat.iqstratTracksStruct.TRACK[iTrack][2].equals("ICON"))
          {
            iTrackWidth = 10;

            if (iTrack == iqstrat.iqstratTracksStruct._ROCK_FOSSILS)
            {
              if (iBio != null) iTrackWidth = iBio[iXData];
            }

            if (iTrack == iqstrat.iqstratTracksStruct._SED_STRUCTURE)
            {
              if (iSed != null) iTrackWidth = iSed[iXData];
            }
          }
          else
            iTrackWidth = cmn.cmnString.stringToInt(
                iqstrat.iqstratTracksStruct.TYPE[j][2]);
        }
      }
    }

    return ( iTrackWidth );
  }

  /** Method isTrack( int iTrack, xsectionDataListStruct stXList )
   * <p> This method will check to see if the track is present and return true
   *     if it is and false if it is not
   * @param iTrack  = the track you are checking
   * @param stXList = Cross Section Data List Structure
   * @return bValue = true it is and false if it is not
   */

  public boolean isTrack( int iTrack, xsectionDataListStruct stXList )
  {
    boolean bValue  = false;
    int     iSelect = 0;
    int     iLength = 0;

    // IF the Cross Section Data List Structure is NOT NULL

    if (stXList != null)
    {
      for (int i=0; i<stXList.iCount; i++)
      {
        // IF the Cross Section Data Structure is NOT NULL

        if (stXList.stItem[i] != null)
        {
          // IF the Cross Section Data Control Structure is NOT NULL

          if (stXList.stItem[i].stControl != null)
          {
            // IF the Well Tracks Selection is NOT NULL

            if (stXList.stItem[i].stControl.iTracks != null)
            {
              // Initialize the starting point array of the Well Tracks

              iLength = stXList.stItem[i].stControl.iTracks.length;

              // FOR EACH Well Track

              for (int j=0; j<iLength; j++)
              {
                iSelect = stXList.stItem[i].stControl.iTracks[j];

                if (iTrack == iSelect)
                  bValue = true;
              }
            }
          }
        }
      }
    }

    return (bValue);
  }

  /** Method getWell()
   *  <p> This method will identify which well was selected
   *  @return iWell = The Well Order Number 0=Reference; 1=Edit
   */

  public int getWell() { return (iWell); }

  /** Method getZeller()
   *  <p> This method will pass the Zeller KID selected
   *  @return sZeller = Zeller KID
   */

  public String getZeller() { return (sZeller); }

  /** Method double getDepth()
   *  <p> This method will pass the depth selected
   *  @return depthSelect = the depth Selected
   */

  public double getDepth() { return (depthSelect); }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;
    int w = this.getWidth();
    int h = this.getHeight();

    image = new BufferedImage( w, h, BufferedImage.TYPE_INT_RGB );

    Graphics2D g2 = image.createGraphics();
    g2.setColor( Color.white );
    g2.fillRect( 0, 0, w, h );
    this.paint(g2);
    g2.dispose();

    return (image);
  }

  /* =============================================================== *
   * ------------------------- SET METHODS ------------------------- *
   * =============================================================== */

  /** Method setObservable(Observable notifier)
   * <p> This method will set the Observerable Object
   * @param notifier = the observable object
   */

  public void setObservable( Observable notifier ) { this.notifier = notifier; }

  /* ------------------------------------------------------------------ *
   * ---------------- Set Global Common Data Structure ---------------- *
   * ------------------------------------------------------------------ */

  /** Method setCommon()
   * <p> This method will set the Global Common Data Structure
   *  @param stCMN  = the Global Common Data Structure
   */

  public void setCommon( cmnStruct stCMN )
  {
    this.stCMN = stCMN;
    repaint();
  }

  /** Method setWorkingDepth()
   *  <p> This method will set the Working Depth()
   *  @param iEdit    = Set the Set Working Depth Indicator
   *  @param depthTop = Set the Working Top Depth
   */

  public void setWorkingDepth(int iEdit, double depthTop)
  {
	this.iEdit    = iEdit;
	this.depthTop = depthTop;
	repaint();
  }

  /** Method setCopyDepth()
   *  <p> This method will set the plot to show the copy &
   *      paste tops information
   *  @param iCopy    = Set the Copy Indicator
   *  @param sZeller  = Set the Zeller Tops KID
   *  @param sName    = Set the Name of the Top
   */

  public void setCopyDepth(int iCopy, String sZeller, String sName)
  {
	this.iCopy   = iCopy;
	this.sZeller = new String( sZeller );
	this.sName   = new String( sName );
  }

  /** Method setEvalDepth()
   *  <p> This method will set the Working Depth()
   *  @param iEval     = Set the Set Evaluator Depth Indicator
   *  @param depthEval = Set the evaluator Top Depth
   */

  public void setEvalDepth(int iEval, double depthEval)
  {
	this.iEval     = iEval;
	this.depthEval = depthEval;
	repaint();
  }

  /** Method setPlotWidth()
   * <p> This method will set the Plot Width
   */

  public void setPlotWidth( )
  {
    int iTrackWidth = 0;
    int iSelect     = -1;
    int iTops       = 0;
    int iLength     = 0;
    int iCount      = 0;
    int iMax        = 0;

    iWidth = DEPTH_TRACK;  // Initial Width is the Depth Track Width

    // IF the Cross Section Data List Structure is NOT NULL

    if (stXList != null)
    {
      // FOR EACH Well in the Cross Section Data List
      // Initialize the Well Plot Width

      iWide  = new int[stXList.iCount];

      for (int i=0; i<stXList.iCount; i++)
      {
        if (stXList.stItem[i] != null)
        {
          if (stXList.stItem[i].stControl != null)
          {
            if (stXList.stItem[i].stControl.iTracks != null)
            {
              iCount = 0;
              for (int j=0; j<stXList.stItem[i].stControl.iTracks.length; j++)
              {
                iCount++;
              }

              if (iCount > iMax) iMax = iCount;
            }
          }
        }
      }

      iStart  = new int[stXList.iCount][iMax];

      for (int i=0; i<stXList.iCount; i++)
      {
        iWide[i] = 0;

        // IF the Cross Section Data Structure is NOT NULL

        if (stXList.stItem[i] != null)
        {
          // IF the Cross Section Data Control Structure is NOT NULL

          if (stXList.stItem[i].stControl != null)
          {
            // IF the Well Tracks Selection is NOT NULL

            if (stXList.stItem[i].stControl.iTracks != null)
            {
              // Initialize the starting point array of the Well Tracks

              iLength = stXList.stItem[i].stControl.iTracks.length;

              // FOR EACH Well Track

              for (int j=0; j<iLength; j++)
              {
                iStart[i][j] = iWidth;  // Get the Starting point of Well Tracks

                // Compute the Width of the Track

                iSelect     = stXList.stItem[i].stControl.iTracks[j];
                iTrackWidth = getTrackWidth( i, iSelect );

                // Set the Primary LAS Tracks to Double Size if Selected

                if ( iLASSize[i] == xsection.gui.xsectionAvailablePanel._DOUBLE )
                {
                  if ((iSelect == iqstrat.iqstratTracksStruct._LAS_INDUCTION) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_MICRO_RESISTIVITY) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_RESISTIVITY) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_LATERLOG) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_AHT) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_SONIC) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_LITHO_DENSITY) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_SPECTRAL_GR))
                  {
                    iTrackWidth = 2 * iTrackWidth;
                  }
				}

                // Add the Width to the total width

                if (iSelect != iqstrat.iqstratTracksStruct._REMARKS)
                {
                  iWidth   = iWidth   + iTrackWidth;
                  iWide[i] = iWide[i] + iTrackWidth;
                }
              }

              // Include a Separation Width between the well

              if (i < stXList.iCount-1)
              {
                iWidth = iWidth + iSeparation;
			  }
            }
          }
        }
      }
    }
  }

  /** Method setPlotHeight(int iScale, double dMaximum, double dMinimum)
   *  <p> This method will set the Plot Height.
   * @param iScale   = The Depth Track Scale identifier
   * @param dMaximum = The Maximum Depth Value
   * @param dMinimum = The Minimum Depth Value
   */

  public void setPlotHeight( int iScale, double dMaximum, double dMinimum )
  {
    this.depthStart = dMinimum;
    this.depthEnd   = dMaximum;
    this.iScale     = iScale;

    iLogHeight      = (int) ((100 * Math.abs( dMaximum - dMinimum )) /
                              iqstrat.iqstratTracksStruct.SCALE[iScale]);
    iHeight         = PLOT_TITLES + iLogHeight;
    dIncrementDepth = iqstrat.iqstratTracksStruct.SCALE[iScale];
  }

  /** Method setTextfields()
   * <p> This method will set the label widgets so the location of the
   *     mouse cursor can be described on the Control Panel
   * @param lblOrder = Label that will hold the Well Order Number
   * @param lblWell  = Label that will hold the Well Name
   * @param lblAPI   = Label that will hold the Well API-Number
   * @param lblTops  = Label that will hold the name of well the cursor
   *                   is moving over.
   * @param lblDepth = Label that will hold the actual LAS or Core Depth
   * @param lblElev  = Label that will hold the Depth at elevation.
   */

  public void setTextfields( JLabel lblOrder, JLabel lblWell,  JLabel lblAPI,
                             JLabel lblTops,  JLabel lblDepth, JLabel lblElev )
  {
	this.lblOrder = lblOrder;
	this.lblWell  = lblWell;
	this.lblAPI   = lblAPI;
    this.lblTops  = lblTops;
    this.lblDepth = lblDepth;
    this.lblElev  = lblElev;
  }

  /** Method setTitles()
   * <p> This method will set the titles for this plot
   * @param sTitle1 = The first title
   * @param sTitle2 = The second title
   */

  public void setTitles( String sTitle1, String sTitle2 )
  {
    this.sTitle1 = new String( sTitle1 );
    this.sTitle2 = new String( sTitle2 );

    repaint();
  }

  /** Method setLASTrackSize(int iLASSize[])
   * <p> This method will set the LAS File Track Width
   * @param iLASSize = Array of LAS File Track Width
   */

  public void setLASTrackSize(int iLASSize[])
  {
    this.iLASSize = iLASSize;
//    plotLAS.setLASTrackSize( iLASSize );
    setPlotWidth();

    repaint();
  }

  /** Method setScale(int iScale, double dMaximum, double dMinimum)
   * <p> This method will set the Plot scale in the
   *     Cross Plot Data List Structure
   * @param iScale   = The Depth Track Scale identifier
   * @param dMaximum = The Maximum Depth Value
   * @param dMinimum = The Minimum Depth Value
   */

  public void setScale( int iScale, double dMaximum, double dMinimum )
  {
    setPlotHeight( iScale, dMaximum, dMinimum );

    if (pLAS       != null) pLAS.setPlotHeight( iScale, dMaximum, dMinimum );
    if (pThinPhi   != null) pThinPhi.setPlotHeight( iScale, dMaximum, dMinimum );

    if (pColorlith != null) pColorlith.setPlotHeight( iScale, dMaximum, dMinimum );
    if (pLithology != null) pLithology.setPlotHeight( iScale, dMaximum, dMinimum );

    if (pColumn    != null) pColumn.setPlotHeight(iScale, dMaximum, dMinimum);

    if (plotLASRock != null) plotLASRock.setPlotHeight(iScale, dMaximum, dMinimum);

    if (pPhi       != null) pPhi.setPlotHeight( iScale, dMaximum, dMinimum );
    if (pFossils   != null) pFossils.setPlotHeight( iScale, dMaximum, dMinimum );
    if (pSedimentary != null) pSedimentary.setPlotHeight( iScale, dMaximum, dMinimum );
    if (pRockColor != null) pRockColor.setPlotHeight( iScale, dMaximum, dMinimum );

    if (pStrat     != null) pStrat.setPlotHeight( iScale, dMaximum, dMinimum );

    repaint();
  }

  /** Method setData()
   * <p> This method will set the cross section data
   * @param stXList = Cross Section Data
   */

  public void setData( xsectionDataListStruct stXList )
  {
    this.stXList = stXList;

    if (stXList != null)
    {
      if (stXList.iCount > 0)
      {
        iBio = new int[stXList.iCount];
        iSed = new int[stXList.iCount];

        for (int i=0; i<stXList.iCount; i++)
        {
          iBio[i] = rock.fossil.fossilUtility.computeWidth(
                      stXList.stItem[i].stFossil );
          iSed[i] = rock.sedimentary.sedimentaryUtility.computeWidth(
                      stXList.stItem[i].stSedimentary );
        }
      }
    }

    setPlotWidth();

    pLAS.setData( stXList );
    pThinPhi.setData( stXList );
    pColorlith.setData( stXList );

    plotLASRock.setData( stXList );

    if ((isTrack( iqstrat.iqstratTracksStruct._ROCK_COLUMN, stXList )) ||
        (isTrack( iqstrat.iqstratTracksStruct._ROCK_COLUMN_PD, stXList )))
    {
      pLithology.setData(stXList);
    }

    if (isTrack( iqstrat.iqstratTracksStruct._ROCK_THIN_PHI, stXList ))
    {
      pPhi.setData( stXList );
      setPHILegend( stXList );
    }

    if (isTrack( iqstrat.iqstratTracksStruct._ROCK_MEASURED_SECTION, stXList ))
    {
      pColumn.setData( stXList );
      pRockLegend.setData( stXList );
    }

    if (isTrack( iqstrat.iqstratTracksStruct._ROCK_FOSSILS, stXList ))
    {
      pFossils.setData( stXList );
      setFossilLegend( stXList );
    }

    if (isTrack( iqstrat.iqstratTracksStruct._SED_STRUCTURE, stXList ))
    {
      pSedimentary.setData( stXList );
      setSedStructLegend( stXList );
    }

    if (isTrack( iqstrat.iqstratTracksStruct._ROCK_COLOR, stXList ))
      pRockColor.setData( stXList );

    pStrat.setData( stXList );
  }

  /** Method setPHILegend()
   *  <p> This method will set the porosity type legend depending on the porosity
   *      types selected.
   */

  public void setPHILegend( xsectionDataListStruct stXList )
  {
	phiListStruct st = null;
    int iPHITypes[]  = null;
    int iContinue    = 0;

    if ((stXList != null) && (stStruct.stPorosity != null))
    {
	  if ((stXList.iCount > 0) && (stStruct.stPorosity.iCount > 0))
	  {
        iPHITypes = new int[stStruct.stPorosity.iCount];
        for (int i=0; i<stStruct.stPorosity.iCount; i++)
          iPHITypes[i] = -1;

		for (int k=0; k<stXList.iCount; k++)
		{
		  st = stXList.stItem[k].stPHI;

          if (st != null)
          {
            if (st.iCount > 0)
            {
			  iContinue = 1;
              iPHITypes = rock.phi.phiUtility.setLegend(
                            iPHITypes, st, stStruct.stPorosity );

            }
          }
		}
      }
    }

    if (iContinue == 1)
    {
      imgPHI = rock.phi.plot.phiPlotLegendBuffered.makePlotImage(
                iPHITypes, stStruct.stPorosity );
	}
  }

  /** Method setFossilLegend()
   *  <p> This method will set the fossil legend depending on the fossils
   *      selected.
   */

  public void setFossilLegend( xsectionDataListStruct stXList )
  {
	fossilListStruct st = null;
    int iFossils[] = null;  // Fossil selected indicator
    int iContinue  = 0;

    if ((stXList != null) && (stStruct.stFossils != null))
    {
	  if ((stXList.iCount > 0) && (stStruct.stFossils.iCount > 0))
	  {
        iFossils = new int[stStruct.stFossils.iCount];
        for (int i=0; i<stStruct.stFossils.iCount; i++)
          iFossils[i] = -1;

		for (int k=0; k<stXList.iCount; k++)
		{
		  st = stXList.stItem[k].stFossil;

          if (st != null)
          {
            if (st.iCount > 0)
            {
			  iContinue = 1;
              iFossils  = rock.fossil.fossilUtility.setLegend(
                            iFossils, st, stStruct.stFossils);
			}
          }
        }
      }
    }

    if (iContinue == 1)
    {
      imgFOSSIL = rock.fossil.plot.fossilPlotLegendBuffered.makePlotImage(
                   iFossils, stStruct.stFossils );
	}
  }

  /** Method setSedStructLegend()
   *  <p> This method will set the fossil legend depending on the Sedimentary
   *      Structures selected.
   */

  public void setSedStructLegend( xsectionDataListStruct stXList )
  {
    sedimentaryListStruct st = null;
    int iSedStructs[] = null;  // Fossil selected indicator
    int iContinue     = 0;

    if ((stXList != null) && (stStruct.stSedStruct != null))
    {
	  if ((stXList.iCount > 0) && (stStruct.stSedStruct.iCount > 0))
	  {
        iSedStructs = new int[stStruct.stSedStruct.iCount];
        for (int i=0; i<stStruct.stSedStruct.iCount; i++)
          iSedStructs[i] = -1;

		for (int k=0; k<stXList.iCount; k++)
		{
		  st = stXList.stItem[k].stSedimentary;

          if (st != null)
          {
            if (st.iCount > 0)
            {
			  iContinue   = 1;
              iSedStructs = rock.sedimentary.sedimentaryUtility.setLegend(
                              iSedStructs, st, stStruct.stSedStruct);
            }
          }
	    }
      }
    }

    if (iContinue == 1)
    {
      imgSED = rock.sedimentary.plot.sedimentaryPlotLegendBuffered.makePlotImage(
                   iSedStructs, stStruct.stSedStruct );
	}
  }

  /** Method setDatum( int iDatum, String sDatum  )
   * <p> This method will set the datum by identifier button
   * @param iDatum = datum by identifier
   * @param sDatum = horizon name
   */

  public void setDatum( int iDatum, String sDatum )
  {
    this.iDatum = iDatum;
    this.sDatum = new String( sDatum );

    if (pLAS         != null) pLAS.setDatum( iDatum, sDatum );
    if (pThinPhi     != null) pThinPhi.setDatum( iDatum, sDatum );

    if (pColorlith   != null) pColorlith.setDatum( iDatum, sDatum );
    if (pLithology   != null) pLithology.setDatum( iDatum, sDatum );

    if (pColumn      != null) pColumn.setDatum( iDatum, sDatum );

    if (plotLASRock  != null) plotLASRock.setDatum( iDatum, sDatum );

    if (pPhi         != null) pPhi.setDatum( iDatum, sDatum );
    if (pFossils     != null) pFossils.setDatum( iDatum, sDatum );
    if (pSedimentary != null) pSedimentary.setDatum( iDatum, sDatum );
    if (pRockColor   != null) pRockColor.setDatum( iDatum, sDatum );

    if (pStrat       != null) pStrat.setDatum( iDatum, sDatum );

//    repaint();
  }

  /** Method setSelected()
   * <p> This method will change the selected tracks that are plotted
   *  @param iWell     = The well order number
   *  @param iSelected = The Tracks that will be displayed in the selected well
   */

  public void setSelected( int iWell, int iSelected[] )
  {
    if (stStruct.iWells > iWell)
    {
	  stXList.stItem[iWell].stControl.iTracks = iSelected;
    }
    setPlotWidth();
    repaint();
  }

  /** Method setLevel( int iLevel )
   *  <p> This method will load the indicator to turn on the confidence
   *      level color map iLevel=1 or turn off iLevel=0. Default is 0
   *  @param iLevel = the confidence level plot switch
   */

  public void setLevel(int iLevel )
  {
	if (pStrat != null)
	{
	  pStrat.setLevel( iLevel );
      repaint();
    }
  }


  /** Method refresh()
   * <p> This method will set the ICS stratigraphic Units List Structure
   */

  public void refresh() { repaint(); }

  /* =============================================================== *
   * ----------------------- COMPUTE METHODS ----------------------- *
   * =============================================================== */

  /** Method computeMouseDepth(int iYm)
   * <p> This method will compute the Mouse Depth
   * @param iYm = the cursor y-coordinate
   * @return depth = depth computed from mouse cursor location
   */

  public double computeMouseDepth(int iYm)
  {
    double depth = 0.0;

    if (iYm > PLOT_TITLES)
    {
      depth = depthStart + (depthEnd-depthStart) * (iYm-PLOT_TITLES)/iLogHeight;
    }

    return (depth);
  }

  /* =============================================================== *
   * -------------------- MouseListener Methods -------------------- *
   * =============================================================== */

  /** Method mouseMoved()
   *  <p> This method will retrieve the location of the mouse pointer and
   *      convert the postion to depth position.
   * @param event - Mouse Moved Event
   */

  public void mouseMoved(MouseEvent event)
  {
    int    iXm       = event.getX();
    int    iYm       = event.getY();
    String sWellName = "";
    String sAPI      = "";
    double depthElev = computeMouseDepth( iYm );
    double depth     = 0.0;
    double dElev     = 0.0;
    int    iW        = 0;
    int    iB        = 0;
    int    iSelect   = -1;
    stratListStruct stS   = null;
    double          dS    = 0.0;
    int             iRef  = 0;
    int             iD    = 0;
    int             iStrt = 0;
    int             iTemp = 0;
    int             iEnd  = 0;

    if (stXList != null)
    {
      for (int i=0; i<stXList.iCount; i++)
      {
        if (stXList.stItem[i] != null)
        {
          if (stXList.stItem[i].stControl != null)
          {
            if (stXList.stItem[i].stControl.iTracks != null)
            {
              for (int j=0; j<stXList.stItem[i].stControl.iTracks.length; j++)
              {
                iB = iStart[i][j];
                iW = getTrackWidth( i, stXList.stItem[i].stControl.iTracks[j] );
                iSelect = stXList.stItem[i].stControl.iTracks[j];

                switch (iSelect)
                {
                  case iqstrat.iqstratTracksStruct._LITHOLOGY_GR:
                  case iqstrat.iqstratTracksStruct._LAS_REFERENCE:
                  case iqstrat.iqstratTracksStruct._LAS_INDUCTION:
                  case iqstrat.iqstratTracksStruct._LAS_MICRO_RESISTIVITY:
                  case iqstrat.iqstratTracksStruct._LAS_RESISTIVITY:
                  case iqstrat.iqstratTracksStruct._LAS_LATERLOG:
                  case iqstrat.iqstratTracksStruct._LAS_AHT:
                  case iqstrat.iqstratTracksStruct._LAS_LITHO_DENSITY_3:
                  case iqstrat.iqstratTracksStruct._LAS_SONIC:
                  case iqstrat.iqstratTracksStruct._LAS_LITHO_DENSITY:
                  case iqstrat.iqstratTracksStruct._LAS_SPECTRAL_GR:
                  case iqstrat.iqstratTracksStruct._LAS_RHOMAA_UMAA:
                  case iqstrat.iqstratTracksStruct._LAS_RHOMAA_NPHI:
                  case iqstrat.iqstratTracksStruct._LAS_SPECTRAL_RATIO:
                  case iqstrat.iqstratTracksStruct._CLRLITH_RU:
                  case iqstrat.iqstratTracksStruct._CLRLITH_RN:
                  case iqstrat.iqstratTracksStruct._ROCK_COLUMN:
                  case iqstrat.iqstratTracksStruct._ROCK_COLUMN_PD:
                  case iqstrat.iqstratTracksStruct._CLRLITH_HALL:
                  case iqstrat.iqstratTracksStruct._CLRLITH_SPGR:

                  case iqstrat.iqstratTracksStruct._THIN_PHI:
                  case iqstrat.iqstratTracksStruct._LAS_SET_GR_COUNTS:
                  case iqstrat.iqstratTracksStruct._LAS_SET_NEUT_COUNTS:

                  case iqstrat.iqstratTracksStruct._ROCK_LAS_CORE_PHI:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_OUTCROP:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_CORE_K:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_PLUG_K:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_SATURATION:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_ARCHIE:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_GR:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_SPGR:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_LINEAR:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_LOG:

                  case iqstrat.iqstratTracksStruct._ROCK_MEASURED_SECTION:
                  case iqstrat.iqstratTracksStruct._SED_STRUCTURE:
                  case iqstrat.iqstratTracksStruct._ROCK_FOSSILS:
                  case iqstrat.iqstratTracksStruct._ROCK_COLOR:
                  case iqstrat.iqstratTracksStruct._REMARKS:
                    // Horizon Tracks
                  case iqstrat.iqstratTracksStruct._STRAT_UNITS_A:
                    if ((iXm > iB) && (iXm < (iB+iW)))
                    {
					  if (iCopy == 0)
					  {
					    iWell       = i;
                        sName       = "";
                        sZeller     = "0";
                        depthSelect = 0;
					  }

                      if (normalCursor != null) setCursor(normalCursor);
				    }
                    break;
                }
              }
            }
          }
        }
      }
    }

    if (stXList != null)
    {
      for (int i=0; i<stXList.iCount; i++)
      {
        if (stXList.stItem[i] != null)
        {
          dElev    = xsection.xsectionDataUtility.getElevation(
                       iDatum, sDatum, stXList.stItem[i] );

          if (stXList.stItem[i].stHeader != null)
          {
            sWellName = stXList.stItem[i].stHeader.sName;
            sAPI      = stXList.stItem[i].stHeader.sAPI;

   		    if (iDatum == iqstrat.iqstratTracksStruct._BY_LOG)
              depth = ( (int) ( (depthElev) * 100.0)) / 100.0;
   		    else
   		    {
              depth = ( (int) ( (dElev - depthElev) * 100.0)) / 100.0;

              if (stXList.stItem[i].stHeader.iType ==
                  iqstrat.iqstratHeadersStruct.OUTCROP)
                depth = ( (int) ( (dElev + depthElev) * 100.0)) / 100.0;
		    }

            dS        = depth;
            depthElev = (double) (Math.round(depthElev * 100.0) / 100.0);

            if (iWell > -1) { stS = stXList.stItem[iWell].stStrat; }

            if ((stS != null) && (i == iWell))
            {
			  iD = (int) dS;
			  for (int k=0; k<stS.iCount; k++)
			  {
				iRef = (int) stS.stItem[k].depthStart;

//				if ((dS >= stS.stItem[k].depthStart-0.25) && (dS <= stS.stItem[k].depthStart+0.25))
				if ((dS >= stS.stItem[k].depthStart-1.0) && (dS <= stS.stItem[k].depthStart+1.0))
//                if (iRef == iD)
                {
				  iStrt = 0;
				  iEnd  = 0;

				  if (stXList.stItem[i].stControl.iTracks.length > 0)
				  {
				    iStrt = iStart[i][0];
				    iTemp = iStart[i][stXList.stItem[i].stControl.iTracks.length-1];
                    iEnd  = iStrt + getTrackWidth( i,
                      stXList.stItem[i].stControl.iTracks[stXList.stItem[i].stControl.iTracks.length-1] );
				  }

				  depth   = stS.stItem[k].depthStart;

				  if (iCopy == 0)
				  {
				    sName   = stS.stItem[k].sName;
//				    sZeller = stS.stItem[k].sKEY;
				    sZeller = stS.stItem[k].sName;
				  }

//				  System.out.println(stS.stItem[k].sKEY+" "+stS.stItem[k].sName + " <> " + iStrt+" "+iEnd);
			    }
			  }
			}
          }

          if ((iXm > iStart[i][0]) && (iXm < (iStart[i][0]+iWide[i])) && (iWell > -1))
          {
            if (lblOrder != null) { lblOrder.setText("("+iWell+")"); }
            if (lblWell  != null) { lblWell.setText(sWellName); }
            if (lblAPI   != null) { lblAPI.setText(sAPI); }
            if (lblTops  != null) { lblTops.setText(sName); }
            if (lblDepth != null) { lblDepth.setText(""+depth); }
            if (lblElev  != null) { lblElev.setText(""+depthElev); }
          }
        }
      }
    }
  }

  public void mouseEntered(MouseEvent event) { }
  public void mouseExited(MouseEvent event) { }

  /** Method mousePressed(MouseEvent)
   *  <p> gets the current coordinates and creates a new rectangle,
   *      updates the rectangle to be drawn and
   *      calls repaint on the parent panel
   *  @param event - MouseEvent info
   */

  public void mousePressed(MouseEvent event) { }

  /** Method mouseDragged(MouseEvent)
   *  <p> updates the size of the rectangle as the user drags the mouse
   *  @param event - MouseEvent info
   */

  public void mouseDragged(MouseEvent event) { }

  /** Method mouseReleased(MouseEvent)
   *  <p> updates the size of the rectangle as the user releases the mouse
   *  @param event - MouseEvent info
   */

  public void mouseReleased(MouseEvent event) { }

  /** Method mouseClicked()
   *  <p> Mouse Clicked Method retrieve the location of the mouse pointer
   *      location and decides if the it was clicked.
   * @param event - Mouse Clicked Event
   */

  public void mouseClicked(MouseEvent event)
  {
    int    iXm       = event.getX();
    int    iYm       = event.getY();
    double depth     = 0.0;
    double dElev     = 0.0;
    double dStart    = 0.0;
    double dEnd      = 0.0;

    int iW      = 0;
    int iB      = 0;
    int iSelect = -1;
    int iCont   = 0;

    iWell = -1;

    if (stXList != null)
    {
      for (int i=0; i<stXList.iCount; i++)
      {
        if (stXList.stItem[i] != null)
        {
          if (stXList.stItem[i].stControl != null)
          {
            if (stXList.stItem[i].stControl.iTracks != null)
            {
              for (int j=0; j<stXList.stItem[i].stControl.iTracks.length; j++)
              {
                iB = iStart[i][j];
                iW = getTrackWidth( i, stXList.stItem[i].stControl.iTracks[j] );
                iSelect = stXList.stItem[i].stControl.iTracks[j];

                switch (iSelect)
                {
                  case iqstrat.iqstratTracksStruct._LITHOLOGY_GR:
                  case iqstrat.iqstratTracksStruct._LAS_REFERENCE:
                  case iqstrat.iqstratTracksStruct._LAS_INDUCTION:
                  case iqstrat.iqstratTracksStruct._LAS_MICRO_RESISTIVITY:
                  case iqstrat.iqstratTracksStruct._LAS_RESISTIVITY:
                  case iqstrat.iqstratTracksStruct._LAS_LATERLOG:
                  case iqstrat.iqstratTracksStruct._LAS_AHT:
                  case iqstrat.iqstratTracksStruct._LAS_LITHO_DENSITY_3:
                  case iqstrat.iqstratTracksStruct._LAS_SONIC:
                  case iqstrat.iqstratTracksStruct._LAS_LITHO_DENSITY:
                  case iqstrat.iqstratTracksStruct._LAS_SPECTRAL_GR:
                  case iqstrat.iqstratTracksStruct._LAS_RHOMAA_UMAA:
                  case iqstrat.iqstratTracksStruct._LAS_RHOMAA_NPHI:
                  case iqstrat.iqstratTracksStruct._LAS_SPECTRAL_RATIO:
                  case iqstrat.iqstratTracksStruct._CLRLITH_RU:
                  case iqstrat.iqstratTracksStruct._CLRLITH_RN:
                  case iqstrat.iqstratTracksStruct._ROCK_COLUMN:
                  case iqstrat.iqstratTracksStruct._ROCK_COLUMN_PD:
                  case iqstrat.iqstratTracksStruct._CLRLITH_HALL:
                  case iqstrat.iqstratTracksStruct._CLRLITH_SPGR:

                  case iqstrat.iqstratTracksStruct._THIN_PHI:
                  case iqstrat.iqstratTracksStruct._LAS_SET_GR_COUNTS:
                  case iqstrat.iqstratTracksStruct._LAS_SET_NEUT_COUNTS:

                  case iqstrat.iqstratTracksStruct._ROCK_LAS_CORE_PHI:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_OUTCROP:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_CORE_K:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_PLUG_K:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_SATURATION:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_ARCHIE:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_GR:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_SPGR:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_LINEAR:
                  case iqstrat.iqstratTracksStruct._ROCK_LAS_LOG:

                  case iqstrat.iqstratTracksStruct._ROCK_MEASURED_SECTION:
                  case iqstrat.iqstratTracksStruct._SED_STRUCTURE:
                  case iqstrat.iqstratTracksStruct._ROCK_FOSSILS:
                  case iqstrat.iqstratTracksStruct._ROCK_COLOR:
                  case iqstrat.iqstratTracksStruct._REMARKS:
                    // Horizon Tracks
                  case iqstrat.iqstratTracksStruct._STRAT_UNITS_A:

                    if ((iXm > iB) && (iXm < (iB+iW)))
                    {
                      dStart = computeMouseDepth( iYm );
//                      dEnd   = dStart;

                      dStart = (dStart * 100.0) / 100.0;
//                      dEnd   = (dEnd * 100.0) / 100.0;

                      dElev = xsection.xsectionDataUtility.getElevation(
                                    iDatum, sDatum, stXList.stItem[i] );

                      if (iDatum == iqstrat.iqstratTracksStruct._BY_LOG)
                        depth = ( (int) ( (dStart) * 100.0)) / 100.0;
		              else
   		              {
                        depth = ( (int) ( (dElev - dStart) * 100.0)) / 100.0;

                        if (stXList.stItem[i].stHeader.iType == iqstrat.iqstratHeadersStruct.OUTCROP)
                          depth = ( (int) ( (dElev + dStart) * 100.0)) / 100.0;
                      }

                      iWell       = i;
                      depthSelect = depth; //dStart;

                      if (iWell > 0)
                        iCont = 1;
                      else if (sZeller.length() > 1)
                        iCont = 1;

                      if (iCont == 1)
                      {
						if ((depthSelect > 0.0) && (iWell > 0))
						{
						  iEdit    = 1;
                          depthTop = depthSelect;
                          repaint();
					    }

                        iCopy = 0;

                        if (notifier != null)
                          notifier.notifyObservers(new String("Datum By"));
//                          notifier.notifyObservers(new String("Edit Tops"));
					  }
                    }
                    break;
                }
              }
            }
          }
        }
      }
    }
  }

  /* =============================================================== *
   * ------------------------ DRAW METHODS ------------------------- *
   * =============================================================== */

  /** Method drawTitles(Graphics g)
   * <p> This method will draw the Titles for this plot
   * @param g - Graphics Parameter.
   *

  public void drawTitles(Graphics g)
  {
    int    iChar  = 5;
    int    iX     = 0;
    int    iY     = 0;

    Font        fsb    = new Font("Serif", Font.BOLD, 14);
    FontMetrics fsbm   = g.getFontMetrics(fsb);

    g.setFont( fsb );
    g.setColor(Color.black);

    iX = iWidth / 2 - iChar * sTitle1.length() / 2;
    iY = 10;
    g.drawString(sTitle1, iX, iY);

    iX = iWidth / 2 - iChar * sTitle2.length() / 2;
    iY = 28;
    g.drawString(sTitle2, iX, iY);
  }
*/

  /** Method drawTrackLabels()
   *  <p> This method will draw the labels on the LAS Plot.
   *   @param g - Graphics Parameter.
   */

  public void drawTrackLabels(Graphics g)
  {
    int length      = 0;
    int iStartTrack = iStartDepth; //0;
    int iWidthTrack = DEPTH_TRACK;
    String sTemp    = "";

    g.setColor(Color.black);

    Font fsb = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    length = sTemp.length();
    g.drawString(sTemp, iWidth/2-length*4, 14);
    g.drawLine(iStartDepth+iStartTrack, LABELSTART, iWidthTrack, LABELSTART);

    g.setColor(Color.black);
    g.drawString("Depth", iStartDepth+iStartDepthLabel, LABELSTART + 10);

    if (iDatum == iqstrat.iqstratTracksStruct._BY_LOG)
      g.drawString("  Log", iStartDepth+iStartDepthLabel, LABELSTART + 30);
    else if (iDatum == iqstrat.iqstratTracksStruct._BY_ELEV)
      g.drawString(" Elev", iStartDepth+iStartDepthLabel, LABELSTART + 30);
    else if (iDatum == iqstrat.iqstratTracksStruct._BY_DATUM)
      g.drawString("Horiz", iStartDepth+iStartDepthLabel, LABELSTART + 30);

//    g.setColor(Color.lightGray);
//    g.drawString("Log", iStartDepth+iStartDepthLabel+50, LABELSTART + 30);

  }

  /** Method drawWellLabel()
   * <p> This method will draw the labels over the wells
   * @param g - Graphics Parameter.
   */

  public void drawWellLabel(Graphics g)
  {
    int    iFound = -1;
    int    iX     = 0;
    int    iY     = 0;
    int    iText  = 15;
    int    iChar  = 5;
    int    length = 0;
    int    iWell  = 0;
    int    iCtr   = iWell - DEPTH_TRACK;
    int    iPos   = DEPTH_TRACK;
    String sTemp  = "";
    int    iElev  = 0;
    double dElev  = 0.0;

    g.setColor(Color.black);

    Font fsb = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    if (stXList != null)
    {
      for (int i=0; i<stXList.iCount; i++)
      {
        iCtr  = iWide[i] / 2;
        iWell = iStart[i][0];
//        iWell = iWide[i];

        for (int j=0; j<plot.plotWellStatusStruct.iCount; j++)
        {
          if (stXList.stItem[i].stHeader.status.equals(
                plot.plotWellStatusStruct.status[j][0]))
            iFound = plot.plotWellStatusStruct.iStatus[j]; //j;
        }

        if (iFound > -1)
        {
          iX = iWell + iCtr;
//          iX = DEPTH_TRACK + i * iWell + iCtr / 2 - 5;
          iY = LABELSTART - 12;
          plotMap.drawWellSymbol(g, iFound, iX, iY);
          g.setColor(Color.black);
        }

//        if (iWell <= 75)
        if (iWide[i] <= 75)
        {
          sTemp  = "( " + i + " )";

          length = sTemp.length();
          iX = iChar * length;
          if (iX > iWell)
            sTemp = new String(sTemp.substring(0, (iX/iChar)));

          iX = iWell + iCtr - iChar * sTemp.length() / 2;
//          iX = DEPTH_TRACK + i * iWell + iCtr / 2 - iChar * sTemp.length() / 2;
          iY = LABELSTART - 12 - iText;
          g.drawString(sTemp, iX, iY);
        }
//        else if (iWell > 75)
        else if (iWide[i] > 75)
        {
          sTemp  = stXList.stItem[i].stHeader.sName;

          length = sTemp.length();
          iX = iChar * length;
          if (iX > iWell)
            sTemp = new String(sTemp.substring(0, (iX/iChar)));

          iX = iWell + iCtr - iChar * sTemp.length() / 2;
//          iX = DEPTH_TRACK + i * iWell + iCtr / 2 - iChar * sTemp.length() / 2;
          iY = LABELSTART - 12 - 5 * iText;
          g.drawString(sTemp, iX, iY);

          sTemp = stXList.stItem[i].stHeader.sAPI;

          if (sTemp.length() > 1)
          {
            length = sTemp.length();
            iX = iChar * length;
            if (iX > iWell)
              sTemp = new String(sTemp.substring(0, (iX/iChar)));

            iX = iWell + iCtr - iChar * sTemp.length() / 2;
//            iX = DEPTH_TRACK + i * iWell + iCtr / 2 - iChar * sTemp.length() / 2;
            iY = LABELSTART - 12 - 4 * iText;
            g.drawString(sTemp, iX, iY);
          }

          sTemp = new String("Lat: "+stXList.stItem[i].stHeader.dLatitude);

          length = sTemp.length();
          iX = iChar * length;
          if (iX > iWell)
            sTemp = new String(sTemp.substring(0, (iX/iChar)));

          iX = iWell + iCtr - iChar * sTemp.length() / 2;
//          iX = DEPTH_TRACK + i * iWell + iCtr / 2 - iChar * sTemp.length() / 2;
          iY = LABELSTART - 12 - 3 * iText;
          g.drawString(sTemp, iX, iY);

          sTemp = new String("Long: "+stXList.stItem[i].stHeader.dLongitude);

          length = sTemp.length();
          iX = iChar * length;
          if (iX > iWell)
            sTemp = new String(sTemp.substring(0, (iX/iChar)));

          iX = iWell + iCtr - iChar * sTemp.length() / 2;
//          iX = DEPTH_TRACK + i * iWell + iCtr / 2 - iChar * sTemp.length() / 2;
          iY = LABELSTART - 12 - 2 * iText;
          g.drawString(sTemp, iX, iY);

          if (stXList.stItem[i].stHeader.dGL > 0.0)
          {
            dElev = stXList.stItem[i].stHeader.dGL;
            iElev = 0;
          }
          else if (stXList.stItem[i].stHeader.dKB > 0.0)
          {
            dElev = stXList.stItem[i].stHeader.dKB;
            iElev = 1;
          }
          else
          {
            dElev = stXList.stItem[i].stHeader.dDF;
            iElev = 2;
          }

          switch (iElev)
          {
            case 0:
              sTemp = new String("Elev: (GL) " + dElev);
              break;
            case 1:
              sTemp = new String("Elev: (KB) " + dElev);
              break;
            case 2:
              sTemp = new String("Elev: (DF) " + dElev);
              break;
          }

          length = sTemp.length();
          iX = iChar * length;
          if (iX > iWell)
            sTemp = new String(sTemp.substring(0, (iX/iChar)));

          iX = iWell + iCtr - iChar * sTemp.length() / 2;
//          iX = DEPTH_TRACK + i * iWell + iCtr / 2 - iChar * sTemp.length() / 2;
          iY = LABELSTART - 12 - iText;
          g.drawString(sTemp, iX, iY);
        }
      }
    }
  }

  /** Method drawDepth(Graphics g)
   * <p> This method will draw the depth scale
   * @param g - Graphics Parameter.
   */

  public void drawDepth(Graphics g)
  {
    int    i=0;
    int    j=0;
    int    k=0;
    int    iCount = 1;
    int    iDepth = 0;
    int    jXinc  = 0;
    int    iStartTrack = 0;
    int    iWidthTrack = DEPTH_TRACK;
    int    iX1 = 0;
    int    iX2 = 0;
    int    iY1 = 0;
    int    iY2 = 0;

    g.setColor(Color.black);

    Font fsb = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    if (stXList != null)
      iCount = stXList.iCount;

    for (k=0; k<iCount; k++)
    {
      if ((iStart != null) && (iWide != null))
      {
        iX1 = iStartTrack + iStart[k][0] - iSeparation;
        iX2 = iStartTrack + iStart[k][0] + iWide[k];
      }
      else
      {
        iX1 = iStartTrack;
        iX2 = iStartTrack + iWidth;
      }

      for (i=0; i<=iLogHeight; i += iIncrementY)
      {
   	    if (iDatum == iqstrat.iqstratTracksStruct._BY_LOG)
   	    {
          iDepth = (int) (depthStart +
                        (double) i * dIncrementDepth / (double) iIncrementY);
		}
		else
		{
          iDepth = (int) (depthStart -
                        (double) i * dIncrementDepth / (double) iIncrementY);
		}

//        iDepth = (int) (depthStart -
//                        (double) i * dIncrementDepth / (double) iIncrementY);

        iY1 = i + PLOT_TITLES;

        jXinc = iIncrementY / 5;
        for (j=0; j<5; j++)
        {
          iY2 = i + (j * jXinc) + PLOT_TITLES;

          if ((iY2 >= PLOT_TITLES) && (iY2 < iHeight))
          {
            g.setColor(Color.lightGray);
            g.drawLine(iX1, iY2, iX2, iY2);
          }
        }

        g.setColor(Color.black);
        g.drawLine(iX1, iY1, iX2, iY1);

        if (k==0)
          g.drawString("" + iDepth, iStartDepthLabel, iY1);
      }

      g.drawLine(iStartTrack, PLOT_TITLES, iStartTrack+iWidthTrack, PLOT_TITLES);
    }
  }

  /** Method drawHorizonLinks()
   *  <p> This method will draw the link between common Horizons of both wells.
   */

  public void drawHorizonLinks(Graphics g)
  {
    int    iStartTrack = iStartDepth;
    int    iWidthTrack = 2*DEPTH_TRACK;
    double dElev[]     = { 0,0, 0,0, 0,0, 0,0 };
//    double dElev1      = 0.0;
//    double dElev2      = 0.0;
    double depth       = 0.0;
    double dRatio      = 0.0;
    int    iDepth1     = 0;
    int    iDepth2     = 0;
    stratListStruct stS[] = null;
//    stratListStruct stS1 = null;
//    stratListStruct stS2 = null;
    int    iB1            = 0;
    int    iB2            = 0;

    if (stXList != null)
    {
	  if (stXList.iCount > 1)
	  {
//        dElev1 = xsection.xsectionDataUtility.getElevation(
//                       iDatum, sDatum, stXList.stItem[0] );

//        dElev2 = xsection.xsectionDataUtility.getElevation(
//                       iDatum, sDatum, stXList.stItem[1] );

//        stS1 = stXList.stItem[0].stStrat;
//        stS2 = stXList.stItem[1].stStrat;

        stS  = new stratListStruct[stXList.iCount];
        for (int q=0; q<stXList.iCount; q++)
        {
          dElev[q] = xsection.xsectionDataUtility.getElevation(
                         iDatum, sDatum, stXList.stItem[q] );

          stS[q] = stXList.stItem[q].stStrat;
		}

//        if ((stS1 != null) && (stS2 != null))
        for (int p=0; p<stXList.iCount-1; p++)
        {
          iB1 = iStart[p][0]   + iWide[p];
          iB2 = iStart[p+1][0];

          for (int j=0; j<stS[p].iCount; j++)
          {
    	    if (iDatum == iqstrat.iqstratTracksStruct._BY_LOG)
              depth = stS[p].stItem[j].depthStart;
            else
              depth = dElev[p] - stS[p].stItem[j].depthStart;

            dRatio = (double) iLogHeight * (depthStart - depth) /
                                           (depthStart - depthEnd);
            iDepth1 = PLOT_TITLES + (int) dRatio;

			for (int k=0; k<stS[p+1].iCount; k++)
			{
//			  if (stS[p+1].stItem[k].sKEY.equals(stS[p].stItem[j].sKEY))
			  if (stS[p+1].stItem[k].sName.equals(stS[p].stItem[j].sName))
			  {
    	        if (iDatum == iqstrat.iqstratTracksStruct._BY_LOG)
                  depth = stS[p+1].stItem[k].depthStart;
                else
                  depth = dElev[p+1] - stS[p+1].stItem[k].depthStart;

                dRatio = (double) iLogHeight * (depthStart - depth) /
                                               (depthStart - depthEnd);
                iDepth2 = PLOT_TITLES + (int) dRatio;

                if ((iDepth1 >= PLOT_TITLES) && (iDepth1 < iHeight) &&
                    (iDepth2 >= PLOT_TITLES) && (iDepth2 < iHeight))
                {
                  g.setColor(Color.black);
                  g.drawLine(iB1, iDepth1-1, iB2, iDepth2-1);
                  g.setColor(Color.yellow);
                  g.drawLine(iB1, iDepth1,   iB2, iDepth2);
                  g.setColor(Color.black);
                  g.drawLine(iB1, iDepth1+1, iB2, iDepth2+1);
/*
                  g.setColor(Color.black);
                  g.drawLine(iStartTrack, iDepth1-1, iStartTrack+iWidthTrack, iDepth2-1);
                  g.setColor(Color.yellow);
                  g.drawLine(iStartTrack, iDepth1, iStartTrack+iWidthTrack, iDepth2);
                  g.setColor(Color.black);
                  g.drawLine(iStartTrack, iDepth1+1, iStartTrack+iWidthTrack, iDepth2+1);
*/
      		    }
			  }
			}
		  }
	    }
	  }
	}
  }

  /** Method drawHorizons()
   *  <p> This method will draw the Horizons on top of the Well Plot Tracks
   */

  public void drawHorizons(Graphics g)
  {
	int i,j;
    int             iStrt  = 0;
    int             iTemp  = 0;
    int             iEnd   = 0;
    int             len    = 0;
    int             len2   = 0;
    stratListStruct stS    = null;
    double          dElev  = 0.0;
    double          dRatio = 0.0;
    double          depth1 = 0.0;
    int             iDepth = 0;
    int             iCont  = 0;

    if (stXList != null)
    {
      for (i=0; i<stXList.iCount; i++)
      {
        if (stXList.stItem[i] != null)
        {
          dElev = xsection.xsectionDataUtility.getElevation(
                       iDatum, sDatum, stXList.stItem[i] );

          if (stXList.stItem[i].stControl != null)
          {
			if (stXList.stItem[i].stControl.iTracks != null)
			{
			  iStrt = 0;
			  iEnd  = 0;
			  len   = stXList.stItem[i].stControl.iTracks.length;

			  if (len > 0)
			  {
				iCont = 1;
//				iCont = 0;
			    if (stXList.stItem[i].stControl.iTracks[len-1] ==
			         iqstrat.iqstratTracksStruct._STRAT_UNITS_A)
			    {
				  iCont = 1;
			      len2  = len-2;
				}
				else
				  len2  = len-1;

			    iStrt = iStart[i][0];
			    iTemp = iStart[i][len2];
                iEnd  = iTemp + getTrackWidth( i,
                                  stXList.stItem[i].stControl.iTracks[len2] );
			  }
			}
		  }

//          iStrt = iStart[i][0];
//          iEnd  = iStart[i][0] + iWide[i];

          if (iCont == 1)
          {
		    stS = stXList.stItem[i].stStrat;

            if (stS != null)
            {
              for (j=0; j<stS.iCount; j++)
              {
    	        if (iDatum == iqstrat.iqstratTracksStruct._BY_LOG)
                  depth1 = stS.stItem[j].depthStart;
                else
                  depth1 = dElev - stS.stItem[j].depthStart;

                dRatio = (double) iLogHeight * (depthStart - depth1) /
                                               (depthStart - depthEnd);
                iDepth = PLOT_TITLES + (int) dRatio;

                if ((iDepth >= PLOT_TITLES) && (iDepth < iHeight))
                {
                  g.setColor(Color.black);
                  g.drawLine(iStrt, iDepth-1, iEnd, iDepth-1);
                  g.setColor(Color.yellow);
                  g.drawLine(iStrt, iDepth, iEnd, iDepth);
                  g.setColor(Color.black);
                  g.drawLine(iStrt, iDepth+1, iEnd, iDepth+1);
      		    }
			  }
			}
		  }
		}
	  }
	}
  }

  /** Method draw(graphics g)
   * <p> This method will draw the stratigraphic plot
   * @param g - Graphics Parameter.
   */

  public void draw(Graphics g)
  {
    int iW      = 0;
    int iB      = 0;
    int iSelect = -1;
    int iLength = 0;
    int iLAS    = 0;
    int iROCK   = 0;
    int iFOSSIL = 0;
    int iSED    = 0;
    int iPHI    = 0;
    int iPage   = 0;

    g.setColor(Color.white);
    g.fillRect(0, 0, iWidth, iHeight);

//    drawTitles(g);
    drawTrackLabels(g);
    drawDepth(g);
    drawWellLabel(g);

    if (stCMN != null)
      g.drawImage( stCMN.pIcon2, 0, 0, 58, 58, this );

    if (stXList != null)
    {
      for (int i=0; i<stXList.iCount; i++)
      {
        if (stXList.stItem[i] != null)
        {
          if (stXList.stItem[i].stControl != null)
          {
            if (stXList.stItem[i].stControl.iTracks != null)
            {
              for (int j=0; j<stXList.stItem[i].stControl.iTracks.length; j++)
              {
                iB = iStart[i][j];
                iW = getTrackWidth( i, stXList.stItem[i].stControl.iTracks[j] );
                iSelect = stXList.stItem[i].stControl.iTracks[j];

                // Set the Primary LAS Tracks to Double Size if Selected

                if ( iLASSize[i] == xsection.gui.xsectionAvailablePanel._DOUBLE )
                {
                  if ((iSelect == iqstrat.iqstratTracksStruct._LAS_INDUCTION) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_MICRO_RESISTIVITY) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_RESISTIVITY) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_LATERLOG) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_AHT) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_SONIC) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_LITHO_DENSITY) ||
                      (iSelect == iqstrat.iqstratTracksStruct._LAS_SPECTRAL_GR))
                  {
                    iW = 2 * iW;
                  }
				}

                if ((iSelect == iqstrat.iqstratTracksStruct._ROCK_COLUMN) ||
                    (iSelect == iqstrat.iqstratTracksStruct._ROCK_COLUMN_PD ))
                {
                  iLAS  = iSelect;
                }

             	if (iSelect == iqstrat.iqstratTracksStruct._ROCK_MEASURED_SECTION)
             	{
		   	      iROCK = iSelect;
			    }

                if (iSelect == iqstrat.iqstratTracksStruct._ROCK_FOSSILS)
		        {
		          iFOSSIL = iSelect;
		        }

                if (iSelect == iqstrat.iqstratTracksStruct._SED_STRUCTURE)
		        {
		          iSED = iSelect;
		        }

                if (iSelect == iqstrat.iqstratTracksStruct._ROCK_POROSITY)
		        {
		          iPHI = iSelect;
		        }

                pLAS.draw( g, iSelect, iB, iW, i );
                pThinPhi.draw( g, iSelect, iB, iW, i );

                pColorlith.draw( g, iSelect, iB, iW, i );
                pLithology.draw( g, iSelect, iB, iW, i );

                plotLASRock.draw( g, iSelect, iB, iW, i );

                pColumn.draw( g, iSelect, iB, iW, i );

                pPhi.draw( g, iSelect, iB, iW, i );
                pFossils.draw( g, iSelect, iB, iW, i );
                pSedimentary.draw( g, iSelect, iB, iW, i );
                pRockColor.draw( g, iSelect, iB, iW, i );

                pStrat.draw( g, iSelect, iB, iW, i );
              }

              iB = iStart[i][0] + iWide[i];
              iW = iWide[i];
            }
          }
        }
      }
    }

    pLegend.draw(g, iLAS , 0, iHeight, iWidth);
    iLength = pRockLegend.draw(g, iROCK, iWidth, jLegend);

    if (imgFOSSIL != null) { iPage = imgFOSSIL.getHeight(null); }
    if ((iFOSSIL == iqstrat.iqstratTracksStruct._ROCK_FOSSILS) && (iLength + iPage < iHeight))
    {
      g.drawImage(imgFOSSIL, iWidth+5, iLength+40, null);
      if (imgFOSSIL != null) { iLength = iLength + imgFOSSIL.getHeight(null); }
    }

    if (imgSED != null) { iPage = imgSED.getHeight(null); }
    if ((iSED == iqstrat.iqstratTracksStruct._SED_STRUCTURE) && (iLength + iPage < iHeight))
	{
      g.drawImage(imgSED, iWidth+5, iLength+20, null);
      if (imgSED != null) { iLength = iLength + imgSED.getHeight(null); }
	}

    if (imgPHI != null) { iPage = imgPHI.getHeight(null); }
    if ((iPHI == iqstrat.iqstratTracksStruct._ROCK_POROSITY) && (iLength + iPage < iHeight))
	{
      g.drawImage(imgPHI, iWidth+5, iLength+20, null);
	}

    drawHorizons(g);
    drawHorizonLinks(g);

    if (notifier != null)
      notifier.notifyObservers(new String("Display Memory Usage"));
  }

  /** Method paint(Graphics g)
   * <p> This method will paint stratigraphic Plot
   * @param g - Graphics Parameter.
   */

  public void paint(Graphics g) { draw( g ); }
}