/*
 * @xsectionDataStruct.java Version 1.0 02/20/2008
 *
 * Copyright (c) 2008 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package xsection;

import iqstrat.iqstratHeadersStruct;           // Header Information Structure
import iqstrat.iqstratControlStruct;           // Plot Control Structure
import iqstrat.iqstratRemarkListStruct;        // Georeport List Structure

import cmn.cmnLASFileListStruct;               // LAS File Information List

import las.lasFileDataStruct;                  // Log File Data Structure

import horizon.strat.stratListStruct;          // Tops List Structure

import lith.lithology.lithologyListStruct;     // LAS Lithology List Structure
import lith.rock.rockColumnListStruct;         // Rock Column List Structure

import rock.rockDataListStruct;                // Rock Data List Data Structure

import rock.color.rockColorListStruct;         // Rock Color List Structure
import rock.fossil.fossilListStruct;           // General Fossil List Structure
import rock.sedimentary.sedimentaryListStruct; // Sedimentary Structure list
import rock.phi.phiListStruct;                 // Porosity Type List Structure

/** CLASS xsectionDataStruct
 *  <p> This Structure holds the Information about data content for a single Well.
 *
 *  @version 1.1 02/20/2008
 *  @author  John R. Victorine
 */

public class xsectionDataStruct
{
  public static final int _IGNORE = 0;
  public static final int _PLOT   = 1;

  public int iPlot = _PLOT;

  // -- Depth Range of Data

  public double depthStart = 0.0;  // Start Depth
  public double depthEnd   = 0.0;  // End Depth

  // -- Porosity Thin Track

  public int    iThin    = cmn.cmnStruct._NO;
  public int    iTHINPHI = iqstrat.iqstratTracksStruct._THIN_NPHI;

  // Control Data

  public iqstratControlStruct    stControl      = null; // Control Data

  // Well / Outcrop Information Data

  public iqstratHeadersStruct    stHeader       = null; // Header Information

  // LAS Data

  public cmnLASFileListStruct    stFiles        = null; // LAS File Information data
  public lasFileDataStruct       stLAS          = null; // LAS File Data

  public lithologyListStruct     stLASLithology = null; // LAS Rock Column

  public int                     iColorlith     = 0;    // Colorlith setting

  // Horizons Data

  public stratListStruct         stStrat        = null; // Stratigraphic Units

  // Rock Data - Measured Sections

  public rockDataListStruct      stCore         = null; // Core Data

  public rockColumnListStruct    stColumn       = null; // Rock Column

  public phiListStruct           stPHI          = null; // Porosity Type Data
  public fossilListStruct        stFossil       = null; // Rock Fossils
  public sedimentaryListStruct   stSedimentary  = null; // Rock Sedimentary
  public rockColorListStruct     stColor        = null; // Rock Color
  public iqstratRemarkListStruct stRemarks      = null; // Remarks

  /** Method delete()
   * <p> This method will set the string variable to null and initialize all the
   *     other variables.
   */

  public void delete()
  {
    if (stControl != null)
      stControl.delete();
    stControl = null;

    if (stHeader != null)
      stHeader.delete();
    stHeader = null;

    // LAS Data

    if (stFiles != null)
      stFiles.delete();
    stFiles = null;

    if (stLAS != null)
      stLAS.delete();
    stLAS = null;

    if (stLASLithology != null)
      stLASLithology.delete();
    stLASLithology  = null; // LAS Rock Column

    // Horizon Data

    if (stStrat != null)
      stStrat.delete();
    stStrat = null;

    // Rock Data

    if (stCore != null)
      stCore.delete();
    stCore = null;

    if (stColumn != null)
      stColumn.delete();
    stColumn = null;

    if (stPHI != null)
      stPHI.delete();
    stPHI = null;

    if (stFossil != null)
      stFossil.delete();
    stFossil = null;

    if (stSedimentary != null)
      stSedimentary.delete();
    stSedimentary = null;

    if (stColor != null)
      stColor.delete();
    stColor = null;

    if (stRemarks != null)
      stRemarks.delete();
    stRemarks = null;
  }
}