/*
 * @xsectionDataUtility.java Version 1.1 02/20/2008
 *
 * Copyright (c) 2008 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package xsection;

import xsection.xsectionDataListStruct;        // Cross Section List Structure
import xsection.xsectionDataStruct;            // Cross Section Data Structure

import iqstrat.iqstratHeadersStruct;           // Header Information Structure
import iqstrat.iqstratControlStruct;           // Plot Control Structure
import iqstrat.iqstratRemarkListStruct;        // Georeport List Structure

import las.lasFileDataStruct;                  // Log File Data Structure

import horizon.strat.stratListStruct;          // Tops List Structure
import horizon.strat.stratStruct;              // Tops Data Structure

import lith.lithology.lithologyListStruct;     // LAS Lithology List Structure
import lith.rock.rockColumnListStruct;         // Rock Column List Structure

import rock.rockDataListStruct;                // Rock Data List Data Structure

import rock.color.rockColorListStruct;         // Rock Color List Structure
import rock.fossil.fossilListStruct;           // General Fossil List Structure
import rock.sedimentary.sedimentaryListStruct; // Sedimentary Structure list
import rock.phi.phiListStruct;                 // Porosity Type List Structure

/** Class xsectionDataUtility
 *  <p> This Class will provide basic utility for the data structures
 *
 *  @version 1.1 02/20/2008
 *  @author  John R. Victorine
 */

public class xsectionDataUtility
{
  /** Method exists( xsectionDataStruct st,
   *                 String sAPI,
   *                 String sKID,
   *                 String sKEY,
   *                 String sName )
   * <p> This method will check to see if the record exists in the list
   * @param  st    = Information XML Data List Structure
   * @param  sAPI  = The API-Number of the Location to be removed.
   *                 To remove Location from Table List only set sAPI = null.
   * @param  sKID  = Well Header KID or KGS Primary Key
   * @param  sKEY  = Primary Key of data defined by user
   * @param  sName = The name of the well or location
   * @return sKEY  = The user created primary key
   */

  public static String exists( xsectionDataStruct st,
                               String sAPI,
                               String sKID,
                               String sKEY,
                               String sName )
  {
    int iFound = -1;

    if (st != null)
    {
      if ((st.stHeader.sKID.equals(sKID)) && (sKID.length() > 1)) iFound = 1;
      if ((st.stHeader.sKEY.equals(sKEY)) && (sKEY.length() > 1)) iFound = 1;
      if ((st.stHeader.sAPI.equals(sAPI)) && (sAPI.length() > 1)) iFound = 1;

      if ((st.stHeader.sName.toUpperCase().equals(sName.toUpperCase())) &&
          (sName.length() > 1))
        iFound = 1;
    }

    sKEY = new String("0");
    if (iFound > -1)
      sKEY = new String(st.stHeader.sKEY);

    return (sKEY);
  }

  /** Method exists_kgs( xsectionDataStruct st, String sKID )
   * <p> This method will check to see if the record exists in the list
   * @param  st    = Information XML Data List Structure
   * @param  sKID  = Well Header KID
   * @return sKID  = Well Header KID
   */

  public static String exists_kgs( xsectionDataStruct st, String sKID )
  {
    int    iFound = -1;
    String sKEY   = "0";

    if (st != null)
    {
      if ((st.stHeader.sWELLKID.equals(sKID)) && (sKID.length() > 1))
        iFound = 1;
    }

    if (iFound > -1)
      sKEY = new String(st.stHeader.sWELLKID);

    return (sKEY);
  }

  /** Method copyList()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static xsectionDataListStruct copyList( xsectionDataListStruct stOld )
  {
    xsectionDataListStruct stNew = null;

    if (stOld != null)
    {
      stNew            = new xsectionDataListStruct();
      stNew.iCount     = stOld.iCount;
      stNew.stItem     = new xsectionDataStruct[stOld.iCount];

      for (int i=0; i<stOld.iCount; i++)
      {
        stNew.stItem[i] = copy( stOld.stItem[i] );
      }
    }

    return (stNew);
  }

  /** Method copy()
   * <p> This method will copy one structure to another
   * @param  stOld = Cross Section Data Structure to be copied
   * @return stNew = New Cross Section Data Structure
   */

  public static xsectionDataStruct copy( xsectionDataStruct stOld )
  {
    int i,j;
    xsectionDataStruct stNew = null;

    if (stOld != null)
    {
      stNew            = new xsectionDataStruct();

      stNew.iPlot      = stOld.iPlot;

      stNew.depthStart = stOld.depthStart;
      stNew.depthEnd   = stOld.depthEnd;

      // -- Porosity Thin Track

      stNew.iThin      = stOld.iThin;
      stNew.iTHINPHI   = stOld.iTHINPHI;
      stNew.iColorlith = stOld.iColorlith;    // Colorlith setting

      // Control Data

      stNew.stControl  = iqstrat.iqstratControlUtility.copy( stOld.stControl );

      // Well / Outcrop Information Data

      stNew.stHeader   = iqstrat.iqstratHeadersUtility.copy( stOld.stHeader );

      // LAS File Data

      stNew.stLAS      = las.lasFileDataUtility.copy( stOld.stLAS );

      // Lithology Data by LAS File Data

      stNew.stLASLithology =
          lith.lithology.lithologyUtility.copyList( stOld.stLASLithology );

      // Horizons Data

      stNew.stStrat    = horizon.strat.stratUtility.copyList( stOld.stStrat );

      // Rock Data

      stNew.stCore     = rock.rockDataUtility.copyList( stOld.stCore );

      stNew.stColumn   = lith.rock.rockColumnUtility.copyList( stOld.stColumn );

      stNew.stFossil   = rock.fossil.fossilUtility.copyList( stOld.stFossil );
      stNew.stSedimentary =
          rock.sedimentary.sedimentaryUtility.copyList( stOld.stSedimentary );
      stNew.stPHI      = rock.phi.phiUtility.copyList( stOld.stPHI );
      stNew.stColor    = rock.color.rockColorUtility.copyList( stOld.stColor );

      stNew.stRemarks  = iqstrat.iqstratRemarkUtility.copyList( stOld.stRemarks );
    }

    return (stNew);
  }

  /** Method getTops( xsectionDataListStruct st )
   * <p> This method will build the common stratigraphic units list.
   * @param  st     = cross section data structure
   * @return stList = common stratigraphic units list.
   */

  public static stratListStruct getTops( xsectionDataListStruct st )
  {
    int i      = 0;
    int j      = 0;
    int iCount = 0;
    stratListStruct stList = null;
    stratStruct     stTop  = null;

    if (st != null)
    {
      if (st.stItem[st.iCount-1].stStrat != null)
      {
        for (i=0; i<st.stItem[st.iCount-1].stStrat.iCount; i++)
        {
          stTop = st.stItem[st.iCount-1].stStrat.stItem[i];
          if (stTop.depthStart > 0.0)
          {
            stTop.sID = new String("" + iCount);
            stList    = horizon.strat.stratUtility.add( stTop, stList );
            iCount++;
          }
        }
      }

      for (i=0; i<st.iCount-1; i++)
      {
        if (st.stItem[i].stStrat != null)
        {
          for (j=0; j<st.stItem[i].stStrat.iCount; j++)
          {
            stTop = st.stItem[i].stStrat.stItem[j];
            if ((stTop.depthStart > 0.0) &&
                (!horizon.strat.stratUtility.isUnit( stTop, stList )))
            {
              stTop.sID = new String("" + iCount);
              stList    = horizon.strat.stratUtility.add( stTop, stList );
              iCount++;
            }
          }
        }
      }
    }

    return (stList);
  }

  /** Method getTops( xsectionDataListStruct st )
   * <p> This method will build the common stratigraphic units list.
   * @param  sTops  = Horizon Name
   * @param  st     = cross section data structure
   * @return bValue = true Horizon Name in all Wells, false if not.
   */

  public static boolean isCommon( String sTops, xsectionDataListStruct st )
  {
	boolean bValue = false;
	int     iCount = 0;

	if ((st != null) && (sTops != null))
	{
      for (int i=0; i<st.iCount; i++)
      {
        if (st.stItem[i].stStrat != null)
        {
          for (int j=0; j<st.stItem[i].stStrat.iCount; j++)
          {
			if (sTops.equals(st.stItem[i].stStrat.stItem[j].sName))
			{
			  iCount++;
			}
		  }
		}
	  }

	  if (iCount == st.iCount)
	    bValue = true;
	}

	return (bValue);
  }

  /** Method getCommon( xsectionDataListStruct st, stratListStruct stList )
   * <p> This method will identify the common stratigraphic units between
   *     the wells in the list.  Common stratigraphic units will automatically
   *     be selected.
   * @param  st     = cross section data structure
   * @param  stList = common stratigraphic units list.
   * @return stList = common stratigraphic units list.
   */

  public static stratListStruct getCommon(
      xsectionDataListStruct st, stratListStruct stList )
  {
    int i,j,k;
    stratListStruct stTops     = null;
    int             iFound     = 0;
    int             iTotal     = 0;
    String          sArray[][] = null;

    if ((st != null) && (stList != null))
    {
      sArray = getAvailable( st, stList );
      iTotal = st.iCount + 2;

      if (sArray != null)
      {
        for (i=0; i<sArray.length; i++)
        {
          iFound = 1;
          for (j=2; j<iTotal; j++)
          {
            if (sArray[i][j].equals("NO"))
              iFound = 0;
          }

          if (iFound == 1)
          {
            for (k=0; k<stList.iCount; k++)
            {
              if (sArray[i][1].equals(stList.stItem[k].sName))
                stList.stItem[k].iStatus = horizon.strat.stratStruct.ON;
            }
          }
        }
      }
    }

    return (stList);
  }

  /** Method getAvailable( xsectionDataListStruct st, stratListStruct stList )
   * <P> This method will build the available tops array to assist the user in
   *     picking horizons that are common for each well in list
   * @param  st     = cross section data structure
   * @param  stList = common stratigraphic units list.
   * @return sArray = Array of available tops
   */

  public static String[][] getAvailable( xsectionDataListStruct st,
                                         stratListStruct        stList )
  {
    int    i          = 0;
    int    j          = 0;
    int    k          = 0;
    String sArray[][] = null;
    stratListStruct stTops = null;

    if ((st != null) && (stList != null))
    {
      sArray = new String[stList.iCount][st.iCount+2];

      for (i=0; i<stList.iCount; i++)
      {
        sArray[i][0] = new String(stList.stItem[i].sID);
        sArray[i][1] = new String(stList.stItem[i].sName);

        for (j=0; j<st.iCount; j++)
        {
          k = j+2;
          sArray[i][k] = "NO";
        }
      }

      for (i=0; i<stList.iCount; i++)
      {
        for (j=0; j<st.iCount; j++)
        {
          if (st.stItem[j] != null)
          {
            if (st.stItem[j].stStrat != null)
            {
              stTops = st.stItem[j].stStrat;

              for (k=0; k<stTops.iCount; k++)
              {
                 if (stTops.stItem[k].sName.equals(stList.stItem[i].sName))
                   sArray[i][j+2] = "YES";
              }
            }
          }
        }
      }
    }

    return (sArray);
  }

  /** Method getElevation( xsectionDataStruct st )
   * <p> This method will retrieve the elevation for a specific well/outcrop
   * @param  st    = The cross section data structure
   * @return dElev = The Elevation Depth
   */

  public static double getElevation( xsectionDataStruct st )
  {
    double dElev =0.0;

    if (st != null)
    {
      if (st.stHeader != null)
      {
        if (st.stHeader.dGL > 0.0)
          dElev = st.stHeader.dGL;
        else if (st.stHeader.dKB > 0.0)
          dElev = st.stHeader.dKB;
        else
          dElev = st.stHeader.dDF;
      }
    }

    return (dElev);
  }

  /** Method getElevation( int iDatum, String sDatum, xsectionDataStruct st )
   * <p> This method will compute the Elevation depending on the Datum ID
   * @param  iDatum = datum by identifier
   * @param  sDatum = horizon name
   * @param  st     = Cross Section Data Structure
   * @return dElev  = Elevation of the Location
   */

  public static double getElevation(
      int iDatum, String sDatum, xsectionDataStruct st )
  {
    double dElev = 0.0;

    if (st != null)
    {
      switch (iDatum)
      {
        case iqstrat.iqstratTracksStruct._BY_ELEV:
          dElev = iqstrat.iqstratHeadersUtility.getElevation( st.stHeader );
          break;

        case iqstrat.iqstratTracksStruct._BY_DATUM:
          dElev = horizon.strat.stratUtility.getElevation( st.stStrat, sDatum );
          break;

        case iqstrat.iqstratTracksStruct._BY_LOG:
          break;
      }
    }

    return (dElev);
  }
}